<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api\Expt_Callbacks;

use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_SettingsApi;

class Expt_ApiCallbacks extends Expt_BaseController {

    // Base URLS for three methods
    protected $base_url_SendSMS = 'https://www.experttexting.com/ExptRestApi/sms/json/Message/Send';
    protected $base_url_QueryBalance = 'https://www.experttexting.com/ExptRestApi/sms/json/Account/Balance?';
    protected $base_url_Status = 'https://www.experttexting.com/ExptRestApi/sms/json/Message/Status?';

    public function register() {
        // NULL
    }
    
    // SEND SMS FUNCTION
    function send() {

        $body = array(
          'username' => esc_attr($this->username),
          'api_key' => esc_attr($this->apikey),
          'api_secret' => esc_attr($this->apisecret),
          'FROM' => esc_attr($this->from),
          'to' => esc_attr($this->to),
          'text' => esc_attr($this->msgtext),
          'type' => esc_attr($this->type),
          't' => '1',
        );
        
        $body = wp_json_encode( $body );

        $response = wp_remote_post( 
          $this->base_url_SendSMS,
          array(
            'method' => 'POST',
            'body' => $body,
            'sslverify' => false,
            'headers' => [
              'Content-Type' => 'application/json',
            ],
          )
        );

        $responseBody = wp_remote_retrieve_body( $response );

        return $responseBody;

    }
    
    // FUNCTION TO QUERY ACCOUNT BALANCE
    function QueryBalance() {
        
        $fieldstring = "username=$this->username&api_key=$this->apikey&api_secret=$this->apisecret&t=1";
        
        $response = wp_remote_get( 
            $this->base_url_QueryBalance . $fieldstring,
            array(
              'method' => 'GET',
              'timeout' => 30,
              'httpversion' => '1.1',
              'redirection' => 10,
              'sslverify' => false,
            )
        );

        if ( is_wp_error($response) ) {

            return $response;
            
        } else {

            return wp_remote_retrieve_body( $response );
        }

    }

    // SMS STATUS FUNCTION
    function msgStatus() {
        
        $fieldstring = "username=$this->username&api_key=$this->apikey&api_secret=$this->apisecret&message_id=$this->messageId&t=1";

        $response = wp_remote_get( 
            $this->base_url_Status . $fieldstring,
            array(
              'method' => 'GET',
              'timeout' => 30,
              'httpversion' => '1.1',
              'redirection' => 10,
              'sslverify' => false,
            )
        );

        if ( is_wp_error($response) ) {

            return $response;
            
        } else {

            return wp_remote_retrieve_body( $response );
        }
          
    }
  

    public function sendLog($sender, $message, $recipient, $message_type, $response, $status, $success, $failed, $message_count, $message_id_array, $resendReq, $resendReqChk = false) {
        global $wpdb, $__Expt_BaseController;

        if ( $resendReqChk == false ) {

            $result = $wpdb->insert(
                $wpdb->prefix . "et_send",
                array(
                  'date' => date('Y-m-d H:i:s', current_time('timestamp')),
                  'sender' => esc_attr($sender),
                  'message' => esc_html($message),
                  'recipient' => esc_html($recipient),
                  'message_type' => esc_attr($message_type),
                  'response' => esc_attr($response),
                  'status' => esc_attr($status),
                  'success' => esc_attr($success),
                  'failed' => esc_attr($failed),
                  'message_count' => esc_attr($message_count),
                  'messages_id' => serialize($__Expt_BaseController->expt_escape_array_data($message_id_array)),
                )
            );
        } 
        elseif( $resendReqChk != false && !empty( $resendReq ) ) {

            $result = $wpdb->update(
                $wpdb->prefix . "et_send",
                array(
                  'date' => date('Y-m-d H:i:s', current_time('timestamp')),
                  'sender' => esc_attr($sender),
                  'message' => esc_html($message),
                  'recipient' => esc_html($recipient),
                  'message_type' => esc_attr($message_type),
                  'response' => esc_attr($response),
                  'status' => esc_attr($status),
                  'success' => esc_attr($success),
                  'failed' => esc_attr($failed),
                  'message_count' => esc_attr($message_count),
                  'messages_id' => serialize($__Expt_BaseController->expt_escape_array_data($message_id_array)),
                ),
                array(
                  'ID' => esc_attr( $resendReq )
                )
            );

        }

        if ($result) {
            /**
             * Run hook after adding Send Log.
             *
             * @param string $recipient.
             * @param string $message.
             *
             */

            $lastid = $wpdb->insert_id;

            do_action('expttxt_send_message', $recipient, $message);

            return $lastid;
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }
    }

    public function etApiBalance() {
        
        if( !empty( get_option( 'etApiAuth' ) ) || empty( get_option( 'et_balance' ) ) ) {
            _e( '<h5>$ 0.0000</h5>' );
        } else {
            _e( '<h5>$'. esc_attr( get_option( 'et_balance' ) ) .'</h5>' );
        }
    }

    public function etApiSetStatus( $cb = '' ) {
        
        if( !empty( get_option( 'etApiAuth' ) ) || empty( get_option( 'et_balance' ) ) ) {
            _e( '<h5 class="text-danger my-2"><span class="dashicons dashicons-dismiss text-danger"></span> ' . __('Inactive', 'experttexting-official') . '</h5>' );
            _e( $cb !== 'dashboard' ? '<p class=" text-muted">' . esc_attr( get_option( 'etApiAuth' ) ) . '</p>' : '' );
        } elseif( empty( get_option( 'et_api_set' ) ) && empty( get_option( 'et_balance' ) ) ) {
            _e( '<h5 class="text-danger my-2"><span class="dashicons dashicons-dismiss text-danger"></span> ' . __('Inactive', 'experttexting-official') . '</h5>' );
            _e( $cb !== 'dashboard' ? '<p class=" text-muted">' . __('API configuration details not provided / missing.', 'experttexting-official') . '</p>' : '' );
        } else {
            _e( '<h5 class="text-success my-2"><span class="dashicons dashicons-yes-alt text-success"></span> ' . __('Active', 'experttexting-official') . '</h5>' );
            _e( $cb !== 'dashboard' ? '<p class=" text-muted">' . __('Successfully Authenticated.', 'experttexting-official') . '</p>' : '' );
        }

    }

    public function refreshBal( $arg = 'false' ) {
      
        update_option( "etApiAuthHidden", "authCheck", "yes" );

        if( $arg == 'true' ) {
            \ExptTxt\Expt_Api\Expt_SettingsApi::apiAuth();
            wp_redirect( admin_url('admin.php?page=et_menu') );
            return;
            
        } elseif( get_option( 'et_balance_datetime' ) ) {

            $chk = $this->ettimecheck( 'et_balance_datetime', 60 );
            if( !$chk ) {
                update_option( "etApiAuthHidden", "", "yes" );
                return;
            }
        }
        
        \ExptTxt\Expt_Api\Expt_SettingsApi::apiAuth();
    }

    public function sendMessage() {

        global $__Expt_Notices;
      
        $getResponse = \ExptTxt\Expt_Api\Expt_SettingsApi::sendMsg();
        $msg = '';

        if ( isset($getResponse) ) {
            
            if ( $getResponse === 'api_error_notice' ) {
                $__Expt_Notices->api_notice();
                $response = $getResponse;
                $msg = __( 'Message sending failed. Please make sure you have configured the API settings correctly.', 'experttexting-official' );
            }
            if ( $getResponse === 'empty_list' ) {
                $response = $getResponse;
                $msg = __( 'The selected user list is empty. Please select another valid user list from send to option.', 'experttexting-official' );
            }
            if ( $getResponse === 'invalid_numbers' ) {
                $response = $getResponse;
                $msg = __( 'Entered mobile number(s) are invalid. Please check and confirm if all the mobile number(s) are valid.', 'experttexting-official' );
            }
            if ( $getResponse === 'empty_number' ) {
                $response = $getResponse;
                $msg = __( 'Enter Recipient Number.', 'experttexting-official' );
            }
            if ( $getResponse === 'empty_message' ) {
                $response = $getResponse;
                $msg = __( 'Enter Message.', 'experttexting-official' );
            }
            if ( $getResponse === 'empty_sender' ) {
                $response = $getResponse;
                $msg = __( 'Enter Sender Number.', 'experttexting-official' );
            }
            if ( $getResponse === 'final_error' ) {
                $response = $getResponse;
                $msg = __( 'An unexpected error occurred, contact ExpertTexting.', 'experttexting-official' );
            }
            
            $__Expt_Notices->notice( $msg, "error" );

            return [ $response, $msg ];
        }
        
    }
    
    // API SETTINGS

    public static function getNumberOfSubscribers($group = "") {

        global $__Expt_BaseController;

        $group = $group == "all_sub" ? "" : absint($group);

        global $wpdb;
        if($group) {
            $result = $wpdb->get_results("SELECT mobile FROM `{$wpdb->prefix}et_subscribes` WHERE status = 1 AND group_ID = '" . $group . "'");
        } else {
            $result = $wpdb->get_results("SELECT mobile FROM `{$wpdb->prefix}et_subscribes` WHERE status = 1");
        }

        if ($result) {
            return $result;

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

    }

    public static function getNameOfGroups() {

        global $wpdb, $__Expt_BaseController;
        
        $result = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}et_subscribes_group`");

        if ($result) {

            return $result;

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

    }

    public static function getNumberOfWordpressUsers() {

        global $wpdb, $__Expt_BaseController;
        
        $result = $wpdb->get_results("SELECT meta_value FROM `{$wpdb->prefix}usermeta` WHERE meta_key = 'expt_mobile'");

        if ($result) {
            
                foreach($result as $res)  {
                    !empty($res->meta_value) ? $resultNew[] = $res : "";
                }

                return $resultNew;

        } else {

                if( $wpdb->last_error ) {
                    $__Expt_BaseController->save_expt_wpdb_log();
                }
        }
          
    }

    public static function getMessageSendCount( $args = '' ) {

        global $wpdb, $__Expt_BaseController;
        
        $result = $wpdb->get_results("SELECT success, failed FROM `{$wpdb->prefix}et_send`");
        $output = 0;

        if ($result) {
          
            foreach($result as $res)  {

                if ( !$args ) {
                  
                    $output = (int)$output + (int)$res->failed + (int)$res->success;

                } else if ( $args == 'success' ) {

                    $output = (int)$output + (int)$res->success;

                } else if ( $args == 'failed' ) {

                    $output = (int)$output + (int)$res->failed;
                }

            }

            return $output;
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }
          
    }

    public static function getLastSendMessages( $args = 5 ) {

        global $wpdb, $__Expt_BaseController;
        
        $result = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}et_send` ORDER BY ID DESC LIMIT 5");
        
        if ($result) {
            return $result;
        
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

    }
    
}
  
  