<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api\Expt_Callbacks;

use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_NotificationsCallbacks;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_ApiCallbacks;

class Expt_IntegrationCallbacks extends Expt_BaseController {

    public $cf7_data;

    public function register() {
        
        // Contact Form 7
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('Contact Form 7') === 'installed' ) {

            if( $this->activated( 'et_integration_cf7', 'contact_form_7_chk' ) ) {
                add_filter('wpcf7_editor_panels', array($this, 'et_cf7_editor_panels'));
                add_action('wpcf7_after_save', array($this, 'et_cf7_save_form'));
                add_action('wpcf7_mail_sent', array($this, 'et_cf7_sms_handler'));
            }

        }

        // Woocommerce
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('WooCommerce') === 'installed' ) {

            if( $this->activated( 'et_integration_woocommerce', 'woocommerce_chk' ) ) {
                add_action('woocommerce_new_order', array($this, 'et_wc_new_order'));
            }

            if( $this->activated( 'et_integration_woocommerce', 'woocommerce_mobile_field_chk' ) ) {
                add_action('woocommerce_after_order_notes', array($this, 'et_wc_user_number_field'));
                add_action('woocommerce_checkout_process', array($this, 'et_wc_user_number_validate_field'));
                add_action('woocommerce_checkout_update_order_meta', array($this, 'et_wc_user_number_update_field'));
            }

            if ( $this->activated( 'et_integration_woocommerce', 'woocommerce_new_order_to_number_chk' ) ) {
                add_action('woocommerce_new_order', array($this, 'et_wc_notification_number'));
            }

            if ( $this->activated( 'et_integration_woocommerce', 'woocommerce_new_order_to_customer_chk' ) ) {
                add_action('woocommerce_new_order', array($this, 'et_wc_notification_customer'));
            }

            if ( $this->activated( 'et_integration_woocommerce', 'woocommerce_stock_to_admin_chk' ) ) {
                add_action('woocommerce_low_stock', array($this, 'et_wc_low_stock_admin'), 10, 1);
            }
            
            if ( $this->activated( 'et_integration_woocommerce', 'woocommerce_status_to_customer_chk' ) ) {
                add_action('woocommerce_order_status_changed', array($this, 'et_wc_notification_change_status'), 10, 3);
            }
            
            if ( $this->activated( 'et_integration_woocommerce', 'woocommerce_notify_by_status_to_customer' ) ) {
                add_action('woocommerce_order_status_changed', array($this, 'et_wc_notification_change_by_status'), 10, 3);
            }

        }
        
        // Easy Digital Downloads
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('Easy Digital Downloads') === 'installed' ) {

            if( $this->activated( 'et_integration_edd', 'easy_digital_downloads_chk' ) ) {
                add_action('edd_complete_purchase', array($this, 'et_edd_new_order'));
            }

            if( $this->activated( 'et_integration_edd', 'easy_digital_downloads_mobile_field_chk' ) ) {
                add_action('edd_purchase_form_user_info', array($this, 'et_edd_user_number_field'));
                add_action('edd_checkout_error_checks', array($this, 'et_edd_validate_number_field'), 10, 2);
                add_filter('edd_payment_meta', array($this, 'et_edd_store_number_field'));
                add_action('edd_payment_personal_details_list', array($this, 'et_edd_purchase_details'), 10, 2);
            }

            if( $this->activated( 'et_integration_edd', 'easy_digital_downloads_new_order_to_number_chk' ) ) {
                add_action('edd_complete_purchase', array($this, 'et_edd_notification_number'));
            }

            if( $this->activated( 'et_integration_edd', 'easy_digital_downloads_new_order_to_customer_chk' ) ) {
                add_action('edd_complete_purchase', array($this, 'et_edd_notification_customer'));

                if ( ! $this->activated( 'et_integration_edd', 'easy_digital_downloads_mobile_field_chk' ) ) {
                    $option = $this->expt_escape_array_data( get_option( 'et_integration_edd' ) );
                    $option['easy_digital_downloads_mobile_field_chk'] = 1;
                    update_option( 'et_integration_edd', $option, 'yes' );
                }
            }

        }

        // Gravity Forms
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('Gravity Forms') === 'installed' ) {
            
            add_action('gform_after_submission', array($this, 'et_gf_message_form'), 10, 2);
        }

        // QuForm
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('Quform') === 'installed' ) {
            
            add_filter('quform_post_process', array($this, 'et_qf_message_form'), 10, 2);
        }

        // BuddyPress
        if( \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed('BuddyPress') === 'installed' ) {
            
            if( $this->activated( 'et_integration_bp', 'buddypress_mobile_field_chk' ) ) {
                global $wpdb;
			    $result = $wpdb->query($wpdb->prepare( "SELECT * FROM {$wpdb->prefix}bp_xprofile_fields WHERE name = %s", 'Mobile Number' ));
                if (!$result) {
                    add_action('bp_init', array($this, 'et_bp_user_number_field'));
                }
            }

            if ( $this->activated( 'et_integration_bp', 'buddypress_mentions_chk' ) ) {
                add_action('bp_activity_sent_mention_email', array($this, 'et_bp_notification_mention'), 10, 5);
            }

            if ( $this->activated( 'et_integration_bp', 'buddypress_comment_post_chk' ) ) {
                add_action('bp_activity_comment_posted', array($this, 'et_bp_notification_comment_post'), 10, 3);
            }
            
        }

    }

    

    /**
     * wpcf7_editor_panels
     * 
     * @since 1.0.0
     * 
     * @param $panels
     * @return $panels
     */
    public function et_cf7_editor_panels( $panels ) {

        $new_page = array(
            'experttexting' => array(
                'title'    => __('ET SMS Notification', 'experttexting-official'),
                'callback' => array($this, 'et_cf7_setup_form')
            )
        );

        $panels = array_merge($panels, $new_page);

        return $panels;
    }

    /**
     * wpcf7_editor_panels Callback
     * 
     * @since 1.0.0
     * 
     * @param $form
     */
    public function et_cf7_setup_form( $form ) {

        $cf7_options       = $this->expt_escape_array_data( get_option('et_wpcf7_' . $form->id()) );
        $cf7_options_field = $this->expt_escape_array_data( get_option('et_wpcf7_form_' . $form->id()) );

        include_once $this->plugin_path . "ExptTemplates/et_cf7_form.php";
    }

    /**
     * wpcf7_after_save
     * 
     * @since 1.0.0
     * 
     * @param $form
     */
    public function et_cf7_save_form( $form ) {

        $et_wpcf7 = isset($_POST['et_wpcf7']) ? $this->expt_sanitize_array_data( $_POST['et_wpcf7'] ) : '';
        $et_wpcf7_form = isset($_POST['et_wpcf7_form']) ? $this->expt_sanitize_array_data( $_POST['et_wpcf7_form'] ) : '';

        update_option('et_wpcf7_' . $form->id(), $et_wpcf7);
        update_option('et_wpcf7_form_' . $form->id(), $et_wpcf7_form);
    }

    /**
     * wpcf7_mail_sent
     * 
     * @since 1.0.0
     * 
     * @param $form
     */
    public function et_cf7_sms_handler( $form ) {

        global $__Expt_NotificationsCallbacks;
        
        $cf7_options       = $this->expt_escape_array_data( get_option('et_wpcf7_' . $form->id()) );
        $cf7_options_field = $this->expt_escape_array_data( get_option('et_wpcf7_form_' . $form->id()) );
        $this->et_set_cf7_data();

        /**
         * Send SMS to the specific number
         */
        if(isset($cf7_options['message']) && isset($cf7_options['phone'])) {

            $phone = explode(',', preg_replace('/\s+/', '', $cf7_options['phone']));

            $message = preg_replace_callback('/\[([a-zA-Z0-9._-]+)\]/', function ( $values ) {

                foreach ( $values as $value ) {
                    if(isset($this->cf7_data[$value])) {
                        return $this->cf7_data[$value];
                    }
                }
                
            }, $cf7_options['message']);

            if( !$phone || !$message ) return;

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );

        }

        /**
         * Send SMS to an specific field
         */
        if(isset($cf7_options_field['message']) && isset($cf7_options_field['phone'])) {

            $phone = preg_replace_callback('/\[([a-zA-Z0-9._-]+)\]/', function ( $values ) {

                foreach ( $values as $value ) {
                    if(isset($this->cf7_data[$value])) {
                        return $this->cf7_data[$value];
                    }
                }
                
            }, $cf7_options_field['phone']);

            // Check if the type of the field is select.
            foreach ( $form->scan_form_tags() as $scan_form_tag ) {
                if( $scan_form_tag['basetype'] == 'select' ) {
                    foreach ( $scan_form_tag['raw_values'] as $raw_value ) {

                        $option = explode('|', $raw_value);
                        
                        if( isset($option[0]) and $option[0] == $phone ) {
                            $phone = $option[1];
                        }
                    }
                }
            }

            if( strpos($phone, ',') !== false ) {
                $phone = explode(',', preg_replace('/\s+/', '', $phone));

            } elseif( strpos($phone, '|') !== false ) {
                $phone = explode('|', preg_replace('/\s+/', '', $phone));
            }

            $phone = is_array( $phone ) ? $phone : array( $phone );

            $message = preg_replace_callback('/\[([a-zA-Z0-9._-]+)\]/', function ( $values ) {

                foreach ( $values as $value ) {
                    if(isset($this->cf7_data[$value])) {
                        return $this->cf7_data[$value];
                    }
                }
                
            }, $cf7_options_field['message']);

            if( !$phone || !$message ) return;

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );
            
        }
    }

    /**
     * Get data from CF7
     */
    private function et_set_cf7_data() {

        $postVar = $this->expt_sanitize_array_data( $_POST );
        
        foreach ( $postVar as $index => $key ) {
            if( is_array($key) ) {
                $this->cf7_data[$index] = implode(', ', $key);
            } else {
                $this->cf7_data[$index] = $key;
            }
        }
    }

    /**
     * woocommerce_new_order
     * 
     * @since 1.0.0
     * 
     * @param $order_id
     */
    public function et_wc_new_order( $order_id ) {

        global $__Expt_NotificationsCallbacks;

        $order = wc_get_order( $order_id );

        if( !$order ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $msg = !empty($msgOption) ? $msgOption['woocommerce_body'] : '';

        if( !$msg ) return;
        
        $variables = array(
            'order_id' => $order_id,
            'order_status' => $order->get_status(),
            'order_total_cost' => (string) $order->get_total(),
            'order_date' => $order->get_date_created()->date("Y-m-d H:i:s"),
            'order_view_url' => $order->get_checkout_order_received_url(),
            'order_payment_url' => $order->get_checkout_payment_url(),
            'customer_first_name' => $order->get_billing_first_name(),
            'customer_email' => $order->get_billing_email(),
            'customer_phone' => $order->get_billing_phone(),
            'customer_country' => $order->get_billing_country(),
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($this->adminNumber) );
        }

    }

    /**
     * woocommerce_after_order_notes
     * 
     * @since 1.1.0
     * 
     * @param $checkout
     */
    public function et_wc_user_number_field( $checkout ) {

        woocommerce_form_field(
			'expt_mobile',
			array(
				'type' => 'tel',
				'class' => array( 'input-text' ),
				'label' => __( 'Mobile Number', 'experttexting-official' ),
				'required' => true,
			),
			$checkout->get_value( 'expt_mobile' )
		);

    }

    /**
     * woocommerce_checkout_process
     * 
     * @since 1.1.0
     * 
     * @param $checkout
     */
    public function et_wc_user_number_validate_field() {
        
		// Error message.
		if ( !$_POST['expt_mobile'] ) {
			wc_add_notice( __( 'Please enter mobile number.', 'experttexting-official' ), 'error' );
		}
	}

    /**
     * woocommerce_checkout_update_order_meta
     * 
     * @since 1.1.0
     * 
     * @param $order_id
     */
    public function et_wc_user_number_update_field( $order_id ) {

		if ( !empty( $_POST['expt_mobile'] ) ) {
			update_post_meta( $order_id, 'expt_mobile', sanitize_text_field( $_POST['expt_mobile'] ) );
		}
	}

    /**
     * woocommerce_new_order
     * 
     * @since 1.1.0
     * 
     * @param $order_id
     */
    public function et_wc_notification_number( $order_id ) {

        global $__Expt_NotificationsCallbacks;

        $order = wc_get_order( $order_id );

        if( !$order ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $msg = !empty($msgOption) ? $msgOption['woocommerce_new_order_to_number_body'] : '';
        $phone = !empty($msgOption) ? $msgOption['woocommerce_new_order_to_number_receiver'] : '';

        $phone = explode(',', preg_replace('/\s+/', '', $phone));

        if( !$msg || !$phone ) return;
        
        $variables = array(
            'order_id' => $order_id,
            'order_status' => $order->get_status(),
            'order_total_cost' => (string) $order->get_total(),
            'order_date' => $order->get_date_created()->date("Y-m-d H:i:s"),
            'order_view_url' => $order->get_checkout_order_received_url(),
            'order_payment_url' => $order->get_checkout_payment_url(),
            'customer_first_name' => $order->get_billing_first_name(),
            'customer_email' => $order->get_billing_email(),
            'customer_phone' => $order->get_billing_phone(),
            'customer_country' => $order->get_billing_country(),
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );

    }

    /**
     * woocommerce_new_order
     * 
     * @since 1.1.0
     * 
     * @param $order_id
     */
    public function et_wc_notification_customer( $order_id ) {

        $mobile = '';

        if( $this->activated( 'et_integration_woocommerce', 'woocommerce_new_order_to_customer_chk' ) ) {

            if( !$_REQUEST['expt_mobile'] ) return;
            $mobile = sanitize_text_field( $_REQUEST['expt_mobile'] );
            
        } else {

            if( !$_REQUEST['billing_phone'] ) return;
            $mobile = sanitize_text_field( $_REQUEST['billing_phone'] );
        }

        global $__Expt_NotificationsCallbacks;

        $order = wc_get_order( $order_id );

        if( !$order ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $msg = !empty($msgOption) ? $msgOption['woocommerce_new_order_to_customer_body'] : '';

        if( !$msg ) return;
        
        $variables = array(
            'order_id' => $order_id,
            'order_status' => $order->get_status(),
            'order_total_cost' => (string) $order->get_total(),
            'order_date' => $order->get_date_created()->date("Y-m-d H:i:s"),
            'order_view_url' => $order->get_checkout_order_received_url(),
            'order_payment_url' => $order->get_checkout_payment_url(),
            'customer_first_name' => $order->get_billing_first_name(),
            'customer_email' => $order->get_billing_email(),
            'customer_phone' => $order->get_billing_phone(),
            'customer_country' => $order->get_billing_country(),
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( isset( $mobile ) && is_numeric( $mobile ) ) {

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($mobile) );
        }

    }

    /**
     * woocommerce_low_stock
     * 
     * @since 1.1.0
     * 
     * @param $stock
     */
    public function et_wc_low_stock_admin( $wc_product ) {

        global $__Expt_NotificationsCallbacks;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $msg = !empty($msgOption) ? $msgOption['woocommerce_stock_to_admin_body'] : '';

        if( !$msg ) return;
        
        $wc_product_data = $wc_product->get_data();

        $variables = array(
            'product_id' => $wc_product_data['id'],
            'product_name' => $wc_product_data['name'],
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($this->adminNumber) );
        }

    }

    /**
     * woocommerce_order_status_changed
     * 
     * @since 1.1.0
     * 
     * @param $order_id, $old_status, $new_status
     */
    public function et_wc_notification_change_status( $order_id, $old_status, $new_status ) {

        global $__Expt_NotificationsCallbacks;

        $order = wc_get_order( $order_id );

        if( !$order ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $msg = !empty($msgOption) ? $msgOption['woocommerce_status_to_customer_body'] : '';

        if( !$msg ) return;

        $expt_mobile = get_post_meta( $order_id, 'expt_mobile', true );
        $billing_phone = get_post_meta( $order_id, '_billing_phone', true );
        $mobile = '';

        if( !empty( $expt_mobile ) ) {
            $mobile = esc_attr( $expt_mobile );
        } elseif( !empty( $billing_phone ) ) {
            $mobile = esc_attr( $billing_phone );
        } else {
            return;
        }
        
        $variables = array(
            'order_id' => $order_id,
            'order_status' => $order->get_status(),
            'order_view_url' => $order->get_checkout_order_received_url(),
            'order_payment_url' => $order->get_checkout_payment_url(),
            'customer_first_name' => $order->get_billing_first_name(),
            'customer_email' => $order->get_billing_email(),
            'customer_phone' => $order->get_billing_phone(),
            'customer_country' => $order->get_billing_country(),
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( isset( $mobile ) && is_numeric( $mobile ) ) {

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($mobile) );
        }

    }

    /**
     * woocommerce_order_status_changed
     * 
     * @since 1.1.0
     * 
     * @param $order_id, $old_status, $new_status
     */
    public function et_wc_notification_change_by_status( $order_id, $old_status, $new_status ) {

        global $__Expt_NotificationsCallbacks;

        $order = wc_get_order( $order_id );

        if( !$order ) return;

        $option = $this->expt_escape_array_data( get_option( 'et_integration_woocommerce' ) );
        $opt = !empty($option) ? $option['woocommerce_notify_by_status_to_customer'] : '';

        if( !is_array($opt) ) return;

        foreach( $opt as $index => $value ) {

            $chk = $value["woocommerce_notify_by_status_to_customer_chk"];
            $status = $value["woocommerce_notify_by_status_to_customer_select"];
            $msg = $value["woocommerce_notify_by_status_to_customer_body"];

            if( !isset( $chk ) || !$msg || $status != $new_status ) return;

            $expt_mobile = get_post_meta( $order_id, 'expt_mobile', true );
            $billing_phone = get_post_meta( $order_id, '_billing_phone', true );
            $mobile = '';

            if( !empty( $expt_mobile ) ) {
                $mobile = esc_attr( $expt_mobile );
            } elseif( !empty( $billing_phone ) ) {
                $mobile = esc_attr( $billing_phone );
            } else {
                return;
            }

            $variables = array(
                'order_id' => $order_id,
                'order_status' => $order->get_status(),
                'order_view_url' => $order->get_checkout_order_received_url(),
                'order_payment_url' => $order->get_checkout_payment_url(),
                'customer_first_name' => $order->get_billing_first_name(),
                'customer_email' => $order->get_billing_email(),
                'customer_phone' => $order->get_billing_phone(),
                'customer_country' => $order->get_billing_country(),
            );

            foreach ($variables as $key => $value) {
            
                $msg = str_replace( "[" . $key . "]", $value, $msg );
            }

            $message = strip_tags($msg);

            if( isset( $mobile ) && is_numeric( $mobile ) ) {

                $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($mobile) );
            }

        }

    }

    /**
     * edd_complete_purchase
     * 
     * @since 1.0.0
     * 
     * @param $payment_id
     */
    public function et_edd_new_order( $payment_id ) {
        
        global $__Expt_NotificationsCallbacks;
        
        $payment_meta = edd_get_payment_meta( $payment_id );
        $transaction_id = edd_get_payment_transaction_id( $payment_id );

        if( !$payment_meta ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_edd' ) );
        $msg = !empty($msgOption) ? $msgOption['easy_digital_downloads_body'] : '';

        if( !$msg ) return;
        
        $variables = array(
            'edd_order_id' => $transaction_id,
            'edd_order_date' => $payment_meta['date'],
            'edd_email' => $payment_meta['email'],
            'edd_first' => $payment_meta['user_info']['first_name'],
            'edd_last' => $payment_meta['user_info']['last_name'],
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($this->adminNumber) );
        }

    }

    /**
     * edd_purchase_form_user_info
     * 
     * @since 1.1.0
     */
    public function et_edd_user_number_field() {
		?>
        <p id="edd-exptmobile-wrap">
			<label class="edd-label" for="edd-exptmobile"><?php _e('Mobile Number', 'experttexting-official'); ?></label>
			<span class="edd-description" id="edd-exptmobile-description"><?php _e('Enter your mobile number to get SMS', 'experttexting-official'); ?></span>
			<input class="edd-input" type="text" name="edd_exptmobile" id="edd-exptmobile" placeholder="<?php _e('Mobile number', 'experttexting-official'); ?>" value="">
		</p>
		<?php
	}

    /**
     * edd_checkout_error_checks
     * 
     * @since 1.1.0
     * 
     * @param $valid_data, $data
     */
    public function et_edd_validate_number_field( $valid_data, $data ) {

		if( empty( $data['edd_exptmobile'] ) ) {
			edd_set_error( 'invalid_exptmobile', __('Please provide your mobile number.', 'experttexting-official') );
		}

	}

    /**
     * edd_payment_meta
     * 
     * @since 1.1.0
     * 
     * @param $payment_meta
     */
    public function et_edd_store_number_field( $payment_meta ) {

		$payment_meta['edd_exptmobile'] = isset( $_POST['edd_exptmobile'] ) ? sanitize_text_field( $_POST['edd_exptmobile'] ) : '';
		return $payment_meta;
	}

    /**
     * edd_payment_personal_details_list
     * 
     * @since 1.1.0
     * 
     * @param $payment_meta, $user_info
     */
    public function et_edd_purchase_details( $payment_meta, $user_info ) {

		$mobile = isset( $payment_meta['edd_exptmobile'] ) ? $payment_meta['edd_exptmobile'] : 'none';
		?>
        <div class="column-container">
            <div class="column">
                <strong><?php _e('Mobile Number:', 'experttexting-official'); ?></strong>
                <?php _e($mobile); ?>
            </div>
        </div>
		<?php
	}

    /**
     * edd_complete_purchase
     * 
     * @since 1.1.0
     * 
     * @param $payment_id
     */
    public function et_edd_notification_number( $payment_id ) {

        global $__Expt_NotificationsCallbacks;
        
        $payment_meta = edd_get_payment_meta( $payment_id );
        $transaction_id = edd_get_payment_transaction_id( $payment_id );

        if( !$payment_meta ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_edd' ) );
        $msg = !empty($msgOption) ? $msgOption['easy_digital_downloads_new_order_to_number_body'] : '';
        $phone = !empty($msgOption) ? $msgOption['easy_digital_downloads_new_order_to_number_receiver'] : '';

        $phone = explode(',', preg_replace('/\s+/', '', $phone));

        if( !$msg || !$phone ) return;
        
        $variables = array(
            'edd_order_id' => $transaction_id,
            'edd_order_date' => $payment_meta['date'],
            'edd_email' => $payment_meta['email'],
            'edd_first' => $payment_meta['user_info']['first_name'],
            'edd_last' => $payment_meta['user_info']['last_name'],
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );

	}

    /**
     * edd_complete_purchase
     * 
     * @since 1.1.0
     * 
     * @param $payment_id
     */
	public function et_edd_notification_customer( $payment_id ) {

		if( !$_REQUEST['edd_exptmobile'] ) return;

        global $__Expt_NotificationsCallbacks;
        
        $payment_meta = edd_get_payment_meta( $payment_id );
        $transaction_id = edd_get_payment_transaction_id( $payment_id );

        if( !$payment_meta ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_edd' ) );
        $msg = !empty($msgOption) ? $msgOption['easy_digital_downloads_new_order_to_customer_body'] : '';

        if( !$msg ) return;
        
        $variables = array(
            'edd_order_id' => $transaction_id,
            'edd_order_date' => $payment_meta['date'],
            'edd_email' => $payment_meta['email'],
            'edd_first' => $payment_meta['user_info']['first_name'],
            'edd_last' => $payment_meta['user_info']['last_name'],
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        if( isset( $_REQUEST['edd_exptmobile'] ) && is_numeric( $_REQUEST['edd_exptmobile'] ) ) {

            $mobile = sanitize_text_field( $_REQUEST['edd_exptmobile'] );
            $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($mobile) );
        }

	}

    /**
     * gform_after_submission
     * 
     * @since 1.2.0
     * 
     * @param $entry
     * @param $form
     */
    public function et_gf_message_form( $entry, $form ) {

        global $__Expt_NotificationsCallbacks;

        // Send to number
        if( $this->activated( 'et_integration_gf', 'gravity_forms_num_chk_'. $form['id'] ) ) {

            $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_gf' ) );
            $msg = !empty($msgOption) ? $msgOption['gravity_forms_num_body_'. $form['id']] : '';
            $phone = !empty($msgOption) ? $msgOption['gravity_forms_num_receiver_'. $form['id']] : '';

            $phone = explode(',', preg_replace('/\s+/', '', $phone));

            if( $msg && $phone ) {

                $variables = array(
                    'gf_form_title' => esc_attr($form['title']),
                    'gf_ip' => esc_attr($entry['ip']),
                    'gf_form_url' => esc_attr($entry['source_url']),
                    'gf_user_agent' => esc_attr($entry['user_agent']),
                );
    
                foreach( $form['fields'] as $field ) {
                    if( isset($entry[$field['id']]) ) {
                        $variables['gf_field_'.$field['label']] = esc_attr($entry[$field['id']]);
                        
                    } elseif( is_array( $field['inputs'] ) ) {
                        foreach( $field['inputs'] as $field_multi ) {
                            $variables['gf_field_'.$field['label']][] = esc_attr($entry[$field_multi['id']]);
                        }
                        $variables['gf_field_'.$field['label']] = implode(" ", $variables['gf_field_'.$field['label']]);
                    }
                }

                foreach ($variables as $key => $value) {
                
                    $msg = str_replace( "[" . $key . "]", $value, $msg );
                }
    
                $message = strip_tags($msg);
    
                $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );
                
            }

        }

        // Send to field
        if( $this->activated( 'et_integration_gf', 'gravity_forms_field_chk_'. $form['id'] ) ) {

            $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_gf' ) );
            $msg = !empty($msgOption) ? $msgOption['gravity_forms_field_body_'. $form['id']] : '';
            $phoneFieldId = !empty($msgOption) ? $msgOption['gravity_forms_field_select_'. $form['id']] : '';

            foreach( $phoneFieldId as $index => $value ) {
                $phone = !empty($entry[$value]) ? esc_attr($entry[$value]) : '';
            }

            if( $msg && $phone ) {

                $variables = array(
                    'gf_form_title' => esc_attr($form['title']),
                    'gf_ip' => esc_attr($entry['ip']),
                    'gf_form_url' => esc_attr($entry['source_url']),
                    'gf_user_agent' => esc_attr($entry['user_agent']),
                );

                foreach( $form['fields'] as $field ) {
                    if( isset($entry[$field['id']]) ) {
                        $variables['gf_field_'.$field['label']] = esc_attr($entry[$field['id']]);
                        
                    } elseif( is_array( $field['inputs'] ) ) {
                        foreach( $field['inputs'] as $field_multi ) {
                            $variables['gf_field_'.$field['label']][] = esc_attr($entry[$field_multi['id']]);
                        }
                        $variables['gf_field_'.$field['label']] = implode(" ", $variables['gf_field_'.$field['label']]);
                    }
                }
    
                foreach ($variables as $key => $value) {
                
                    $msg = str_replace( "[" . $key . "]", $value, $msg );
                }
    
                $message = strip_tags($msg);
    
                $__Expt_NotificationsCallbacks->sendMessageReq( $message, array( $phone ) );
                
            }

        }

	}

    /**
     * quform_post_process
     * 
     * @since 1.2.0
     * 
     * @param $result
     * @param $form
     */
    public function et_qf_message_form( $result, $form ) {

        global $__Expt_NotificationsCallbacks;
        $qf_form_fields = array();
        $qf_form_values = $form->getValues();
        $qf_form_id = esc_attr( $_REQUEST['quform_form_id'] );

        if ( class_exists( 'Quform_Repository' ) ) {
            $quFormInstance = new \Quform_Repository;
            $quForms = $quFormInstance->allForms();
            if( $quForms ) {
                foreach( $quForms as $form ) {

                    if( $form['id'] == $qf_form_id ) {
                        if( $form['elements'] ) {
                            foreach( $form['elements'] as $elements ) {
                                foreach( $elements['elements'] as $field ) {
                                    if( isset( $field['label'] ) && $field['type'] != 'submit' ) {
                                        $qf_form_fields[$field['label']] = $field['id'];
                                    }
                                }
                            }
                        }
                    }

                }
            }
        }

        // Send to number
        if( $this->activated( 'et_integration_qf', 'qu_forms_num_chk_'. $qf_form_id ) ) {

            $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_qf' ) );
            $msg = !empty($msgOption) ? $msgOption['qu_forms_num_body_'. $qf_form_id] : '';
            $phone = !empty($msgOption) ? $msgOption['qu_forms_num_receiver_'. $qf_form_id] : '';

            $phone = explode(',', preg_replace('/\s+/', '', $phone));

            if( $msg && $phone ) {

                $variables = array(
                    'qf_post_title' => esc_attr( $_POST['post_title'] ),
                    'qf_form_url' => esc_attr( $_POST['form_url'] ),
                    'qf_referring_url' => esc_attr( $_POST['referring_url'] ),
                );

                if( is_array( $qf_form_fields ) && count( $qf_form_fields ) ) {
                    foreach( $qf_form_fields as $label => $id ) {
                        $form_val = $qf_form_values['quform_'.$qf_form_id.'_'.$id];
                        $variables['qf_field_'.esc_attr( $label )] = is_array( $form_val ) ? implode(" ", $form_val) : $form_val;
                    }
                }
    
                foreach ($variables as $key => $value) {
                
                    $msg = str_replace( "[" . $key . "]", $value, $msg );
                }
    
                $message = strip_tags($msg);
    
                $__Expt_NotificationsCallbacks->sendMessageReq( $message, $phone );
                
            }

        }

        // Send to field
        if( $this->activated( 'et_integration_qf', 'qu_forms_field_chk_'. $qf_form_id ) ) {

            $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_qf' ) );
            $msg = !empty($msgOption) ? $msgOption['qu_forms_field_body_'. $qf_form_id] : '';
            $phoneFieldId = !empty($msgOption) ? $msgOption['qu_forms_field_select_'. $qf_form_id] : '';

            foreach( $phoneFieldId as $index => $value ) {
                $phone = !empty($qf_form_values['quform_'.$qf_form_id.'_'.$value]) ? esc_attr($qf_form_values['quform_'.$qf_form_id.'_'.$value]) : '';
            }

            if( $msg && $phone ) {

                $variables = array(
                    'qf_post_title' => esc_attr( $_POST['post_title'] ),
                    'qf_form_url' => esc_attr( $_POST['form_url'] ),
                    'qf_referring_url' => esc_attr( $_POST['referring_url'] ),
                );

                if( is_array( $qf_form_fields ) && count( $qf_form_fields ) ) {
                    foreach( $qf_form_fields as $label => $id ) {
                        $form_val = $qf_form_values['quform_'.$qf_form_id.'_'.$id];
                        $variables['qf_field_'.esc_attr( $label )] = is_array( $form_val ) ? implode(" ", $form_val) : $form_val;
                    }
                }
    
                foreach ($variables as $key => $value) {
                
                    $msg = str_replace( "[" . $key . "]", $value, $msg );
                }
    
                $message = strip_tags($msg);
    
                $__Expt_NotificationsCallbacks->sendMessageReq( $message, array( $phone ) );
                
            }

        }

	}

    /**
     * bp_init
     * 
     * @since 1.2.0
     */
    public function et_bp_user_number_field() {
        
		global $bp;
		$xfield_args = array(
			'field_group_id' => 1,
			'name' => __('Mobile Number', 'experttexting-official'),
			'description' => __('Enter your mobile number to get SMS', 'experttexting-official'),
			'can_delete' => true,
			'field_order' => 1,
			'is_required' => false,
			'type' => 'textbox'
		);

		xprofile_insert_field($xfield_args);
	}

    /**
     * bp_activity_sent_mention_email
     * 
     * @since 1.2.0
     */
    public function et_bp_notification_mention( $activity, $subject, $message, $content, $receiver_user_id ) {
        
        global $__Expt_NotificationsCallbacks;

		$user_posted = get_userdata( $activity->user_id );
		$user_receiver = get_userdata( $receiver_user_id );

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_bp' ) );
        $msg = !empty($msgOption) ? $msgOption['buddypress_mentions_body'] : '';

        $phone = $this->et_bp_get_mobile( $receiver_user_id );

        if( !$msg || !$phone ) return;

        $variables = array(
            'bp_user_posted_display_name' => $user_posted->display_name,
            'bp_primary_link' => $activity->primary_link,
            'bp_time' => $activity->date_recorded,
            'bp_mention_content' => $content,
            'bp_user_receiver_display_name' => $user_receiver->display_name,
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($phone) );
        
	}

    /**
     * bp_activity_comment_posted
     * 
     * @since 1.2.0
     */
    public function et_bp_notification_comment_post( $comment_id, $r, $activity) {
        
        global $__Expt_NotificationsCallbacks;

		$user_posted = get_userdata( $r['user_id'] );
		$user_receiver = get_userdata( $activity->user_id );

        $msgOption = $this->expt_escape_array_data( get_option( 'et_integration_bp' ) );
        $msg = !empty($msgOption) ? $msgOption['buddypress_comment_post_body'] : '';

        $phone = $this->et_bp_get_mobile( $activity->user_id );

        if( !$msg || !$phone ) return;

        $variables = array(
            'bp_user_posted_display_name' => $user_posted->display_name,
            'bp_comment_content' => $r['content'],
            'bp_user_receiver_display_name' => $user_receiver->display_name,
        );

        foreach ($variables as $key => $value) {
            
            $msg = str_replace( "[" . $key . "]", $value, $msg );
        }
        
        $message = strip_tags($msg);

        $__Expt_NotificationsCallbacks->sendMessageReq( $message, array($phone) );

	}
    
    /**
     * Get BuddyPress mobile number
     * 
     * @since 1.2.0
     */
	private function et_bp_get_mobile( $user_id ) {

		global $wpdb;
		$field = $wpdb->get_row($wpdb->prepare( "SELECT `id` FROM {$wpdb->prefix}bp_xprofile_fields WHERE name = %s", 'Mobile Number' ));

		if( $field ) {
			$result = $wpdb->get_row($wpdb->prepare( "SELECT * FROM {$wpdb->prefix}bp_xprofile_data WHERE field_id = %d AND user_id = %d", $field->id, $user_id ));

			if ( !$result ) {
				return false;
			}
			return $result->value;
		}
        return false;
	}
    

}