<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api\Expt_Callbacks;

use ExptTxt\Expt_Base\Expt_BaseController;

class Expt_ManagerCallbacks extends Expt_BaseController {

    public function etSanitizeSetGroup( $input ) {

        $output = $this->expt_sanitize_array_data( $input );

        $sanitizeFreeArray = array(
            'newsletter_privacy_body'
        );
        $output = $this->etSanitizeFree( $input, $output, $sanitizeFreeArray );

        return $output;
    }

    public function etSanitizeFree( $input, $output, $keysArray ) {

        if( !is_array( $keysArray ) ) return $output;

        $limited_html_tags = array(
            'span' => array(
                'style' => array(),
            ),
            'div' => array(
                'style' => array(),
            ),
            'a' => array(
                'href' => array(),
                'target' => array(),
                'title' => array(),
                'style' => array()
            ),
            'ul' => array(
                'style' => array(),
            ),
            'li' => array(
                'style' => array(),
            ),
            'p' => array(
                'style' => array(),
            ),
            'mark' => array(
                'style' => array(),
            ),
            'code' => array(
                'style' => array(),
            ),
            'strong' => array(),
            'b' => array(),
            'br' => array(),
            'em' => array(),
            'i' => array(),
        );

        foreach( $keysArray as $key ) {
            if( isset($input[$key]) ) {
                $output[$key] = wp_kses( $input[$key], $limited_html_tags );
            }
        }

        return $output;
    }

    public function etAdminSectionManager() {

        esc_html_e('Manage the features of this plugin.', 'experttexting-official');
    }

    public function etGeneralSetManager() {

        esc_html_e('Manage the general settings of this plugin.', 'experttexting-official');
    }

    public function etIntegrationSetManager() {
        
        esc_html_e('Manage and configure the integration settings.', 'experttexting-official');
    }

    public function etNotificationsSetManager() {
        
        esc_html_e('Manage and configure the notification settings.', 'experttexting-official');
    }

    public function etNewsletterSetManager() {
        
        esc_html_e('Manage and configure the newsletter settings.', 'experttexting-official');
    }

    public function etApiSetManager() {
        
        esc_html_e('Manage and configure the API settings', 'experttexting-official');
    }

    public function etSetSMSManager($args) {
        
        esc_html_e('Send SMS to your contacts', 'experttexting-official');
    }

    public function checkboxField( $args ) {
        
        $name = esc_attr( $args['label_for'] );
        $option_name = esc_attr( $args['option_name'] );
        $subText = $args['subText'];
        $checkbox = $this->expt_escape_array_data( get_option( $option_name ) );

        $checked = isset($checkbox[$name]) ? ($checkbox[$name] ? true : false) : false;

        $printField = '<label class="switch"><input type="checkbox" name="' . esc_attr( $option_name ) . '[' . esc_attr( $name ) . ']' . '" value="1" class="et-checkbox-class" ' . ( $checked ? 'checked' : '' ) . '><span class="slider round"></span></label><br>' . wp_kses( $subText, $this->allowed_html );
        _e( $printField );
    }

    public function textboxField($args) {
        
        $name = esc_attr($args['label_for']);
        $additionalArgs = $args['additionalArgs'];
        $option_name = esc_attr($args['option_name']);
        $subText = $args['subText'];
        $sectionIs = $args['sectionIs'];
        $value = $this->expt_escape_array_data( get_option( $option_name ) );
        $val = (!empty($value) && !empty($value[$name])) ? esc_attr( $value[$name] ) : "";

        // Section's customized values
        if ( $sectionIs == 'managers_API' ) {
            $val = ($name == "et_api_sender" && empty( $value[$name] )) ? "DEFAULT" : $val;
        }
        if ( $sectionIs == 'managers_SendSMS' ) {
            $sender = $this->apiSender;
            $val = ($name == "et_send_from" ? ( $sender ? $sender : "DEFAULT" ) : "");
        }
        
        $printField = '<input ' . wp_kses( $additionalArgs, $this->allowed_html ) . ' name="' . esc_attr( $option_name ) . '[' . esc_attr( $name ) . ']' . '" value="' . esc_attr( $val ) . '">' . wp_kses( $subText, $this->allowed_html );
        _e( $printField );
        
    }

    public function textareaField($args) {

        $name = esc_attr($args['label_for']);
        $additionalArgs = $args['additionalArgs'];
        $option_name = esc_attr($args['option_name']);
        $subText = $args['subText'];
        $isHtmlSupported = !empty($args['isHtmlSupported']) ? $args['isHtmlSupported'] : false;
        $sectionIs = $args['sectionIs'];
        $value = $this->expt_escape_array_data( get_option( $option_name ) );
        $val = (!empty($value) && !empty($value[$name])) ? esc_attr( $value[$name] ) : "";

        $printField = '<textarea ' . wp_kses( $additionalArgs, $this->allowed_html ) . ' rows=5 name="' . esc_attr( $option_name ) . '[' . esc_attr( $name ) . ']">' . ( $isHtmlSupported ? wp_kses( $val, $this->allowed_html ) : esc_attr( $val ) ) . '</textarea>' . '<br>' . wp_kses( $subText, $this->allowed_html );
        _e( $printField );
        
    }

    public function noFieldOnlyText($args) {

        $printField = !empty($args['subText']) ? wp_kses( $args['subText'], $this->allowed_html ) : '';
        _e( $printField );
    }

    public function selectField($args) {

        global $__Expt_ApiCallbacks;

        $name = esc_attr($args['label_for']);
        $additionalArgs = $args['additionalArgs'];
        $option_name = esc_attr($args['option_name']);
        $subText = $args['subText'];
        $selectOptions = $args['select_option'];
        $sectionIs = $args['sectionIs'];
        $value = $this->expt_escape_array_data( get_option( $option_name ) );
        $val = (!empty($value) && !empty($value[$name])) ? $value[$name] : array();

        $res = '';

        if ( $sectionIs == 'managers_SendSMS' ) {

            $subscribers = $__Expt_ApiCallbacks->getNumberOfSubscribers();
            $groups = $__Expt_ApiCallbacks->getNameOfGroups();
            $wpUsers = $__Expt_ApiCallbacks->getNumberOfWordpressUsers();

            // Subscriber
            $subText .= "<div style='' id='select_subscriber_users'><select name='et_send_sms[et_send_to][]' id='subscriber_users' class='chosen-select regular-text mb-2'>";
            $subText .= '<option value="all_sub">'. __( 'All Subscribers', 'experttexting-official' ) .' ('. ( $subscribers ? absint(count($subscribers)) : 0 ) .' '. __( 'Active', 'experttexting-official' ) .')</option>';
            if ( $groups ) {
                foreach ( $groups as $group ) {
                    $groupCount = $__Expt_ApiCallbacks->getNumberOfSubscribers($group->ID);
                    $count = $groupCount ? count($groupCount) : 0;
                    $subText .= '<option value="'. absint($group->ID) .'">'. esc_attr($group->name) .' ('. absint($count) .' '. __( 'Active', 'experttexting-official' ) .')</option>';
                }
            }
            $subText .= "</select></div>";

            // Wordpress Users
            $subText .= "<div style='display: none;' id='select_wordpress_users'>";
            $wpCount = $wpUsers ? absint(count($wpUsers)) : 0;
            $subText .= '<span>'. sprintf(__( '%s user(s) have mobile number', 'experttexting-official' ), $wpCount) .'</span>';
            $subText .= "</div>";
            
            // Add Numbers
            $subText .= "<div style='display: none;' id='select_add_numbers'><select placeholder='". __( 'Add number(s) with country code', 'experttexting-official' ) ."' data-placeholder='". __( 'Add number(s) with country code', 'experttexting-official' ) ."' name='et_send_sms[et_send_to][]' id='add_numbers' class='chosen-select add-numbers regular-text mb-2' multiple></select></div>";
            
        }

        $res .= '<select data-placeholder=" " name="' . esc_attr( $option_name ) . '[' . esc_attr( $name ) . '][]" ' . wp_kses( $additionalArgs, $this->allowed_html ) . '>';
        
        foreach ($selectOptions as $key => $value) {

            if ( $val != "" && $sectionIs != 'managers_SendSMS' ) {
                
                foreach ($val as $key2 => $value2) {
                    
                    if ( $value2 == $value ) {

                        $res .= '<option ' . 'selected' . ' value="' . esc_attr( $value ) . '">' . esc_attr( $key ) . '</option>';
                    }
                }

            } else {
                $res .= '<option value="' . esc_attr( $value ) . '">' . esc_attr( $key ) . '</option>';
            }
        }

        // Check non-selected values
        if ( $val != "" && $sectionIs != 'managers_SendSMS' ) {

            $diff = array_diff( $selectOptions, $val );

            foreach ($diff as $keyDiff => $valueDiff) {
        
                $res .= '<option value="' . esc_attr( $valueDiff ) . '">' . esc_attr( $keyDiff ) . '</option>';
            }
        } 

        $res .= '</select>' . '<br>' . wp_kses( $subText, $this->allowed_html );

        $printField = !empty($res) ? $res : '';
        _e( $printField );

    }

    public function repeatFieldsWC($args) {

        $name = esc_attr($args['label_for']);
        $option_name = esc_attr($args['option_name']);
        $value = $this->expt_escape_array_data( get_option( $option_name ) );

        $val = (!empty($value) && !empty($value[$name])) ? $value[$name] : array();
        $option = '';
        $k = 0;
        $wc_order_statuses = wc_get_order_statuses();
        
        foreach( $wc_order_statuses as $status_key => $status_name ) {
            $key = str_replace('wc-', '', $status_key);
            $option .= '<option value="'. $key .'">'. $status_name .'</option>';
        }

        _e('
        <script>
            function addAnotherField() {
                var len = jQuery(".woocommerce_notify_by_status_to_customer").length;
                field_data = `
                    <div class="'. $name .'" style="display: none;" id="'. $name . '`+ len +`">
                        <label class="switch">
                            <input type="checkbox" name="et_integration_woocommerce['. $name .'][`+ len +`]['. $name .'_chk]" value="1" class="et-checkbox-class">
                        <span class="slider round"></span></label><br><span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever the status of order is changed to below selected status.</span>
                        
                        <br><br>
                        
                        <select name="et_integration_woocommerce['. $name .'][`+ len +`]['. $name .'_select]" class="chosen-select regular-text mb-2">
                        '. $option .'
                        </select>
                        
                        <br><br>
                        
                        <textarea maxlength="800" class="regular-text mb-1" rows="5" name="et_integration_woocommerce['. $name .'][`+ len +`]['. $name .'_body]"></textarea>
                        <br><span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important;padding-left: 0 !important"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>
                        <br><button type="button" onclick="deleteAnotherField('. $name . '`+ len +`);" class="button btn-danger">'. __( 'Delete', 'experttexting-official' ) .'</button>
                        <hr>
                    </div>
                `;
                jQuery(field_data).appendTo("#woocommerce_notify_by_status_to_customer_div").slideDown("fast");
                jQuery(".chosen-select").chosen();
            }

            function deleteAnotherField(Fid) {
                jQuery(Fid).slideUp("fast", function() {
                    jQuery(this).remove();
                });
            }
        </script>
        ');

        $res = '<div id="'. $name .'_div">';
        
        if( count( $val ) && is_array( $val ) ) {

            foreach( $val as $index => $item ) {

                $checked = (!empty($item["woocommerce_notify_by_status_to_customer_chk"])) ? ($item["woocommerce_notify_by_status_to_customer_chk"] ? true : false) : false;
                $val_body = (!empty($item["woocommerce_notify_by_status_to_customer_body"])) ? esc_attr($item["woocommerce_notify_by_status_to_customer_body"]) : '';
                $val_select = (!empty($item["woocommerce_notify_by_status_to_customer_select"])) ? esc_attr($item["woocommerce_notify_by_status_to_customer_select"]) : '';
                $option = '';

                foreach( $wc_order_statuses as $status_key => $status_name ) {
                    $key = str_replace('wc-', '', $status_key);
                    $option .= '<option value="'. $key .'" '. (($val_select == $key) ? 'selected' : '') .'>'. $status_name .'</option>';
                }

                $res .= '
                    <div class="'. $name .'" id="'. $name . $index .'">
                        <label class="switch">
                            <input type="checkbox" name="et_integration_woocommerce['. $name .']['. $index .']['. $name .'_chk]" value="1" class="et-checkbox-class"'. ( $checked ? 'checked' : '' ) .'>
                        <span class="slider round"></span></label><br><span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever the status of order is changed to below selected status.</span>
                    
                        <br><br>

                        <select name="et_integration_woocommerce['. $name .']['. $index .']['. $name .'_select]" class="chosen-select regular-text mb-2">
                            '. $option .'
                        </select>

                        <br><br>

                        <textarea maxlength="800" class="regular-text mb-1" rows="5" name="et_integration_woocommerce['. $name .']['. $index .']['. $name .'_body]">' . esc_attr( $val_body ) . '</textarea>
                        <br><span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important;padding-left: 0 !important"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>
                    ';

                ($index > 0) ? $res .= '<button type="button" onclick="deleteAnotherField('. $name . $index .');" class="button btn-danger">'. __( 'Delete', 'experttexting-official' ) .'</button>' : '';

                $res .= '<hr>
                </div>';

            }

        } else {

            $res .= '
                <div class="'. $name .'" id="'. $name . $k .'">
                    <label class="switch">
                        <input type="checkbox" name="et_integration_woocommerce['. $name .']['. $k .']['. $name .'_chk]" value="1" class="et-checkbox-class">
                    <span class="slider round"></span></label><br><span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever the status of order is changed to below selected status.</span>
                    
                    <br><br>
                    
                    <select name="et_integration_woocommerce['. $name .']['. $k .']['. $name .'_select]" class="chosen-select regular-text mb-2">
                    '. $option .'
                    </select>
                    
                    <br><br>
                    
                    <textarea maxlength="800" class="regular-text mb-1" rows="5" name="et_integration_woocommerce['. $name .']['. $k .']['. $name .'_body]"></textarea>
                    <br><span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important;padding-left: 0 !important"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>
                    <hr>
                </div>
            ';

        }

        $res .= '</div><button type="button" onclick="addAnotherField();" class="button btn-success">'. __( 'Add Another Order Status', 'experttexting-official' ) .'</button>';

        $printField = !empty($res) ? $res : '';
        _e( $printField );
    }


    public function selectCountriesField($args) {

        $name = esc_attr($args['label_for']);
        $additionalArgs = $args['additionalArgs'];
        $option_name = esc_attr($args['option_name']);
        $subText = $args['subText'];
        $selectOptions = $args['select_option'];
        $value = $this->expt_escape_array_data( get_option( $option_name ) );
        $val = (!empty($value) && !empty($value[$name])) ? $value[$name] : array();

        $res = '<select data-placeholder=" " name="' . esc_attr( $option_name ) . '[' . esc_attr( $name ) . '][]" ' . wp_kses( $additionalArgs, $this->allowed_html ) . '>';
        
        foreach ($selectOptions as $key => $value) {

            foreach($value as $arr_key => $arr_value) {

                if ( $val != "" ) {
                    
                    foreach ($val as $key2 => $value2) {
                        
                        if ( $value2 == $arr_value ) {
    
                            $res .= '<option ' . 'selected' . ' value="' . esc_attr( $arr_value ) . '">' . esc_attr( $arr_key ) . '</option>';
                        }
                    }
    
                } else {
                    $res .= '<option value="' . esc_attr( $arr_value ) . '">' . esc_attr( $arr_key ) . '</option>';
                }
                
            }

        }

        // Check non-selected values
        if ( $val != "" ) {

            foreach ($selectOptions as $key => $value) {

                $diff = array_diff( $value, $val );
    
                foreach ($diff as $keyDiff => $valueDiff) {
            
                    $res .= '<option value="' . esc_attr( $valueDiff ) . '">' . esc_attr( $keyDiff ) . '</option>';
                }
            }

        } 

        $res .= '</select>' . '<br>' . wp_kses( $subText, $this->allowed_html );

        $printField = !empty($res) ? $res : '';
        _e( $printField );

    }

}
