<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api\Expt_Callbacks;

use ExptTxt\Expt_Base\Expt_BaseController;

class Expt_NewsletterCallbacks extends Expt_BaseController {

    /**
     * Get Subscribers Counting
     *
     * @return array|object|null
     */
    public static function getSubsCount($status = "") {

        global $wpdb, $__Expt_BaseController;
        $table   = $wpdb->prefix . 'et_subscribes';
        if($status != "") {
            $prepare = $wpdb->get_results("SELECT * FROM {$table} where status = {$status}");
        } else {
            $prepare = $wpdb->get_results("SELECT * FROM {$table}");
        }

        if(!$prepare) {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

        } else {
            
            $count = is_array($prepare) ? count($prepare) : 0;
            return $count;
        }


    }



    /**
     * Get Subscribers Groups
     *
     * @return array|object|null
     */
    public static function getSubsGroupCount($group_id) {

        global $wpdb, $__Expt_BaseController;
        $table   = $wpdb->prefix . 'et_subscribes';
        $prepare = $wpdb->prepare("SELECT COUNT(*) FROM {$table} WHERE `group_ID` = %d", $group_id);

        if(!$prepare) {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

        } else {
            
            $count = $wpdb->get_var($prepare);
            return $count;
        }


    }



    /**
     * Get Groups
     *
     * @return array|object|null
     */
    public static function getGroups() {

        global $wpdb, $__Expt_BaseController;
        $result = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}et_subscribes_group`");

        if ($result) {
            return $result;

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

    }


     /**
     * Get Group by group ID
     *
     * @param $group_id
     *
     * @return object|null
     */
    public static function getGroup($group_id) {

        global $wpdb, $__Expt_BaseController;

        $db_prepare = $wpdb->prepare("SELECT * FROM `{$wpdb->prefix}et_subscribes_group` WHERE `ID` = %d", $group_id);

        if($db_prepare) {

            $result = $wpdb->get_row($db_prepare);
            
            if ($result) {
                return $result;
            }

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

        }

        return null;
    }


    /**
     * Add Group
     *
     * @param Not param
     *
     * @return array
     */
    public static function addGroup($name) {

        global $wpdb, $__Expt_BaseController;

        if (empty($name)) {
            return array('result' => 'error', 'message' => __('Name is empty!', 'experttexting-official'));
        }

        $table   = $wpdb->prefix . 'et_subscribes_group';
        $prepare = $wpdb->prepare("SELECT COUNT(ID) FROM {$table} WHERE `name` = %s", $name);
        $count   = $wpdb->get_var($prepare);
        if ($count) {
            return array(
                'result'  => 'error',
                'message' => sprintf(__('Group Name "%s" exists!', 'experttexting-official'), $name)
            );
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

            $result = $wpdb->insert(
                $wpdb->prefix . "et_subscribes_group",
                array(
                    'name' => $name,
                )
            );

            if ($result) {

                /**
                 * Run hook after adding group.
                 *
                 * @param string $name
                 * result query.
                 */
                do_action('expttxt_add_group', $name);

                return array('result' => 'success', 'message' => __('Group successfully added.', 'experttexting-official'));
            } else {

                if( $wpdb->last_error ) {
                    $__Expt_BaseController->save_expt_wpdb_log();
                }

                return array(
                    'result'  => 'error',
                    'message' => __('Having problem while adding group, please try again later.', 'experttexting-official')
                );
            }
        }

    }



    /**
     * Update Group
     *
     * @param $id
     * @param $name
     *
     * @return array|void
     * @internal param param $Not
     */
    public static function updateGroup($id, $name) {

        global $wpdb, $__Expt_BaseController;

        if (empty($id) or empty($name)) {
            return;
        }

        $table   = $wpdb->prefix . 'et_subscribes_group';
        $prepare = $wpdb->prepare("SELECT ID FROM {$table} WHERE `name` = %s", $name);
        $query_id   = $wpdb->get_var($prepare);

        if ($query_id) {

            if ($query_id == $id) {
                return array();
            } else {
                return array(
                    'result'  => 'error',
                    'message' => sprintf(__('Group Name "%s" exists!', 'experttexting-official'), $name)
                );
            }

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

            $result = $wpdb->update(
                $wpdb->prefix . "et_subscribes_group",
                array(
                    'name' => $name,
                ),
                array(
                    'ID' => $id
                )
            );

            if ($result) {

                /**
                 * Run hook after updating group.
                 *
                 * @param string $name
                 * result query.
                 */
                do_action('expttxt_update_group', $name, $id);
                
                return array('result' => 'success', 'message' => __('Group successfully updated.', 'experttexting-official'));
            } else {
                
                if( $wpdb->last_error ) {
                    $__Expt_BaseController->save_expt_wpdb_log();
                }

                return array(
                    'result'  => 'error',
                    'message' => sprintf(__('Having problem while updating group "%s", duplicate entries or group not found! please try again later.', 'experttexting-official'), $name)
                );
            }
        }
    }



    /**
     * Delete Group
     *
     * @param Not param
     *
     * @return false|int|void
     */
    public static function deleteGroup($id) {
        
        global $wpdb, $__Expt_BaseController;

        if (empty($id)) {
            return;
        }

        $result = $wpdb->delete(
            $wpdb->prefix . "et_subscribes_group",
            array(
                'ID' => $id,
            )
        );

        if ($result) {

            return $result;
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }
    }



    /**
     * Delete Group with Subscribers
     *
     * @param Not param
     *
     * @return false|int|void
     */
    public static function deleteGroupWithSubscribers($id) {
        
        global $wpdb, $__Expt_BaseController;

        if (empty($id)) {
            return;
        }

        $table   = $wpdb->prefix . 'et_subscribes';
        $prepare = $wpdb->prepare("SELECT * FROM {$table} WHERE `group_ID` = %s", $id);
        $query_id   = $wpdb->get_var($prepare);


        $result = $wpdb->delete(
            $wpdb->prefix . "et_subscribes_group",
            array(
                'ID' => $id,
            )
        );

        if ($result) {

            if(!$query_id) return $result;

            $result2 = $wpdb->delete(
                $wpdb->prefix . "et_subscribes",
                array(
                    'group_ID' => $id,
                )
            );

            if ($result2) {
                
                return $result2;
            } else {

                if( $wpdb->last_error ) {
                    $__Expt_BaseController->save_expt_wpdb_log();
                }
            }

        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }
    }


    /**
     * Add Subscriber
     *
     * @param $name
     * @param $mobile
     * @param string $group_id
     * @param string $status
     * @param null $key
     *
     * @return array
     */
    public static function addSubscriber($name, $mobile, $group_id = '', $status = '1', $key = null) {

        global $wpdb, $__Expt_BaseController;

        if (self::isDuplicate($mobile, $group_id)) {
            return array('result' => 'error', 'message' => __('The mobile number has been duplicate.', 'experttexting-official'));
        }

        if( !isset( $mobile ) || empty( $mobile ) ) {
            return array('result' => 'error', 'message' => __('The mobile number should not be empty.', 'experttexting-official'));
        }

        if( !isset( $name ) || empty( $name ) ) {
            return array('result' => 'error', 'message' => __('The Name should not be empty.', 'experttexting-official'));
        }

        $result = $wpdb->insert(
            $wpdb->prefix . "et_subscribes",
            array(
                'date'         => date('Y-m-d H:i:s', current_time('timestamp')),
                'name'         => $name,
                'mobile'       => $mobile,
                'status'       => $status,
                'activate_key' => $key,
                'group_ID'     => $group_id,
            )
        );

        if ($result) {
            /**
             * Run hook after adding subscribe.
             *
             * @param string $name name.
             * @param string $mobile mobile.
             *
             */
            do_action('expttxt_add_subscriber', $name, $mobile);

            return array('result' => 'success', 'message' => __('Subscriber successfully added.', 'experttexting-official'));
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

            return array('result' => 'error', 'message' => __('Having problem while adding subscriber, please try again later.', 'experttexting-official'));
        }
    }


    /**
     * Get Subscriber
     *
     * @param $id
     *
     * @return array|object|void|null
     */
    public static function getSubscriber($id) {

        global $wpdb, $__Expt_BaseController;
        $result = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}et_subscribes` WHERE ID = '" . $id . "'");

        if ($result) {

            return $result;
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }
    }


    /**
     * Update Subscriber
     *
     * @param $id
     * @param $name
     * @param $mobile
     * @param string $group_id
     * @param string $status
     *
     * @return array
     */
    public static function updateSubscriber($id, $name, $mobile, $group_id = '', $status = '1') {

        global $wpdb, $__Expt_BaseController;

        if (empty($id) or empty($name) or empty($mobile)) {
            return array('result' => 'error', 'message' => __('The fields must be valued.', 'experttexting-official'));
        }

        if (self::isDuplicate($mobile, $group_id, $id)) {
            return array('result' => 'error', 'message' => __('The mobile number has been duplicate.', 'experttexting-official'));
        }

        $result = $wpdb->update(
            $wpdb->prefix . "et_subscribes",
            array(
                'name'     => $name,
                'mobile'   => $mobile,
                'group_ID' => $group_id,
                'status'   => $status,
            ),
            array(
                'ID' => $id
            )
        );

        if ($result) {

            /**
             * Run hook after updating subscribe.
             *
             * @param string $result result query.
             *
             */

            $updateSub = array(
                'ID' => $id,
                'name' => $name,
                'mobile' => $mobile,
                'status' => $status,
                'group_id' => $group_id
            );

            do_action('expttxt_update_subscriber', $updateSub );

            return array('result' => 'success', 'message' => __('Subscriber successfully updated.', 'experttexting-official'));
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }

            return array('result' => 'error', 'message' => __('Having problem while updating subscriber, duplicate entries or subscriber not found! please try again.', 'experttexting-official'));
        }
    }


    /**
     * Check the mobile number is duplicate
     *
     * @param $mobile_number
     * @param null $group_id
     * @param null $id
     *
     * @return mixed
     */

    public static function isDuplicate($mobile_number, $group_id = null, $id = null) {

        global $wpdb, $__Expt_BaseController;
        $sql = "SELECT * FROM `{$wpdb->prefix}et_subscribes` WHERE mobile = '" . $mobile_number . "'";

        if ($group_id) {
            $sql .= " AND group_id = '" . $group_id . "'";
        }

        if ($id) {
            $sql .= " AND id != '" . $id . "'";
        }

        $result = $wpdb->get_row($sql);

        if($result) {

            return $result;
        } else {

            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

    }

}