<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api\Expt_Callbacks;

use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_ApiCallbacks;

class Expt_NotificationsCallbacks extends Expt_BaseController {

    public function register() {
        
        // When new post published
        add_action('add_meta_boxes', array($this, 'onNewPost_meta_box'));
        add_action('publish_post', array($this, 'onNewPost'), 10, 2);
        // When post approved
        add_action('pending_to_publish', array($this, 'onApprovedPost'), 10, 3);
        // When new user registered in Wordpress
        add_action('register_new_user', array($this, 'onUserRegister'), 10, 1);
        // When new comment post
        add_action('comment_post', array($this, 'onNewComment'), 10, 3);
        // When user login on Wordpress
        add_action('wp_login', array($this, 'onUserLogin'), 10, 2);
            
    }

    public function onNewPost_meta_box() {

        if ( ! $this->activated( 'et_notifications', 'on_new_post_chk' ) ) return;
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }
        
        add_meta_box(
            'et_subscribe_meta_box', __('ExpertTexting SMS Notification', 'experttexting-official'),
            array( $this, 'onNewPost_meta_box_handler'),
            get_post_types(['public' => true]), 'normal', 'high'
        );
    }

    public function onNewPost_meta_box_handler() {

        global $wpdb, $__Expt_BaseController;

        $groups = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}et_subscribes_group`");
        $subscribers  = $wpdb->query("SELECT * FROM {$wpdb->prefix}et_subscribes WHERE status = '1'");

        if(!$groups || !$subscribers) {
            
            if( $wpdb->last_error ) {
                $__Expt_BaseController->save_expt_wpdb_log();
            }
        }

        include_once $this->plugin_path . "ExptTemplates/et_newpost_metabox.php";
    }


    /**
     * When new post published
     * 
     * @since 1.0.0
     * 
     * @param $new_status
     * @param $old_status
     * @param $post
     */
    function onNewPost( $ID, $post ) {
        
        if ( ! $this->activated( 'et_notifications', 'on_new_post_chk' ) ) return;

        global $wpdb, $__Expt_ApiCallbacks;

        if ( isset($_REQUEST['et_send_subscribe']) && $_REQUEST['et_send_subscribe'] == 'yes' ) {

            $msg = sanitize_textarea_field( $_REQUEST['et_message_body'] );

            if ( !$msg ) return;

            $onNewPostArr = json_decode(json_encode( $post ), true);
            $message = $this->replace_text($onNewPostArr, $this->notiVariables('postVariables'), $msg);

            if ( $_REQUEST['et_subscribe_group'] == 'all' ) {

                $this->sendMessageReq( $message );

            } else {

                $numbers = $__Expt_ApiCallbacks->getNumberOfSubscribers( absint($_REQUEST['et_subscribe_group']) );
                if ( $numbers ) {
                    foreach($numbers as $num)  {
                        $recipients[] = $num->mobile;
                    }
                } else {
                    return;
                }

                $this->sendMessageReq( $message, $recipients);
            }
            
        }

    }

    /**
     * When post approved
     * 
     * @param $post
     */
    function onApprovedPost( $post ) {

        if ( ! $this->activated( 'et_notifications', 'author_post_chk' ) ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_notifications' ) );
        $postType = !empty($msgOption) ? $msgOption['author_post_types'] : '';
        $msg = !empty($msgOption) ? sanitize_textarea_field( $msgOption['author_post_body'] ) : '';

        if ( !$msg ) return;

        $pages = $posts = false;
        foreach ( $postType as $i ) {
            if ( $i == 'pages' ) { $pages = true; }
            if ( $i == 'posts' ) { $posts = true; }
        }

        if ( ($pages == true && $post->post_type == 'page') || ($posts == true && $post->post_type == 'post') ) {

            $onApprovedPostArr = json_decode(json_encode( $post ), true);
            $message = $this->replace_text($onApprovedPostArr, $this->notiVariables('postVariables'), $msg);
            // Post Author's Number
            $postAuthor = $post->post_author;
            $postAuthor = get_user_meta($postAuthor);

            if ( $postAuthor['expt_mobile'] ) {

                $this->sendMessageReq( $message, $postAuthor['expt_mobile'] );
            }

        } else {
            return;
        }

    }

    /**
     * When new user registered in Wordpress
     * 
     * @param $user_id
     */
    function onUserRegister( $user_id ) {

        if ( ! $this->activated( 'et_notifications', 'new_register_chk' ) ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_notifications' ) );
        $msg = !empty($msgOption) ? $msgOption['new_register_admin_body'] : '';
        $msgUser = !empty($msgOption) ? sanitize_textarea_field( $msgOption['new_register_user_body'] ) : '';

        if ( !$msg || !$msgUser ) return;

        $userData = get_userdata( $user_id );
        $userMobile = get_user_meta( $user_id );
        
        $onUserRegisterArr = json_decode(json_encode( $userData ), true);
        $onUserRegisterArr = $onUserRegisterArr['data'];
        $message = $this->replace_text($onUserRegisterArr, $this->notiVariables('userRegisterVariables'), $msg);
        $messageUser = $this->replace_text($onUserRegisterArr, $this->notiVariables('userRegisterVariables'), $msgUser);

        if ( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $this->sendMessageReq( $message, array($this->adminNumber) );
        }
        if ( $userMobile['expt_mobile'] ) {

            $this->sendMessageReq( $messageUser, $userMobile['expt_mobile'] );
        }

    }

    /**
     * When new comment post
     * 
     * @param $comment
     */
    function onNewComment( $comment ) {

        if ( ! $this->activated( 'et_notifications', 'new_comment_chk' ) ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_notifications' ) );
        $msg = !empty($msgOption) ? sanitize_textarea_field( $msgOption['new_comment_body'] ) : '';

        if ( !$msg ) return;

        $commentData = get_comment( $comment );
        
        $onNewCommentArr = json_decode(json_encode( $commentData ), true);
        $message = $this->replace_text($onNewCommentArr, $this->notiVariables('commentVariables'), $msg);

        if ( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $this->sendMessageReq( $message, array($this->adminNumber) );
        }
        
    }

    /**
     * When user login on Wordpress
     * 
     * @param $user_login
     * @param $user
     */
    function onUserLogin( $user_login, $user ) {

        if ( ! $this->activated( 'et_notifications', 'user_login_chk' ) ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_notifications' ) );
        $msg = !empty($msgOption) ? sanitize_textarea_field( $msgOption['user_login_body'] ) : '';

        if ( !$msg ) return;

        $onUserLoginArr = json_decode(json_encode( $user ), true);
        $onUserLoginArr = $onUserLoginArr['data'];
        $message = $this->replace_text($onUserLoginArr, $this->notiVariables('userLoginVariables'), $msg);

        if ( $this->adminNumber && is_numeric($this->adminNumber) ) {

            $this->sendMessageReq( $message, array($this->adminNumber) );
        }

    }

    /**
     * When user subscribes newsletter, welcome message
     * 
     * @param $subscriber
     */
    function onUserSubscribesNewsletter( $subscriber ) {

        if ( ! $this->activated( 'et_newsletter', 'newsletter_welcome_chk' ) ) return;

        $msgOption = $this->expt_escape_array_data( get_option( 'et_newsletter' ) );
        $msg = !empty($msgOption) ? sanitize_textarea_field( $msgOption['newsletter_welcome_body'] ) : '';

        if ( !$msg ) return;

        $subscriberArr = json_decode(json_encode( $subscriber ), true);

        $message = $this->replace_text($subscriberArr, $this->notiVariables('welcomeMessageVariables'), $msg);

        if ( $subscriberArr['mobile'] ) {

            $this->sendMessageReq( $message, array($subscriberArr['mobile']) );
        }

    }

    /**
     * When user login with mobile
     * 
     * @param $code, $user_mobile, $userdata
     */
    function etLoginWithMobileCallback( $code, $user_mobile, $userdata, $option_name ) {

        $msgOption = $this->expt_escape_array_data( get_option( 'et_menu' ) );
        $msg = !empty($msgOption) ? sanitize_textarea_field( $msgOption[$option_name] ) : '';

        if ( !$msg ) return;

        $userDataArr = json_decode(json_encode( $userdata ), true);
        $userDataArr = $userDataArr['data'];

        $userDataArr['user_mobile'] = $user_mobile;
        $userDataArr['user_code'] = $code;
        $userDataArr['site_name'] = get_bloginfo('name');
        $userDataArr['site_url'] = get_site_url();

        $message = $this->replace_text($userDataArr, $this->notiVariables('loginWithMobileVariables'), $msg);

        if ( $user_mobile ) {

            $this->sendMessageReq( $message, array($user_mobile) );
        }

    }

    /**
     * Notification variables
     * 
     * @param $arg
     * 
     * @return array list
     */
    public function notiVariables( $arg ) {

        if ( $arg == 'postVariables' ) {
            return array(
                'post_title' => 'post_title',
                'post_content' => 'post_content',
                'post_url' => 'guid',
                'post_date' => 'post_date'
            );
        } elseif ( $arg == 'userRegisterVariables' ) {
            return array(
                'user_login' => 'user_login',
                'user_email' => 'user_email',
                'date_register' => 'user_registered'
            );
        } elseif ( $arg == 'commentVariables' ) {
            return array(
                'comment_author' => 'comment_author',
                'comment_author_email' => 'comment_author_email',
                'comment_author_url' => 'comment_author_url',
                'comment_author_IP' => 'comment_author_IP',
                'comment_date' => 'comment_date',
                'comment_content' => 'comment_content',
                'comment_url' => 'comment_ID'
            );
        } elseif ( $arg == 'userLoginVariables' ) {
            return array(
                'username_login' => 'user_login',
                'display_name' => 'display_name',
                'user_email' => 'user_email',
            );
        } elseif ( $arg == 'welcomeMessageVariables' ) {
            return array(
                'subscribe_name' => 'name',
                'subscribe_mobile' => 'mobile',
            );
        } elseif ( $arg == 'loginWithMobileVariables' ) {
            return array(
                'user_login' => 'user_login',
                'user_email' => 'user_email',
                'user_display_name' => 'display_name',
                'user_mobile' => 'user_mobile',
                'user_code' => 'user_code',
                'site_name' => 'site_name',
                'site_url' => 'site_url',
            );
        }
    }

    /**
     * Replace user's message body Snippet Text
     * 
     * @param $post
     * @param $obj
     * @param $msg
     * 
     * return Clean Message with Replace short codes
     */
    public function replace_text( $post, $obj, $msg ) {

        $postContentCountOption = $this->expt_escape_array_data( get_option( 'et_notifications' ) );
        $postContentCountOption = isset( $postContentCountOption['post_content_count'] ) ? $postContentCountOption['post_content_count'] : '';

        $postContentCount = !empty($postContentCountOption) && $postContentCountOption > 0 ? $postContentCountOption : 10;

        foreach ($obj as $key => $value) {

            if ( $key == 'post_content' ) {
                $post_content = strip_tags($post[$value]);
                $post_content_crop = wp_trim_words( $post_content, $postContentCount, ' [...]' );
                $msg = str_replace( "[" . $key . "]", $post_content_crop, $msg );

            } elseif ( $key == 'comment_url' ) {
                $postId = get_post( $post['comment_post_ID'] );
                $commentURL = json_decode(json_encode( $postId ), true);
                $msg = str_replace( "[" . $key . "]", $commentURL['guid'] . '#comment-' . $post[$value], $msg );

            } else {
                $msg = str_replace( "[" . $key . "]", $post[$value], $msg );
            }
        }

        $msg = strip_tags($msg);
        return $msg;
    }


    /**
     * Send message request
     * 
     * @param $message
     * @param $mobile
     * 
     */
    public function sendMessageReq( $message, $mobile = '' ) {

        global $__Expt_ApiCallbacks;
        
        if ( !$mobile ) {
            
            // Get all subscriber's numbers
            $allSubscribers = $__Expt_ApiCallbacks->getNumberOfSubscribers();
            $allSubscribersArr = json_decode(json_encode( $allSubscribers ), true);
            
            foreach ( $allSubscribersArr as $mobileNo ) {
                $mobileArr[] = $mobileNo['mobile'];
            }
            
        } else {
            $mobileArr = $mobile;
        }
        
        $et_send_msg = array(
            'et_send_from' => $this->apiSender,
            'et_send_to' => $mobileArr,
            'et_send_msg' => $message,
            'et_send_check_unicode' => 'false'
        );
        
        update_option( "et_send_sms", $et_send_msg, "yes" );

        $res = $__Expt_ApiCallbacks->sendMessage();

        if( $res ) {

            $this->save_expt_log( $res, $et_send_msg );

            ob_clean();
            return;
        }
    }

}
