<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Api;
use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_ApiCallbacks;

class Expt_SettingsApi {

    public $admin_pages = array();
    public $admin_subpages = array();

    public $settings = array();
    public $sections = array();
    public $fields = array();

    public function register() {
        if ( !empty($this->admin_pages) ) {
            add_action( 'admin_menu', array( $this, 'add_admin_menu' ) );
        }

        if ( !empty($this->settings) ) {
            add_action( 'admin_init', array( $this, 'registerCustomFields' ) );
            add_action( 'admin_init', array( $this, 'apiAuth' ) );
        }

    }

    /**
     * Checks for an integration if a WordPress plugin is installed.
     *
     * @param string $pluginTitle
     * @return string/boolean
     */
    public static function is_plugin_installed( $pluginTitle ) {

        global $__Expt_Notices;
        // get all the plugins
        $installedPlugins = get_plugins();
        foreach ($installedPlugins as $installedPlugin => $data) {
            // check for the plugin title
            if ($data['Title'] == $pluginTitle) {
                /**
                 * Detect if the plugin is active.
                 */
                if ( is_plugin_active( $installedPlugin ) ) {
                    return 'installed'; 
                } else {
                    return 'inactive-plugin';
                }
            }
        }

        return false;
    }
    
    public static function apiAuth() {

        global $__Expt_BaseController, $__Expt_ApiCallbacks, $__Expt_Notices;

        $username = $__Expt_BaseController->apiUsername;
        $apikey = $__Expt_BaseController->apikey;
        $apisecret = $__Expt_BaseController->apisecret;

        if( !get_option( 'etApiAuth' ) ) {
            add_option( "etApiAuth", "", "", "yes" );
            add_option( "et_balance", "", "", "yes" );
            add_option( "et_balance_datetime", "", "", "yes" );
            add_option( "etApiAuthHidden", "", "", "yes" );
        }

        if( !empty( get_option( 'etApiAuthHidden' ) ) ) {

            if ( !$__Expt_BaseController->apiValues || ( !$__Expt_BaseController->apiUsername && !$__Expt_BaseController->apikey && !$__Expt_BaseController->apisecret ) ) {
                update_option( "etApiAuth", "", "yes" );
                update_option( "et_balance", "", "yes" );
                update_option( "etApiAuthHidden", "", "yes" );
                return;
            }

            $__Expt_ApiCallbacks->username = $username;
            $__Expt_ApiCallbacks->apikey = $apikey;
            $__Expt_ApiCallbacks->apisecret = $apisecret;

            $result = $__Expt_ApiCallbacks->QueryBalance();

            update_option( "et_balance_datetime", date('Y-m-d H:i:s'), "yes" );
			
			if ( is_wp_error( $result ) ) {
				$res = $result;

			} else {
				$res = json_decode($result, true);
			}
            
            if (gettype($res) == "array" && !is_wp_error( $res ) ) {
            
                if ($res["Status"] == 0) {
                    update_option( "etApiAuth", "", "yes" );
                    update_option( "et_balance", $res["Response"]["Balance"], "yes" );

                } else {
                    update_option( "etApiAuth", $res["ErrorMessage"], "yes" );
                    update_option( "et_balance", "", "yes" );

                    $__Expt_Notices->api_notice();
                    // Generate Expt Log
                    $__Expt_BaseController->save_expt_log( ['Api Auth Failed', $res["ErrorMessage"]], $res );
                }

                update_option( "etApiAuthHidden", "", "yes" );
                
            } else {
                $__Expt_Notices->notice( (!empty($res) ? esc_attr($res) : '') . __(' Please try again later. <br>If the issue still persist, contact your hosting provider OR ExpertTexting support.', 'experttexting-official'), "error" );
                update_option( "etApiAuthHidden", "", "yes" );
                $__Expt_Notices->api_notice_issue();
                // Generate Expt Log
                $__Expt_BaseController->save_expt_log( ['Api Response Error/Failed', 'An error occurred while configuring API.'], (!empty($res) ? esc_attr($res) : '') );
            }

        }
    }

    public static function sendMsg() {

        global $__Expt_BaseController, $__Expt_ApiCallbacks, $__Expt_Notices;

        $checkAuth = esc_attr( get_option( 'etApiAuth' ) );
        $balance = esc_attr( get_option( 'et_balance' ) );
        $msgType = 'text';

        if ( $checkAuth != "" || ( $checkAuth == "" && !$balance ) || !$__Expt_BaseController->apiValues ) {
            
            return 'api_error_notice';
        }

        $msgNumbers = array();

        $resendReq = $__Expt_BaseController->expt_escape_array_data( get_option( 'resend_msg_request' ) );
        $sendForm = $__Expt_BaseController->expt_escape_array_data( get_option( 'et_send_sms' ) );

        $username = $__Expt_BaseController->apiUsername;
        $apikey = $__Expt_BaseController->apikey;
        $apisecret = $__Expt_BaseController->apisecret;

        $from = $sendForm['et_send_from'];
        $selection = isset( $sendForm['et_send_to_selection'][0] ) ? esc_attr( $sendForm['et_send_to_selection'][0] ) : '';

        if ( !$resendReq ) {

            if ( $selection == "subscriber_users" ) {

                $groupID = absint($sendForm['et_send_to'][0]);
                $numbers = $__Expt_ApiCallbacks->getNumberOfSubscribers( $groupID );

                if ( $numbers ) {
                    foreach($numbers as $num)  {
                        
                        $msgNumbers[] = $num->mobile;
                    }
                } else {

                    return 'empty_list';
                }
                
            }
            if ( $selection == "wordpress_users" ) {

                $numbers = $__Expt_ApiCallbacks->getNumberOfWordpressUsers();

                if ( $numbers ) {
                    foreach($numbers as $num)  {
                        
                        !empty($num->meta_value) ? $msgNumbers[] = $num->meta_value : "";
                    }
                } else {

                    return 'empty_list';
                }
                
            }
            if ( $selection == "add_numbers" || empty($selection) ) {

                if( isset($sendForm['et_send_to']) ) {
                    
                    $msgNumbers = $sendForm['et_send_to'];
                    $msgNumbers = array_filter($msgNumbers);
                    foreach ($msgNumbers as $to) {
                        if( !is_numeric($to) ) {

                            return 'invalid_numbers';
                        }
                    }

                }
                
            }

        } else {

            if( isset($sendForm['et_send_to']) ) {
                    
                $msgNumbers = $sendForm['et_send_to'];
                $msgNumbers = array_filter($msgNumbers);
                foreach ($msgNumbers as $to) {
                    if( !is_numeric($to) ) {

                        return 'invalid_numbers';
                    }
                }

            }
        }


        $txtMsg = $sendForm['et_send_msg'];
        $checkUnicode = $sendForm['et_send_check_unicode'];

        if ( $__Expt_BaseController->activated( 'et_menu', 'send_unicode' ) || $checkUnicode == "true" ) {
            $msgType = 'unicode';
        }

        $msgStatus = array();
        $message_id_array = array();
        $successRate = 0;
        $failedRate = 0;

        /**
         * Modify sender number
         *
         * @param string $from
         * @since 1.1.0
         */
        $from = apply_filters( 'expttxt_from', $from );

        /**
         * Modify receiver number
         *
         * @param array $msgNumbers
         * @since 1.1.0
         */
        $msgNumbers = apply_filters( 'expttxt_to', $msgNumbers );

        /**
         * Modify message
         *
         * @param string $txtMsg
         * @since 1.1.0
         */
        $txtMsg = apply_filters( 'expttxt_msgtext', $txtMsg );
        
        $__Expt_ApiCallbacks->username = $username;
        $__Expt_ApiCallbacks->apikey = $apikey;
        $__Expt_ApiCallbacks->apisecret = $apisecret;
        $__Expt_ApiCallbacks->from = $from;
        $__Expt_ApiCallbacks->msgtext = $txtMsg;
        $__Expt_ApiCallbacks->type = $msgType;

        if ( !$msgNumbers ) {
            return 'empty_number';
        } elseif ( !$txtMsg ) {
            return 'empty_message';
        } elseif ( !$from ) {
            return 'empty_sender';
        } else {
            update_option( "et_send_sms", "", "yes" );
        }

        $message_count = '';
        
        foreach ($msgNumbers as $to) {

            $__Expt_ApiCallbacks->to = $to;
            
            $res = json_decode($__Expt_ApiCallbacks->send(), true);
            
            if (gettype($res) == "array" ) {

                if ($res["Status"] == 0) {
                    
                    $message_id_array[] = [ $to, esc_attr( $res["Response"]["message_id"] ) ];
                    $message_count = esc_attr( $res["Response"]["message_count"] );

                    $price = (float)esc_attr( $res["Response"]["price"] );
                    $balance = (float)esc_attr( get_option( 'et_balance' ) );
                    $remainCredit = $balance - $price;

                    $credit = number_format( (float)$remainCredit, 4, '.', '' );
                    $bal = number_format( (float)$balance, 4, '.', '' );

                    $updateCredit = $credit > 0 ? $credit : $bal;
                    update_option( "et_balance", esc_attr( $updateCredit ), "yes" );

                    ++$successRate;
                    
                } else {

                    $msgStatus[$to] = esc_attr( $res["ErrorMessage"] );
                    $statStr = esc_attr( $res["ErrorMessage"] );
                    ++$failedRate;
                }

            } else {

                return 'final_error';
            }
        }

        $msgStatusStr = '';
        if ( empty($msgStatus) ) {
            $msgStatusStr = "All Sent";
        } else {
            foreach($msgStatus as $key => $val) {
                $msgStatusStr .= $key . ': ' . $val . ', ';
            }
        }

        
        /**
         * For update previous entry when resend message.
         * Modify value of resendReqChk
         * 
         * Default value: false
         *
         * @param bool|string $resendReqChk
         * @since 1.1.0
         */
        $resendReqChk = false;
        $resendReqChk = apply_filters( 'expttxt_resend_update_prev_values', $resendReqChk );
        
        $lastMsgID = $__Expt_ApiCallbacks->sendLog($from, $txtMsg, implode(", ", $msgNumbers) , $msgType, $msgStatusStr, (!empty($statStr) ? $statStr : "Successful"), $successRate, $failedRate, $message_count, $message_id_array, $resendReq, $resendReqChk ); 

        if ( !empty($selection) || !empty( get_option( 'resend_msg_request' ) ) ) {

            if( $successRate > 0 ) {
                $__Expt_Notices->notice( sprintf(__( 'Message Sent. Success: %s & Failed: %s ', 'experttexting-official' ), $successRate, $failedRate) . ' | <a href="' . esc_attr( admin_url('admin.php?page=et_subscribers_outbox&sendmsgID=' . absint($lastMsgID) ) ) . '">' . __( 'View Message', 'experttexting-official' ) . '</a>', "success" );
            } else {
                $__Expt_Notices->notice( __( 'Message Sending Failed.', 'experttexting-official' ) . ' | <a href="' . esc_attr( admin_url('admin.php?page=et_subscribers_outbox&sendmsgID=' . absint($lastMsgID) ) ) . '">' . __( 'View Message', 'experttexting-official' ) . '</a>' , "error" );
            }

        }

        update_option( "resend_msg_request", "", "yes" );

        return;
    }


    public function addPages( array $pages ) {
        $this->admin_pages = $pages;

        return $this;
    }

    public function withSubPage( String $title = null ) {
        if ( empty($this->admin_pages) ) {
            return $this;
        }

        $admin_page = $this->admin_pages[0];

        $subpage = array( 
            array(
                'parent_slug' => $admin_page['menu_slug'],
                'page_title' => $admin_page['page_title'],
                'menu_title' => ($title) ? $title : $admin_page['menu_title'],
                'capability' => $admin_page['capability'],
                'menu_slug' => $admin_page['menu_slug'],
                'callback' => $admin_page['callback']
            )
        );

        $this->admin_subpages = $subpage;

        return $this;
    }

    public function addSubPages( array $pages ) {
        $this->admin_subpages = array_merge( $this->admin_subpages, $pages ) ;

        return $this;
    }

    public function add_admin_menu() {

        foreach ($this->admin_pages as $page) {
            add_menu_page( $page['page_title'], $page['menu_title'], $page['capability'], $page['menu_slug'], $page['callback'], $page['icon_url'], $page['position'] );
        }

        foreach ($this->admin_subpages as $page) {
            add_submenu_page( $page['parent_slug'], $page['page_title'], $page['menu_title'], $page['capability'], $page['menu_slug'], $page['callback'] );
        }
    }

    public function setSettings( array $settings ) {
        $this->settings = $settings;

        return $this;
    }

    public function setSections( array $sections ) {
        $this->sections = $sections;

        return $this;
    }

    public function setFields( array $fields ) {
        $this->fields = $fields;

        return $this;
    }

    public function registerCustomFields() {

        // Register Settings
        foreach ($this->settings as $setting) {
            register_setting( $setting['option_group'], $setting['option_name'], ( isset( $setting['callback'] ) ? $setting['callback'] : '' ) );
        }

        // Add Settings Section
        foreach ($this->sections as $section) {
            add_settings_section( $section['id'], $section["title"], ( isset( $section['callback'] ) ? $section['callback'] : '' ), $section['page'] );
        }
        
        // Add Settings Field
        foreach ($this->fields as $field) {
            add_settings_field( $field['id'], $field['title'], ( isset( $field['callback'] ) ? $field['callback'] : '' ), $field['page'], $field['section'], ( isset( $field['args'] ) ? $field['args'] : '' ) );
        }
        
    }

}
