<?php 
/**
 * @package  ExpertTextingOfficial
 */
namespace ExptTxt\Expt_Api\Expt_Widgets;

use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_NewsletterCallbacks;
use WP_Widget;

/**
* 
*/
class Expt_Widget extends WP_Widget {

	public $widget_ID;
	public $widget_name;
	public $widget_desc;
	public $widget_options = array();
	public $control_options = array();

	function __construct() {

		$this->widget_ID = 'et_widget';
		$this->widget_name = __( 'ExpertTexting SMS Newsletter', 'experttexting-official' );
		$this->widget_desc = __( 'ExpertTexting SMS Newsletter Form', 'experttexting-official' );

		$this->widget_options = array(
			'classname' => $this->widget_ID,
			'name' => $this->widget_name,
			'description' => $this->widget_desc,
			'customize_selective_refresh' => true,
		);

		$this->control_options = array(
			'width' => 400,
			'height' => 550,
		);

	}

	public function register() {
		
		parent::__construct( $this->widget_ID, $this->widget_name, $this->widget_options, $this->control_options );

		add_action( 'widgets_init', array( $this, 'widgetsInit' ) );

		return $this->widget_ID;
	}

	public function widgetsInit() {

		register_widget( $this );
	}

	public function widget( $args, $instance ) {

		_e( $args['before_widget'] );
		
		self::subscriber_form( '', $args, $instance );

		_e( $args['after_widget'] );
	}

	public function form( $instance ) {
		$title = ! empty( $instance['title'] ) ? $instance['title'] : esc_html__( 'Subscribe to SMS Notifications', 'experttexting-official' );
		$desc = ! empty( $instance['desc'] ) ? $instance['desc'] : esc_html__( 'Subscribe to our newsletter to receive the latest posts, updates, news, and much more right on your mobile through SMS.', 'experttexting-official' );
		?>
		<p>
			<label for="<?php esc_attr_e( $this->get_field_id( 'title' ) ); ?>"><?php esc_attr_e( 'Title:', 'experttexting-official' ); ?></label> 
			<input class="widefat" id="<?php esc_attr_e( $this->get_field_id( 'title' ) ); ?>" name="<?php esc_attr_e( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php esc_attr_e( $title ); ?>">
		</p>
		<p>
			<label for="<?php esc_attr_e( $this->get_field_id( 'desc' ) ); ?>"><?php esc_attr_e( 'Description:', 'experttexting-official' ); ?></label> 
			<textarea class="widefat" id="<?php esc_attr_e( $this->get_field_id( 'desc' ) ); ?>" name="<?php esc_attr_e( $this->get_field_name( 'desc' ) ); ?>" rows="4" cols="50"><?php esc_attr_e( $desc ); ?></textarea>
		</p>
		<?php 
	}

	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = sanitize_text_field( $new_instance['title'] );
		$instance['desc'] = ! empty( $new_instance['desc'] ) ? $new_instance['desc'] : '';

		return $instance;
	}


	public static function subscriber_form( $sc = '', $args = '', $instance = '' ) {

		global $__Expt_BaseController;

		$wid = ! empty($args['widget_id']) ? $args['widget_id'] : "";

		if ( ! empty( $instance['title'] ) ) {
			_e( ($args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title']) );
		}
		if ( ! empty( $instance['desc'] ) ) {
			_e( ('<p class="et_widget_desc">' . $instance['desc'] . '</p>') );
		}
		if ( ! empty( $sc['title'] ) ) {
			_e( ('<h2 class="widget-title">' . $sc['title'] . '</h2>') );
		}
		if ( ! empty( $sc['desc'] ) ) {
			_e( ('<p class="et_widget_desc">' . $sc['desc'] . '</p>') );
		}
		?>

		<!-- Status -->
		<div class="et_sms_form_status" id="et_sms_form_status_<?php esc_attr_e( $wid ); ?>">
			<small class="field-msg js-form-submission"><?php _e( 'Subscribing in process, please wait...', 'experttexting-official' ); ?></small>
			<small class="field-msg js-form-submission-activation"><?php _e( 'Activation in process, please wait...', 'experttexting-official' ); ?></small>
			<small class="field-msg success js-form-success"><?php _e( 'You are successfully subscribed, thank you!', 'experttexting-official' ); ?></small>
			<small class="field-msg success js-form-success-unsub"><?php _e( 'You are successfully unsubscribed!', 'experttexting-official' ); ?></small>
			<small class="field-msg error js-form-error"><?php _e( 'There was a problem with the Subcription Form, please try again!', 'experttexting-official' ); ?></small>
			<small class="field-msg error js-form-exist"><?php _e( 'The Mobile Number is already registered.', 'experttexting-official' ); ?></small>
			<small class="field-msg error js-form-notfound"><?php _e( 'The Mobile Number is not found.', 'experttexting-official' ); ?></small>
			<small class="field-msg error js-form-active-code"><?php _e( 'The Activation Code is not valid.', 'experttexting-official' ); ?></small>
			<small class="field-msg error js-form-min-error"><?php _e( sprintf( __( 'Your mobile number should be minimum of %s digits', 'experttexting-official' ), (!empty( get_option( 'et_newsletter' ) ) ? esc_attr( get_option( 'et_newsletter' )['newsletter_mobile_min'] ) : '') ) ); ?></small>
			<small class="field-msg error js-form-max-error"><?php _e( sprintf( __( 'Your mobile number should be maximum of %s digits', 'experttexting-official' ), (!empty( get_option( 'et_newsletter' ) ) ? esc_attr( get_option( 'et_newsletter' )['newsletter_mobile_max'] ) : '') ) ); ?></small>
			<small class="field-msg error js-form-invalid-number"><?php _e( 'The entered mobile number is invalid', 'experttexting-official' ); ?></small>
		</div>

		<!-- Sub -->
		<form class="et_sms_form" action="#" name="et_sms_form_<?php esc_attr_e( $wid ); ?>" id="et_sms_form_<?php esc_attr_e( $wid ); ?>" method="post" data-url="<?php esc_attr_e( admin_url('admin-ajax.php') ); ?>">
			<fieldset>
				<div>
					<label for="et_sms_name"><?php esc_attr_e( 'Your Name', 'experttexting-official' ); ?></label> 
					<small class="field-msg error validate" data-error="invalidName"><?php _e( 'Your Name is Required', 'experttexting-official' ); ?></small>
					<input class="et_widget_input alphaCharOnly" name="et_sms_name" id="et_sms_name" type="text" placeholder="Enter your Name" maxlength="50">
				</div>
				<?php $placeholder = !empty( get_option( 'et_newsletter' ) ) ? esc_attr( get_option( 'et_newsletter' )['newsletter_mobile_field_placeholder'] ) : ''; ?>
				<div>
					<label for="et_sms_mob"><?php esc_attr_e( 'Your Mobile', 'experttexting-official' ); ?></label> 
					<small class="field-msg error validate" data-error="invalidMobile"><?php _e( 'Your Mobile Number is Required', 'experttexting-official' ); ?></small>
					<input class="et_widget_input expttxt-intl-input" name="et_sms_mob" id="et_sms_mob" type="tel" placeholder="<?php _e( ($placeholder ? $placeholder : __( 'Enter your Mobile Number', 'experttexting-official' )) ) ?>" data-maxlength="20" oninput="this.value=this.value.slice(0,this.dataset.maxlength)">
				</div>
				<?php if ( $__Expt_BaseController->activated( 'et_menu', 'show_groups' ) ) {
					
					$groups = Expt_NewsletterCallbacks::getGroups();
					if ( $groups ): ?>
					<div>
						<label for="et_sms_group"><?php _e( 'Group', 'experttexting-official' ); ?></label> 
						<select class="et_widget_input" name="et_sms_group" class="form-control">
							<?php foreach ($groups as $items): ?>
								<option value="<?php esc_attr_e( absint($items->ID) ); ?>"><?php esc_attr_e($items->name); ?></option>
							<?php endforeach; ?>
						</select>
					</div>
					<?php endif; ?>

					<?php } ?>

				<div>
					<small class="field-msg error validate" data-error="invalidSub"><?php _e( 'Please choose one', 'experttexting-official' ); ?></small>
					<input type="radio" name="et_sms_subscribers" id="et_sms_sub" value="subscribe" checked="checked">
					<label for="et_sms_sub"><?php _e( 'Subscribe', 'experttexting-official' ); ?></label>
					<input type="radio" name="et_sms_subscribers" id="et_sms_unsub" value="unsubscribe">
					<label for="et_sms_unsub"><?php _e( 'Unsubscribe', 'experttexting-official' ); ?></label>
				</div>
				<?php
					if ( $__Expt_BaseController->activated( 'et_menu', 'show_newsletter_privacy' ) ) {
					$descOption = get_option( 'et_menu' );
        			$desc = !empty($descOption) ? wp_kses( html_entity_decode( $descOption['newsletter_privacy_body'] ), $__Expt_BaseController->allowed_html ) : '';
				?>
				<div style="line-height: normal;">
					<small class="field-msg error validate" data-error="invalidAgreeCheck"><?php _e( 'Please agree to our Privacy Policy', 'experttexting-official' ); ?></small>
					<input type="checkbox" name="et_sms_agree_chkbox" id="et_sms_agree_chkbox" value="1">
					<label for="et_sms_agree_chkbox" style="font-size: 14px;"><?php ( isset( $desc ) && !empty( $desc ) ? _e( $desc ) : _e( 'By checking this, you agree with the storage and handling of your data by this website. Your phone number will be used to send you notifications and updates via SMS.', 'experttexting-official' ) ); ?></label>
				</div>
				<?php } ?>
				<div>
					<input class="et_sms_btn_submit" type="submit" name="et_sms_submit" value="<?php _e( 'Subscribe', 'experttexting-official' ); ?>">
				</div>
				
					<input type="hidden" name="action" value="handle_subscribe_form">
					<input type="hidden" name="nonce" value="<?php _e( wp_create_nonce("et-subscribe-nonce") ) ?>">
				
			</fieldset>
		</form>
		<!-- Activation -->
		<form class="et_sms_form hide" action="#" name="et_sms_form_activation_<?php esc_attr_e( $wid ); ?>" id="et_sms_form_activation_<?php esc_attr_e( $wid ); ?>" method="post" data-url="<?php esc_attr_e( admin_url('admin-ajax.php') ); ?>">
			<fieldset>
				<hr>
				<div><?php _e( 'Thank you for joining SMS newsletter & notifications. To complete the joining process, please enter the activation code that has been sent to your number.', 'experttexting-official' ); ?></div>
				<br>
				<hr>
				<div>
					<label for="et_sms_name"><?php _e( 'Activation Code', 'experttexting-official' ); ?></label> 
					<small class="field-msg error validate" data-error="invalidActivation"><?php _e( 'Activation Code is Required', 'experttexting-official' ); ?></small>
					<input class="et_widget_input" name="et_sms_activation_code" type="text" placeholder="<?php _e( 'Enter the Activation Code', 'experttexting-official' ); ?>" maxlength="6">
				</div>
				<div>
					<input class="et_sms_btn_submit" type="submit" name="et_sms_submit_activation" value="<?php _e( 'Activate', 'experttexting-official' ); ?>">
				</div>

					<input type="hidden" name="action" value="handle_subscribe_form_activation">
					<input type="hidden" name="nonce" value="<?php _e( wp_create_nonce("et-subscribe-active-nonce") ) ?>">
				
			</fieldset>
		</form>

		<?php

	}

}

