<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Base;

use ExptTxt\Expt_Base\Expt_BaseController;

class Expt_Activate {

    public static function activate() {

        flush_rewrite_rules();

        $__Expt_BaseController = new Expt_BaseController();
        $get_plugin_version = $__Expt_BaseController->plugin_version;

        self::table_sql();

        add_action ( 'wp_head', 'javascript_variables' );

        $default = array();

        if ( !get_option( 'et_menu' ) ) {
            update_option( 'et_menu', array(
                'login_with_mobile_body' => 'Your login verification code is [user_code]',
                'login_2FA_body' => 'Your login verification code is [user_code]'
                 ) );
            
        } else {
            $et_menu = get_option( 'et_menu' );
            empty( $et_menu['login_with_mobile_body'] ) ? $et_menu['login_with_mobile_body'] = 'Your login verification code is [user_code]' : '';
            empty( $et_menu['login_2FA_body'] ) ? $et_menu['login_2FA_body'] = 'Your login verification code is [user_code]' : '';
            update_option( 'et_menu', $et_menu );
        }
            
        if ( !get_option( 'et_api_set' ) ) {
            update_option( 'et_api_set', $default );
        }
            
        if ( !get_option( 'et_send_sms' ) ) {
            update_option( 'et_send_sms', $default );
        }

        if ( !get_option( 'et_plugin_db_version' ) ) {
            add_option('et_plugin_db_version', $get_plugin_version);
        }

    }

    public function upgrade_inner_plugin() {

        add_action('plugins_loaded', array($this, 'upgrade_et_plugin'));
    }

    public function activated_display() {

        add_action('admin_menu', array($this, 'welcome_screen_page'));
        add_action('activated_plugin', array($this, 'welcome_redirect'));
        add_action('init', array($this, 'welcome_redirect2'));
        add_action('admin_head', array($this, 'remove_menu_entry'));
    }

    public function welcome_screen_page() {

        add_dashboard_page('Welcome to ExpertTexting Official', 'Welcome', 'read', 'et-plugin-welcome', array($this, 'welcome_page'));
    }
    
    public function welcome_page() {
        
        $__Expt_BaseController = new Expt_BaseController();
        return require_once( $__Expt_BaseController->plugin_path . "ExptTemplates/welcome.php" );
    }

    public function welcome_redirect( $plugin ) {

        if( $plugin == 'experttexting-official/experttextingofficial.php' ) {
            delete_option( 'et_plugin_update_redirect_to' );
            wp_redirect( admin_url('index.php?page=et-plugin-welcome') );
            die();
        }
    }

    public function welcome_redirect2() {

        if( get_option( 'et_plugin_update_redirect_to' ) ) {
            delete_option( 'et_plugin_update_redirect_to' );
            wp_redirect( admin_url('index.php?page=et-plugin-welcome') );
            die();
        }
    }

    public function remove_menu_entry() {
        
        remove_submenu_page( 'index.php', 'et-plugin-welcome' );
    }

    public static function table_sql() {
        global $wpdb;
        require_once(dirname( __FILE__, 6 ) . '/wp-admin/includes/upgrade.php');

        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . 'et_subscribes';
        if ($wpdb->get_var("show tables like '{$table_name}'") != $table_name) {
            $create_sms_subscribes = ("CREATE TABLE IF NOT EXISTS {$table_name}(
            `ID` int(10) NOT NULL auto_increment,
            `date` DATETIME,
            `name` VARCHAR(250),
            `mobile` VARCHAR(20) NOT NULL,
            `status` tinyint(1),
            `activate_key` TEXT,
            `group_ID` int(5),
            PRIMARY KEY(`ID`)) $charset_collate;");

            dbDelta($create_sms_subscribes);
        }

        $table_name = $wpdb->prefix . 'et_subscribes_group';
        if ($wpdb->get_var("show tables like '{$table_name}'") != $table_name) {
            $create_sms_subscribes_group = ("CREATE TABLE IF NOT EXISTS {$table_name}(
            `ID` int(10) NOT NULL auto_increment,
            `name` VARCHAR(250),
            PRIMARY KEY(`ID`)) $charset_collate");

            dbDelta($create_sms_subscribes_group);
        }

        $table_name = $wpdb->prefix . 'et_send';
        if ($wpdb->get_var("show tables like '{$table_name}'") != $table_name) {
            $create_sms_send = ("CREATE TABLE IF NOT EXISTS {$table_name}(
            `ID` int(10) NOT NULL auto_increment,
            `date` DATETIME,
            `sender` VARCHAR(20) NOT NULL,
            `message` TEXT NOT NULL,
            `recipient` LONGTEXT NOT NULL,
            `message_type` VARCHAR(20) NOT NULL,
  			`response` LONGTEXT NOT NULL,
  			`status` LONGTEXT NOT NULL,
            `success` bigint(20) NOT NULL,
            `failed` bigint(20) NOT NULL,
            `message_count` TEXT NOT NULL,
            `messages_id` TEXT NOT NULL,
            PRIMARY KEY(`ID`)) $charset_collate");

            dbDelta($create_sms_send);
        }
    }

    /**
     * Upgrade plugin requirements if needed
     */
    public static function upgrade( $upgrader_object, $options, $current_plugin_path_name ) {

        require_once(dirname( __FILE__, 6 ) . '/wp-admin/includes/upgrade.php');

        $__Expt_BaseController = new Expt_BaseController();
 
        if ( $options['action'] == 'update' && $options['type'] == 'plugin' && isset( $options['plugins'] ) ) {
            foreach( $options['plugins'] as $each_plugin ) {

                if ( $each_plugin == $current_plugin_path_name ) {
                    
                    add_option('et_plugin_update_redirect_to', true);
                    $get_plugin_version = $__Expt_BaseController->plugin_version;
                }
            }
        } elseif( $options['action'] == 'install' && $options['type'] == 'plugin' ) {
            
            $get_plugin_version = $__Expt_BaseController->plugin_version;
        }

        $install_et_ver = esc_attr( get_option( 'et_plugin_db_version' ) );

        if ( $install_et_ver < $get_plugin_version ) {

            // Up to date
            update_option('et_plugin_db_version', esc_attr( $get_plugin_version ) );
            add_option('et_plugin_update_redirect_to', true);
            
        }
    }

    public function upgrade_et_plugin() {

        global $wpdb;

        $__Expt_BaseController = new Expt_BaseController();
        $get_plugin_version = $__Expt_BaseController->plugin_version;

        if( $get_plugin_version == '1.2.0' ) {
                
            /**
             * If update from v1.0.0
             * Replace and update combined et_integration option with separate options.
             */
            if( get_option( 'et_integration' ) ) {
                $et_integration = get_option( 'et_integration' );
                
                if( !get_option( 'et_integration_cf7' ) ) {
                    $contact_form_7_chk = !empty( $et_integration['contact_form_7_chk'] ) ? array( 'contact_form_7_chk' => $et_integration['contact_form_7_chk'] ) : array();
                    update_option( 'et_integration_cf7', $contact_form_7_chk );
                }
                
                if( !get_option( 'et_integration_woocommerce' ) ) {
                    $woocommerce_chk = !empty( $et_integration['woocommerce_chk'] ) ? array( 'woocommerce_chk' => $et_integration['woocommerce_chk'] ) : array();
                    $woocommerce_body = !empty( $et_integration['woocommerce_body'] ) ? array( 'woocommerce_body' => $et_integration['woocommerce_body'] ) : array();
                    update_option( 'et_integration_woocommerce', array_merge( $woocommerce_chk, $woocommerce_body ) );
                }
                
                if( !get_option( 'et_integration_edd' ) ) {
                    $easy_digital_downloads_chk = !empty( $et_integration['easy_digital_downloads_chk'] ) ? array( 'easy_digital_downloads_chk' => $et_integration['easy_digital_downloads_chk'] ) : array();
                    $easy_digital_downloads_body = !empty( $et_integration['easy_digital_downloads_body'] ) ? array( 'easy_digital_downloads_body' => $et_integration['easy_digital_downloads_body'] ) : array();
                    update_option( 'et_integration_edd', array_merge( $easy_digital_downloads_chk, $easy_digital_downloads_body ) );
                }

                delete_option( 'et_integration' );
            }


            /**
             * If update from or less than v1.1.0
             * Alter 'wp_et_send' table and add two more columns.
             */
            $collate = $wpdb->collate;
            
            $table_name = $wpdb->prefix . 'et_send';
            $table_message_count = $wpdb->get_row("SHOW COLUMNS FROM `{$table_name}` LIKE 'message_count';");
            $table_messages_id = $wpdb->get_row("SHOW COLUMNS FROM `{$table_name}` LIKE 'messages_id';");

            if( !isset( $table_message_count ) ) {

                $wpdb->query(
                    "ALTER TABLE `{$table_name}`
                    ADD COLUMN message_count TEXT COLLATE $collate NOT NULL"
                );
            }

            if( !isset( $table_messages_id ) ) {

                $wpdb->query(
                    "ALTER TABLE `{$table_name}`
                    ADD COLUMN messages_id TEXT COLLATE $collate NOT NULL"
                );
            }

        }

        set_transient( '__expttxt_need_upgrade', '__expttxt_need_upgrade_' . $get_plugin_version, 12 * HOUR_IN_SECONDS );
    }

    function javascript_variables(){ ?>
        <script type="text/javascript">
            var ajax_url = '<?php esc_attr_e( admin_url( "admin-ajax.php" ) ); ?>';
            var ajax_nonce = '<?php esc_attr_e( wp_create_nonce( "secure_nonce_name" ) ); ?>';
        </script><?php
    }
}