<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Base;

class Expt_BaseController {

    public $plugin_path;
    public $plugin_url;
    public $plugin_basename;

    public $expt_error_log_file;
    public $plugin_version;

    public $managers = array();
    public $managers_API = array();
    public $managers_SendSMS = array();
    public $managers_Notifications = array();
    public $managers_General = array();
    public $managers_Newsletter = array();
    public $managers_Integration = array();

    public $apiValues;
    public $apiUsername;
    public $apikey;
    public $apisecret;
    public $apiSender;
    public $adminNumber;

    public $integrate_arr;
    public $allowed_html;
    
    public function __construct() {
        $this->plugin_path = plugin_dir_path( dirname( __DIR__ ) );
        $this->plugin_url = plugin_dir_url( dirname( __DIR__ ) );
        $this->plugin_basename = plugin_basename( dirname( __FILE__, 3 ) ) . '/experttextingofficial.php';

        // Error Log File
        $this->expt_error_log_file = $this->plugin_path . 'ExptErrorLog/'. date('d-M-Y', current_time('timestamp')) .'_expt_error_log_file.log';
        $this->expt_error_log_file_PATHURL = $this->plugin_path . 'ExptErrorLog/';

        // API Values
        $this->apiValues = $this->expt_escape_array_data( get_option( 'et_api_set' ) );

        // Default Values
        $this->defaultValues( 'et_menu', 'login_with_mobile_body', 'Your login verification code is [user_code]' );
        $this->defaultValues( 'et_menu', 'login_2FA_body', 'Your login verification code is [user_code]' );

        if ( !empty( $this->apiValues ) ) {
            
            $this->apiUsername = $this->apiValues['et_api_username'];
            $this->apikey = $this->apiValues['et_api_key'];
            $this->apisecret = $this->apiValues['et_api_secret'];
            $this->apiSender = $this->apiValues['et_api_sender'];
        } else {

            $this->apiUsername = $this->apikey = $this->apisecret = '';
            $this->apiSender = 'DEFAULT';
        }

        // Admin Number
        $this->adminNumber = !empty( get_option( 'et_general_set' ) ) ? esc_attr( get_option( 'et_general_set' )['et_general_admin_number'] ) : '';
        
        // Get plugin Data.
        if (!function_exists('get_plugin_data')) {
            require_once(ABSPATH . 'wp-admin/includes/plugin.php');
        }
        
        // Plugin version
        $plugin_data = get_plugin_data($this->plugin_path . 'experttextingofficial.php');
        $this->plugin_version = $plugin_data['Version'];

        // Add display style attribute in WP Safe Styles
        add_filter( 'safe_style_css', function( $styles ) {
            $styles[] = 'display';
            return $styles;
        } );

        $this->integrate_arr = array(
            'integration_cf7' => 'Contact Form 7',
            'integration_woocommerce' => 'WooCommerce',
            'integration_edd' => 'Easy Digital Downloads',
            'integration_gf' => 'Gravity Forms',
            'integration_qf' => 'Quform',
            'integration_bp' => 'BuddyPress',
        );

        // These are the only allowed html attributes
        $this->allowed_html = array(
            'span' => array(
                'class' => array(),
                'id' => array(),
                'style' => array(),
            ),
            'h6' => array(
                'class' => array(),
                'style' => array(),
            ),
            'h5' => array(
                'class' => array(),
                'style' => array(),
            ),
            'button' => array(
                'class' => array(),
                'id' => array(),
                'name' => array(),
                'onclick' => array(),
                'type' => array(),
            ),
            'input' => array(
                'class' => array(),
                'id' => array(),
                'style' => array(),
                'name' => array(),
                'type' => array(),
                'placeholder' => array(),
                'value' => array(),
            ),
            'select' => array(
                'class' => array(),
                'id' => array(),
                'style' => array(),
                'name' => array(),
                'placeholder' => array(),
                'data-placeholder' => array(),
                'multiple' => array(),
            ),
            'option' => array(
                'value' => array(),
            ),
            'div' => array(
                'class' => array(),
                'id' => array(),
                'style' => array(),
            ),
            'a' => array(
                'href' => array(),
                'target' => array(),
                'class' => array(),
                'title' => array()
            ),
            'img' => array(
                'src' => array(),
                'class' => array(),
                'title' => array()
            ),
            'ul' => array(
                'style' => array(),
            ),
            'li' => array(),
            'p' => array(),
            'strong' => array(),
            'b' => array(),
            'mark' => array(),
            'br' => array(),
            'hr' => array(),
            'code' => array(),
            'em' => array(),
            'i' => array(),
        );
        
        /**
         * noFieldOnlyText: ID => [Callback, Title, CallbackFn, SubText/Text]
         * checkboxField: ID => [Callback, Title, CallbackFn, OptionName, SubText/Text]
         * textboxField: ID => [Callback, Title, CallbackFn, OptionName, Additional Args, SubText/Text]
         * textareaField: ID => [Callback, Title, CallbackFn, OptionName, Additional Args, isHtmlSupported, SubText/Text]
         * selectField: ID => [Callback, Title, CallbackFn, OptionName, Additional Args, SubText/Text, Array[select options]]
         */

        // Feature Settings
        $this->managers = array(
            'feature_newsletter_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Newsletter', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'check_widget' => array(
                'callback' => 'callbacks_mngr',
                'title' => __('Enable Newsletter Form Widget', 'experttexting-official'),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to display the widget of newsletter form.</span><br><br><span class="fw-normal text-muted">After enabling, go to <a href="'. admin_url("widgets.php") .'">Widgets</a> and add a <b>"ExpertTexting SMS Newsletter"</b> widget. <br>Or use shortcode <b>[expt-sub-form title="TITLE" desc="DESCRIPTION"]</b> in Posts, Pages or Widget.</span>'
            ),
            'verify_subscriber' => array(
                'callback' => 'callbacks_mngr',
                'title' => __('Enable Activation Code in Newsletter Form', 'experttexting-official'),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to verify user subscription with the activation code.</span>'
            ),
            'show_groups' => array(
                'callback' => 'callbacks_mngr',
                'title' => __('Enable Showing Groups on Newsletter Form', 'experttexting-official'),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to show group choice to the user who subscribes.</span>'
            ),
            // Privacy Policy | @since 1.2.0
            'feature_newsletter_privacy_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Newsletter - Privacy Policy', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'show_newsletter_privacy' => array(
                'callback' => 'callbacks_mngr',
                'title' => __('Enable Showing Privacy Policy Checkbox on Newsletter Form', 'experttexting-official'),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to show Privacy Policy checkbox to the user who subscribes.</span>'
            ),
            'newsletter_privacy_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Description of Privacy Policy', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'isHtmlSupported' => true,
                'subText' => '<span class="fw-normal text-muted">Enter the content of the Privacy Policy checkbox. <br> <b>HTML Allowed Tags:</b> <span class="lh-base"><mark>'.htmlentities('<span style="">').'</mark> | <mark>'.htmlentities('<a href="" title="" target="">').'</mark> | <mark>'.htmlentities('<ul style="">').'</mark> | <mark>'.htmlentities('<li style="">').'</mark> | <mark>'.htmlentities('<p style="">').'</mark> | <mark>'.htmlentities('<strong>').'</mark> | <mark>'.htmlentities('<mark style="">').'</mark> | <mark>'.htmlentities('<b>').'</mark> | <mark>'.htmlentities('<br>').'</mark> | <mark>'.htmlentities('<div style="">').'</mark> | <mark>'.htmlentities('<code style="">').'</mark> | <mark>'.htmlentities('<em>').'</mark> | <mark>'.htmlentities('<i>').'</mark></span> </span>',
            ),
            'feature_show_credit_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Show Credit', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'show_balance' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Show Credit in Send SMS Page', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to show your account credit in send SMS page.</span>'
            ),
            'show_balance_admin_menu' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Show Credit in Admin Menu Bar', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to show your account credit in admin menu bar.</span>'
            ),
            'feature_unicode_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Unicode', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'send_unicode' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Send Message as Unicode', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to send SMS messages using Unicode for non-English characters (such as Persian, Arabic, Chinese or Cyrillic characters).</span>'
            ),
            'feature_add_Number_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Add Number Field', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'add_mobile_field' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Add Mobile Number Field', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to add "Mobile Number" field to user profile and register form.</span>'
            ),
            // Intl Mobile Input | @since 1.2.0
            'feature_intl_tel_input_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('International Mobile Number Input', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'intl_tel_input_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable International Mobile Input', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to switch mobile number input fields to international mobile number input.</span>'
            ),
            'intl_tel_input_only_countries' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Only Countries', 'experttexting-official' ),
                'callbackFn' => 'selectCountriesField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'class="chosen-select regular-text mb-2" multiple',
                'subText' => '<span class="fw-normal text-muted">Specify only the countries that you want to display.</span>',
                'selectOptions' => $this->expt_countries()
            ),
            'intl_tel_input_preferred_countries' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Preferred Countries', 'experttexting-official' ),
                'callbackFn' => 'selectCountriesField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'class="chosen-select regular-text mb-2" multiple',
                'subText' => '<span class="fw-normal text-muted">Specify the countries you want to display at the top of the list.</span>',
                'selectOptions' => $this->expt_countries()
            ),
            'intl_tel_input_national_mode' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'National Mode', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to switch to national mode. It allow users to enter national numbers (and not have to think about international dial codes).</span>'
            ),
            'intl_tel_input_separate_dialcode' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Separate Dial Code', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to display the country dial code next to the selected flag so it\'s not part of the typed number. Note that this will disable National Mode</span>'
            ),
            // Login with mobile | @since 1.1.0
            'feature_login_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Login with Mobile', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'login_with_mobile' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Login with Mobile', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to add "Login with Mobile" to login form.</span>'
            ),
            'login_with_mobile_retries' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Max Retries', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0"',
                'subText' => '<br><span class="fw-normal text-muted">The maximum number of times the login code will be sent to the user.</span><br><span class="fw-normal text-muted">For no limits, set it to: 0 (Default: 2)</span>',
            ),
            'login_with_mobile_retries_reset' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Retries Expire Time (in minutes)', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0"',
                'subText' => '<br><span class="fw-normal text-muted">This option resets the attempts after a certain time when the user reaches the maximum number of retries. (Default: 2)</span>',
            ),
            'login_with_mobile_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>User login code:</strong> <mark>[user_code]</mark></li> <li><strong>Username:</strong> <mark>[user_login]</mark></li> <li><strong>User email:</strong> <mark>[user_email]</mark></li> <li><strong>Nickname:</strong> <mark>[user_display_name]</mark></li> <li><strong>User mobile:</strong> <mark>[user_mobile]</mark></li> <li><strong>Site name:</strong> <mark>[site_name]</mark></li> <li><strong>Site URL:</strong> <mark>[site_url]</mark></li> </ul></span>',
            ),
            // Two-Factor Authentication | @since 1.1.0
            'feature_2FA_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Two-Factor Authentication', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'login_2FA' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Two-Factor Authentication with OTP', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_menu',
                'subText' => '<span class="fw-normal text-muted">Enable to add Two-Factor Authentication with OTP.<br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>How it works</strong></li> <li>1. User will enter the username/email and password</li> <li>2. Then, a code will be sent to user\'s mobile number</li> </ul></span>'
            ),
            'login_2FA_method' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'OTP Method', 'experttexting-official' ),
                'callbackFn' => 'selectField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'class="chosen-select regular-text mb-2 text-dark"',
                'subText' => '<span class="fw-normal text-muted">Select to enable or make it optional for user\'s choice.</span>',
                'selectOptions' => array(
                    __( 'Enable for All Users', 'experttexting-official' ) => 'enable_for_all_2fa',
                    __( 'Optional - User can enable/disable in their profile', 'experttexting-official' ) => 'optional_enable_disable_2fa',
                    )
            ),
            'login_2FA_runtime' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'OTP Runtime', 'experttexting-official' ),
                'callbackFn' => 'selectField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'class="chosen-select regular-text mb-2 text-dark"',
                'subText' => '<span class="fw-normal text-muted">Select a runtime whether the OTP requires for login is on every time or just once.</span>',
                'selectOptions' => array(
                    __( 'Every time', 'experttexting-official' ) => 'everytime_2fa',
                    __( 'Just once', 'experttexting-official' ) => 'just_once_2fa',
                    )
            ),
            'login_2FA_retries' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Max Retries', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0"',
                'subText' => '<br><span class="fw-normal text-muted">The maximum number of times the login OTP code will be sent to the user.</span><br><span class="fw-normal text-muted">For no limits, set it to: 0 (Default: 2)</span>',
            ),
            'login_2FA_retries_reset' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Retries Expire Time (in minutes)', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0"',
                'subText' => '<br><span class="fw-normal text-muted">This option resets the attempts after a certain time when the user reaches the maximum number of retries. (Default: 2)</span>',
            ),
            'login_2FA_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_menu',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>User login code:</strong> <mark>[user_code]</mark></li> <li><strong>Username:</strong> <mark>[user_login]</mark></li> <li><strong>User email:</strong> <mark>[user_email]</mark></li> <li><strong>Nickname:</strong> <mark>[user_display_name]</mark></li> <li><strong>User mobile:</strong> <mark>[user_mobile]</mark></li> <li><strong>Site name:</strong> <mark>[site_name]</mark></li> <li><strong>Site URL:</strong> <mark>[site_url]</mark></li> </ul></span>',
            ),
        );

        // API Settings
        $this->managers_API = array(
            'et_api_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('API Settings', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>'
            ),
            'et_api_desc' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'API Description', 'experttexting-official' ),
                'callbackFn' => 'noFieldOnlyText',
                'subText' => 'You can find the API credentials under <a href="https://www.experttexting.com/appv2/Dashboard/Setting" target="_blank">"Account Settings"</a> in ExpertTexting.'
            ),
            'et_api_username' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Username', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_api_set',
                'additionalArgs' => 'type="text" class="regular-text mb-1" maxlength="50"',
                'subText' => '<br><span class="fw-normal text-muted">Enter Your Username of ExpertTexting</span>'
            ),
            'et_api_key' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'API Key', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_api_set',
                'additionalArgs' => 'type="text" class="regular-text mb-1" maxlength="100"',
                'subText' => '<br><span class="fw-normal text-muted">Enter Your API Key of ExpertTexting</span>'
            ),
            'et_api_secret' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'API Secret', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_api_set',
                'additionalArgs' => 'type="password" class="regular-text mb-1" maxlength="50"',
                'subText' => '<button type="button" class="button ms-2" onclick="showHideField(\'et_api_set[et_api_secret]\')"><span class="dashicons dashicons-visibility m-1"></span></button><br><span class="fw-normal text-muted">Enter Your API Secret of ExpertTexting</span>'
            ),
            'et_api_sender' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Sender Number (Optional)', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_api_set',
                'additionalArgs' => 'type="text" class="regular-text mb-1 alphanumericCharOnly" maxlength="20"',
                'subText' => '<br><span class="fw-normal text-muted">Enter Dedicated Number or Sender ID of ExpertTexting</span>'
            ),
            'et_api_status' => array(
                'callback' => 'callbacks_api',
                'title' => 'Status',
                'callbackFn' => 'etApiSetStatus',
            ),
            'et_api_balance' => array(
                'callback' => 'callbacks_api',
                'title' => 'Balance',
                'callbackFn' => 'etApiBalance',
            )
        );


        // SMS Fields
        $managers_SendSMSFields = array(
            'et_send_from' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Send from', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_send_sms',
                'additionalArgs' => 'type="text" class="regular-text mb-1 alphanumericCharOnly" maxlength="20"',
            ),
            'et_send_to_selection' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Send to', 'experttexting-official' ),
                'callbackFn' => 'selectField',
                'optionName' => 'et_send_sms',
                'additionalArgs' => 'class="chosen-select regular-text mb-2" id="sendSmsSelection"',
                'selectOptions' => array(
                    __( 'Subscribe Users', 'experttexting-official' ) => 'subscriber_users',
                    __( 'Wordpress Users', 'experttexting-official' ) => 'wordpress_users',
                    __( 'Number(s)', 'experttexting-official' ) => 'add_numbers'
                    )
            ),
            'et_send_msg' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_send_sms',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span>Characters: <span id="charCount">0</span> of 800 &nbsp; | &nbsp; <span class="fw-bold" id="unicodeCheck">Normal Message</span> <span style="display: none;" class="fw-bold text-danger" id="ucEnabled">Unicode Message: Enabled</span></span>',
            )
        );

        // If Balance feature is activated
        if ( ! $this->activated( 'et_menu', 'show_balance' ) ) {
            $managers_SendSMSBalance = array();
        } else {
            $managers_SendSMSBalance = array(
                'et_send_balance' => array(
                    'callback' => 'callbacks_api',
                    'title' => 'Balance',
                    'callbackFn' => 'etApiBalance'
                )
            );
        }

        $this->managers_SendSMS = array_merge($managers_SendSMSFields, $managers_SendSMSBalance);

        // Notification Settings
        $this->managers_Notifications = array(
            'post_content_count_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Post Content Words Count', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'post_content_count' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Post Words Count', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'type="number" class="regular-text mb-1"',
                'subText' => '<br><span class="fw-normal text-muted">The number of words of the post to show in notifications. (Default : 10)</span>',
            ),
            'post_content_count' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Post Words Count', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'type="number" class="regular-text mb-1"',
                'subText' => '<br><span class="fw-normal text-muted">The number of words of the post to show in notifications. (Default : 10)</span>',
            ),
            // Published new post
            'on_new_post_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Published New Posts', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'on_new_post_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_notifications',
                'subText' => '<span class="fw-normal text-muted">Send an SMS to subscribers when a new post is published.</span>',
            ),
            'on_new_post_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Post title:</strong> <mark>[post_title]</mark></li> <li><strong>Post content:</strong> <mark>[post_content]</mark></li> <li><strong>Post url:</strong> <mark>[post_url]</mark></li> <li><strong>Post date:</strong> <mark>[post_date]</mark></li> </ul></span>',
            ),
            // Author of the post
            'author_post_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Author of the Post', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'author_post_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_notifications',
                'subText' => '<span class="fw-normal text-muted">Send an SMS to the author of the post when the post is published. <br>Make sure to "Add Mobile number field" enabled in "Features" settings.</span>',
            ),
            'author_post_types' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Post Types', 'experttexting-official' ),
                'callbackFn' => 'selectField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'class="chosen-select regular-text mb-2" multiple',
                'subText' => '<span class="fw-normal text-muted">Select post types that you want to use this option.</span>',
                'selectOptions' => array(
                    'Posts' => 'posts',
                    'Pages' => 'pages'
                )
            ),
            'author_post_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Post title:</strong> <mark>[post_title]</mark></li> <li><strong>Post content:</strong> <mark>[post_content]</mark></li> <li><strong>Post url:</strong> <mark>[post_url]</mark></li> <li><strong>Post date:</strong> <mark>[post_date]</mark></li> </ul></span>',
            ),
            // Register new user
            'new_register_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Register a New User', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'new_register_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_notifications',
                'subText' => '<span class="fw-normal text-muted">Send an SMS to the admin and user when the user registers on WordPress.</span>',
            ),
            'new_register_admin_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body for Admin', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>User login:</strong> <mark>[user_login]</mark></li> <li><strong>User email:</strong> <mark>[user_email]</mark></li> <li><strong>Register date:</strong> <mark>[date_register]</mark></li> </ul></span>',
            ),
            'new_register_user_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body for User', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>User login:</strong> <mark>[user_login]</mark></li> <li><strong>User email:</strong> <mark>[user_email]</mark></li> <li><strong>Register date:</strong> <mark>[date_register]</mark></li> </ul></span>',
            ),
            // New comment
            'new_comment_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('New Comment', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'new_comment_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_notifications',
                'subText' => '<span class="fw-normal text-muted">Send an SMS to the admin when a new comment is posted.</span>',
            ),
            'new_comment_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Comment author:</strong> <mark>[comment_author]</mark></li> <li><strong>Author email:</strong> <mark>[comment_author_email]</mark></li> <li><strong>Author URL:</strong> <mark>[comment_author_url]</mark></li> <li><strong>Author IP:</strong> <mark>[comment_author_IP]</mark></li> <li><strong>Comment date:</strong> <mark>[comment_date]</mark></li> <li><strong>Comment content:</strong> <mark>[comment_content]</mark></li> <li><strong>Comment URL:</strong> <mark>[comment_url]</mark></li> </ul></span>',
            ),
            // User login
            'user_login_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('User login', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'user_login_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_notifications',
                'subText' => '<span class="fw-normal text-muted">Send an SMS to the admin when a user logs in.</span>',
            ),
            'user_login_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_notifications',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Username:</strong> <mark>[username_login]</mark></li> <li><strong>Nickname:</strong> <mark>[display_name]</mark></li> <li><strong>User Email:</strong> <mark>[user_email]</mark></li> </ul></span>',
            )
        );

        // General Settings
        $this->managers_General = array(
            'et_general_admin_number_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Admin Mobile Number', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'et_general_admin_number' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Admin Mobile Number', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_general_set',
                'additionalArgs' => 'type="tel" class="regular-text mb-1" maxlength="20" placeholder="Enter Number with Country Code"',
                'subText' => '<br><span class="fw-normal text-muted">Enter admin mobile number to get SMS from this plugin (Notifications & Integrations)</span>',
            ),
            'et_general_log_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('ExpertTexting Log', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'et_general_log_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable ExpertTexting Logging', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_general_set',
                'subText' => '<span class="fw-normal text-muted">Logging helps solve problems by logging debug and error messages into WordPress. Logging messages can contain important information. The purpose of logging is to be used only temporarily when attempting to diagnose problems. Once the problem has been identified and resolved, it should be disabled.</span>',
            )
        );

        // Newsletter Settings
        $this->managers_Newsletter = array(
            // Welcome Message
            'newsletter_welcome_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Newsletter Welcome SMS', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'newsletter_welcome_chk' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Enable this Notification', 'experttexting-official' ),
                'callbackFn' => 'checkboxField',
                'optionName' => 'et_newsletter',
                'subText' => '<span class="fw-normal text-muted">Send a welcome SMS to the new subscriber when they subscribe to your newsletter.</span>',
            ),
            'newsletter_welcome_body' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Message Body', 'experttexting-official' ),
                'callbackFn' => 'textareaField',
                'optionName' => 'et_newsletter',
                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Subscribe name:</strong> <mark>[subscribe_name]</mark></li> <li><strong>Subscribe mobile:</strong> <mark>[subscribe_mobile]</mark></li> </ul></span>',
            ),
            // Newsletter Mobile Number Field Terms
            'newsletter_mobile_heading' => array(
                'callback' => 'callbacks_mngr',
                'title' => '<h6 class="text-uppercase">'. __('Mobile Number Field Terms', 'experttexting-official') .'</h6>',
                'callbackFn' => 'noFieldOnlyText',
                'subText' => '<hr>',
            ),
            'newsletter_mobile_field_placeholder' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Field Placeholder', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_newsletter',
                'additionalArgs' => 'type="text" class="regular-text mb-1" maxlength="50"',
                'subText' => '<br><span class="fw-normal text-muted">Enter mobile number field placeholder</span>',
            ),
            'newsletter_mobile_min' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Minimum Number', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_newsletter',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0" max="20"',
                'subText' => '<br><span class="fw-normal text-muted">Enter minimum (Min: 0)</span>',
            ),
            'newsletter_mobile_max' => array(
                'callback' => 'callbacks_mngr',
                'title' => __( 'Maximum Number', 'experttexting-official' ),
                'callbackFn' => 'textboxField',
                'optionName' => 'et_newsletter',
                'additionalArgs' => 'type="number" class="regular-text mb-1" min="0" max="20"',
                'subText' => '<br><span class="fw-normal text-muted">Enter maximum (Max: 20)</span>',
            )
        );

        // Integration Settings
        $this->managers_Integration = array(
            // Contact Form 7
            'Contact Form 7' => array(
                'contact_form_7_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('SMS Meta Box', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'contact_form_7_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'SMS Meta Box', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_cf7',
                    'subText' => '<span class="fw-normal text-muted">Enable to Add WordPress SMS Meta-Box to Contact Form 7 plugin.</span>',
                ),
            ),
            // WooCommerce
            'WooCommerce' => array(
                'woocommerce_mobile_field_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Mobile Number Field', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_mobile_field_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Add Mobile Number Field', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to add "Mobile Number" field to checkout page. Otherwise it will use the WooCommerce\'s default Phone field.</span>',
                ),
                // Notify for New Order (Admin)
                'woocommerce_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Admin)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "admin" whenever a new order is placed.</span>',
                ),
                'woocommerce_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order total cost:</strong> <mark>[order_total_cost]</mark></li> <li><strong>Order date/time:</strong> <mark>[order_date]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>',
                ),
                // Notify for New Order (Numbers)
                'woocommerce_new_order_to_number_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Numbers)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_new_order_to_number_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "number" whenever a new order is placed.</span>',
                ),
                'woocommerce_new_order_to_number_receiver' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'SMS Receiver(s)', 'experttexting-official' ),
                    'callbackFn' => 'textboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'type="text" class="regular-text mb-1 numericCommaSepOnly"',
                    'subText' => '<br><span class="fw-normal text-muted">Enter mobile number to get SMS notification. <br><br> To send to more than one number, separate the numbers with a comma. <br> (e.g. 10000000001,10000000002,10000000003)</span>',
                ),
                'woocommerce_new_order_to_number_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order total cost:</strong> <mark>[order_total_cost]</mark></li> <li><strong>Order date/time:</strong> <mark>[order_date]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>',
                ),
                // Notify for New Order (Customer)
                'woocommerce_new_order_to_customer_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Customer)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_new_order_to_customer_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever a new order is placed.</span>',
                ),
                'woocommerce_new_order_to_customer_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order total cost:</strong> <mark>[order_total_cost]</mark></li> <li><strong>Order date/time:</strong> <mark>[order_date]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>',
                ),
                // Notify of Stock (Admin)
                'woocommerce_stock_to_admin_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify of Stock <br> (To Admin)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_stock_to_admin_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Low Stock', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "admin" whenever the stock is low.</span>',
                ),
                'woocommerce_stock_to_admin_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Product ID:</strong> <mark>[product_id]</mark></li> <li><strong>Product name:</strong> <mark>[product_name]</mark></li> </ul></span>',
                ),
                // Notify of Status (Customer)
                'woocommerce_status_to_customer_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify of Status <br> (To Customer)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_status_to_customer_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Order Status', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_woocommerce',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever the status of order is changed.</span>',
                ),
                'woocommerce_status_to_customer_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_woocommerce',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[order_id]</mark></li> <li><strong>Order status:</strong> <mark>[order_status]</mark></li> <li><strong>Order view URL:</strong> <mark>[order_view_url]</mark></li> <li><strong>Order payment URL:</strong> <mark>[order_payment_url]</mark></li> <li><strong>Customer first name:</strong> <mark>[customer_first_name]</mark></li> <li><strong>Customer email:</strong> <mark>[customer_email]</mark></li> <li><strong>Customer phone:</strong> <mark>[customer_phone]</mark></li> <li><strong>Customer country:</strong> <mark>[customer_country]</mark></li> </ul></span>',
                ),
                // Notify by Status (Customer)
                'woocommerce_notify_by_status_to_customer_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify by Status <br> (To Customer)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'woocommerce_notify_by_status_to_customer' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Order Status & Message Body', 'experttexting-official' ),
                    'callbackFn' => 'repeatFieldsWC',
                    'optionName' => 'et_integration_woocommerce',
                ),
            ),
            // Easy Digital Downloads
            'Easy Digital Downloads' => array(
                // Mobile Number Field
                'easy_digital_downloads_mobile_field_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Mobile Number Field', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'easy_digital_downloads_mobile_field_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Add Mobile Number Field', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_edd',
                    'subText' => '<span class="fw-normal text-muted">Enable to add "Mobile Number" field to checkout page.</span>',
                ),
                // Notify for New Order (Admin)
                'easy_digital_downloads_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Admin)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'easy_digital_downloads_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_edd',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "admin" whenever a new order is placed and a payment is marked as complete.</span>',
                ),
                'easy_digital_downloads_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_edd',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[edd_order_id]</mark></li> <li><strong>Customer email:</strong> <mark>[edd_email]</mark></li> <li><strong>Customer first name:</strong> <mark>[edd_first]</mark></li> <li><strong>Customer last name:</strong> <mark>[edd_last]</mark></li> <li><strong>Order Date:</strong> <mark>[edd_order_date]</mark></li> </ul></span>',
                ),
                // Notify for New Order (Numbers)
                'easy_digital_downloads_new_order_to_number_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Numbers)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'easy_digital_downloads_new_order_to_number_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_edd',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "number" whenever a new order is placed and a payment is marked as complete.</span>',
                ),
                'easy_digital_downloads_new_order_to_number_receiver' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'SMS Receiver(s)', 'experttexting-official' ),
                    'callbackFn' => 'textboxField',
                    'optionName' => 'et_integration_edd',
                    'additionalArgs' => 'type="text" class="regular-text mb-1 numericCommaSepOnly"',
                    'subText' => '<br><span class="fw-normal text-muted">Enter mobile number to get SMS notification. <br><br> To send to more than one number, separate the numbers with a comma. <br> (e.g. 10000000001,10000000002,10000000003)</span>',
                ),
                'easy_digital_downloads_new_order_to_number_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_edd',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[edd_order_id]</mark></li> <li><strong>Customer email:</strong> <mark>[edd_email]</mark></li> <li><strong>Customer first name:</strong> <mark>[edd_first]</mark></li> <li><strong>Customer last name:</strong> <mark>[edd_last]</mark></li> <li><strong>Order Date:</strong> <mark>[edd_order_date]</mark></li> </ul></span>',
                ),
                // Notify for New Order (Customer)
                'easy_digital_downloads_new_order_to_customer_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Notify for New Order <br> (To Customer)', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'easy_digital_downloads_new_order_to_customer_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'New Order', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_edd',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the "customer" whenever a new order is placed and a payment is marked as complete. <br> Make sure to "Add Mobile number field" enabled above.</span>',
                ),
                'easy_digital_downloads_new_order_to_customer_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_edd',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Order Id:</strong> <mark>[edd_order_id]</mark></li> <li><strong>Customer email:</strong> <mark>[edd_email]</mark></li> <li><strong>Customer first name:</strong> <mark>[edd_first]</mark></li> <li><strong>Customer last name:</strong> <mark>[edd_last]</mark></li> <li><strong>Order Date:</strong> <mark>[edd_order_date]</mark></li> </ul></span>',
                ),
            ),
            // BuddyPress
            'BuddyPress' => array(
                // Mobile Number Field
                'buddypress_mobile_field_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Mobile Number Field', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'buddypress_mobile_field_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Add Mobile Number Field', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_bp',
                    'subText' => '<span class="fw-normal text-muted">Enable to add "Mobile Number" field to BuddyPress profile page.</span>',
                ),
                // Mentions
                'buddypress_mentions_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Mentions', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'buddypress_mentions_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Enable', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_bp',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the user when someone mentioned. <br> For Example <b>@username</b>.</span>',
                ),
                'buddypress_mentions_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_bp',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Posted user\'s display name:</strong> <mark>[bp_user_posted_display_name]</mark></li> <li><strong>User\'s profile permalink:</strong> <mark>[bp_primary_link]</mark></li> <li><strong>Time:</strong> <mark>[bp_time]</mark></li> <li><strong>Mention message content:</strong> <mark>[bp_mention_content]</mark></li> <li><strong>Receiver user\'s display name:</strong> <mark>[bp_user_receiver_display_name]</mark></li> </ul></span>',
                ),
                // Comment Reply
                'buddypress_comment_reply_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '<h6 class="text-uppercase">'. __('Post a Comment', 'experttexting-official') .'</h6>',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '<hr>',
                ),
                'buddypress_comment_post_chk' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Enable', 'experttexting-official' ),
                    'callbackFn' => 'checkboxField',
                    'optionName' => 'et_integration_bp',
                    'subText' => '<span class="fw-normal text-muted">Enable to send an SMS to the user when someone post a comment on activity.</span>',
                ),
                'buddypress_comment_post_body' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => __( 'Message Body', 'experttexting-official' ),
                    'callbackFn' => 'textareaField',
                    'optionName' => 'et_integration_bp',
                    'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                    'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Posted user\'s display name:</strong> <mark>[bp_user_posted_display_name]</mark></li> <li><strong>Comment content:</strong> <mark>[bp_comment_content]</mark></li> <li><strong>Receiver user\'s display name:</strong> <mark>[bp_user_receiver_display_name]</mark></li> </ul></span>',
                ),
            ),
        );
        
        // Gravity Forms
        if( class_exists( 'RGFormsModel' ) ) {
            $gfForms = \RGFormsModel::get_forms( null, 'title' );
            $gfform_options = array();
            $gfform_options_arr = array();
            $gfform_options_field_arr = array();
            if( $gfForms ) {
                foreach( $gfForms as $form ) {
                    
                    $gf_single_form_options = array();

                    $form_meta = \RGFormsModel::get_form_meta( $form->id );
                    if( $form_meta ) {
                        foreach( $form_meta['fields'] as $field ) {
                            if( isset( $field['label'] ) ) {
                                $gf_single_form_options[$field['label']] = $field['id'];
                            } elseif( isset( $field->id ) ) {
                                $gf_single_form_options[$field->label] = $field->id;
                            }
                        }
                    }

                    $field_tags = '';
                    if( is_array( $gf_single_form_options ) && count( $gf_single_form_options ) ) {
                        $field_tags = '<li><strong>Fields:</strong> <span class="lh-base">';
                        foreach( $gf_single_form_options as $label => $id ) {
                            $field_tags .= '<mark>[gf_field_'. esc_attr( $label ) .']</mark> , ';
                        }
                        $field_tags .= '</span></li>';
                    }

                    $gfform_options_arr = array(
                        'gravity_forms_heading_'. esc_attr( $form->id ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => '<hr><h5 class="text-uppercase">'. __( 'Form ID '. esc_attr( $form->id ) ) .'</h5><hr>',
                            'callbackFn' => 'noFieldOnlyText',
                            'subText' => '<hr><h5 class="text-uppercase text-center">'. __( esc_attr( $form->title ) ) .'</h5><hr>',
                        ),
                        // Send to numbers
                        'gravity_forms_num_heading_'. esc_attr( $form->id ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => '<h6 class="text-uppercase">'. __( 'Send To Specified Numbers' ) .'</h6>',
                            'callbackFn' => 'noFieldOnlyText',
                            'subText' => '',
                        ),
                        'gravity_forms_num_chk_'. esc_attr( $form->id ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'Enable Send SMS to Number(s)', 'experttexting-official' ),
                            'callbackFn' => 'checkboxField',
                            'optionName' => 'et_integration_gf',
                            'subText' => '<span class="fw-normal text-muted">Enable to send an SMS Send an sms message to numbers when submit form.</span>',
                        ),
                        'gravity_forms_num_receiver_'. esc_attr( $form->id ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'SMS Receiver(s)', 'experttexting-official' ),
                            'callbackFn' => 'textboxField',
                            'optionName' => 'et_integration_gf',
                            'additionalArgs' => 'type="text" class="regular-text mb-1 numericCommaSepOnly"',
                            'subText' => '<br><span class="fw-normal text-muted">Enter mobile number to get SMS notification. <br><br> To send to more than one number, separate the numbers with a comma. <br> (e.g. 10000000001,10000000002,10000000003)</span>',
                        ),
                        'gravity_forms_num_body_'. esc_attr( $form->id ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'Message Body', 'experttexting-official' ),
                            'callbackFn' => 'textareaField',
                            'optionName' => 'et_integration_gf',
                            'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                            'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Form Name:</strong> <mark>[gf_form_title]</mark></li> <li><strong>IP:</strong> <mark>[gf_ip]</mark></li> <li><strong>Form URL:</strong> <mark>[gf_form_url]</mark></li> <li><strong>User agent:</strong> <mark>[gf_user_agent]</mark></li> '. $field_tags .' </ul></span>',
                        )
                    );

                    if(( is_array( $gf_single_form_options ) && count( $gf_single_form_options ) )) {
                        $gfform_options_field_arr = array(
                            // Send to field
                            'gravity_forms_field_heading_'. esc_attr( $form->id ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => '<h6 class="text-uppercase">'. __( 'Send to form' ) .'</h6>',
                                'callbackFn' => 'noFieldOnlyText',
                                'subText' => '',
                            ),
                            'gravity_forms_field_chk_'. esc_attr( $form->id ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Enable Send SMS to Field', 'experttexting-official' ),
                                'callbackFn' => 'checkboxField',
                                'optionName' => 'et_integration_gf',
                                'subText' => '<span class="fw-normal text-muted">Enable to Send an sms message to field when submit form. <br>(<b>Note:</b> Field of Mobile Number must be included in Gravity Forms.)</span>',
                            ),
                            'gravity_forms_field_select_'. esc_attr( $form->id ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Select Gravity Form Field', 'experttexting-official' ),
                                'callbackFn' => 'selectField',
                                'optionName' => 'et_integration_gf',
                                'additionalArgs' => 'class="chosen-select regular-text mb-2"',
                                'subText' => '<span class="fw-normal text-muted">Select Gravity Form field of Mobile Number.</span>',
                                'selectOptions' => ( is_array( $gf_single_form_options ) && count( $gf_single_form_options ) ) ? $this->expt_escape_array_data( $gf_single_form_options ) : array()
                            ),
                            'gravity_forms_field_body_'. esc_attr( $form->id ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Message Body', 'experttexting-official' ),
                                'callbackFn' => 'textareaField',
                                'optionName' => 'et_integration_gf',
                                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Form Name:</strong> <mark>[gf_form_title]</mark></li> <li><strong>IP:</strong> <mark>[gf_ip]</mark></li> <li><strong>Form URL:</strong> <mark>[gf_form_url]</mark></li> <li><strong>User agent:</strong> <mark>[gf_user_agent]</mark></li> '. $field_tags .' </ul></span>',
                            )
                        );
                    }
                    
                    $gfform_options_arr = array_merge($gfform_options_arr, $gfform_options_field_arr);
                    $gfform_options = array_merge($gfform_options, $gfform_options_arr);
                }

            } else {
                $gfform_options = array(
                    'gravity_forms_heading' => array(
                        'callback' => 'callbacks_mngr',
                        'title' => '<h6 class="text-uppercase">'. __( 'No Form Created', 'experttexting-official' ) .'</h6>',
                        'callbackFn' => 'noFieldOnlyText',
                        'subText' => 'No form has been created yet. Create form to use this integration.',
                    ),
                );
            }

        } else {
            $gfform_options = array(
                'gravity_forms_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '',
                )
            );
        }

        $this->managers_Integration['Gravity Forms'] = $gfform_options;

        // Quform
        if ( class_exists( 'Quform_Repository' ) ) {
            $quFormInstance = new \Quform_Repository;
            $quForms = $quFormInstance->allForms();
            $quform_options = array();
            $quform_options_arr = array();
            $quform_options_field_arr = array();
            if( $quForms ) {
                foreach( $quForms as $form ) {
                    
                    $qf_single_form_options = array();

                    if( $form['id'] ) {
                        if( $form['elements'] ) {
                            foreach( $form['elements'] as $elements ) {
                                foreach( $elements['elements'] as $field ) {
                                    if( isset( $field['label'] ) && $field['type'] != 'submit' ) {
                                        $qf_single_form_options[$field['label']] = $field['id'];
                                    }
                                }
                            }
                        }
                    }

                    $field_tags = '';
                    if( is_array( $qf_single_form_options ) && count( $qf_single_form_options ) ) {
                        $field_tags = '<li><strong>Fields:</strong> <span class="lh-base">';
                        foreach( $qf_single_form_options as $label => $id ) {
                            $field_tags .= '<mark>[qf_field_'. esc_attr( $label ) .']</mark> , ';
                        }
                        $field_tags .= '</span></li>';
                    }

                    $quform_options_arr = array(
                        'qu_forms_heading_'. esc_attr( $form['id'] ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => '<hr><h5 class="text-uppercase">'. __( 'Form ID '. esc_attr( $form['id'] ) ) .'</h5><hr>',
                            'callbackFn' => 'noFieldOnlyText',
                            'subText' => '<hr><h5 class="text-uppercase text-center">'. __( esc_attr( $form['name'] ) ) .'</h5><hr>',
                        ),
                        // Send to numbers
                        'qu_forms_num_heading_'. esc_attr( $form['id'] ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => '<h6 class="text-uppercase">'. __( 'Send To Specified Numbers' ) .'</h6>',
                            'callbackFn' => 'noFieldOnlyText',
                            'subText' => '',
                        ),
                        'qu_forms_num_chk_'. esc_attr( $form['id'] ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'Enable Send SMS to Number(s)', 'experttexting-official' ),
                            'callbackFn' => 'checkboxField',
                            'optionName' => 'et_integration_qf',
                            'subText' => '<span class="fw-normal text-muted">Enable to send an SMS Send an sms message to numbers when submit form.</span>',
                        ),
                        'qu_forms_num_receiver_'. esc_attr( $form['id'] ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'SMS Receiver(s)', 'experttexting-official' ),
                            'callbackFn' => 'textboxField',
                            'optionName' => 'et_integration_qf',
                            'additionalArgs' => 'type="text" class="regular-text mb-1 numericCommaSepOnly"',
                            'subText' => '<br><span class="fw-normal text-muted">Enter mobile number to get SMS notification. <br><br> To send to more than one number, separate the numbers with a comma. <br> (e.g. 10000000001,10000000002,10000000003)</span>',
                        ),
                        'qu_forms_num_body_'. esc_attr( $form['id'] ) => array(
                            'callback' => 'callbacks_mngr',
                            'title' => __( 'Message Body', 'experttexting-official' ),
                            'callbackFn' => 'textareaField',
                            'optionName' => 'et_integration_qf',
                            'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                            'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Post Title:</strong> <mark>[qf_post_title]</mark></li> <li><strong>Form URL:</strong> <mark>[qf_form_url]</mark></li> <li><strong>Referring URL:</strong> <mark>[qf_referring_url]</mark></li> '. $field_tags .' </ul></span>',
                        ),
                    );

                    if(( is_array( $qf_single_form_options ) && count( $qf_single_form_options ) )) {
                        $quform_options_field_arr = array(
                            // Send to field
                            'qu_forms_field_heading_'. esc_attr( $form['id'] ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => '<h6 class="text-uppercase">'. __( 'Send to form' ) .'</h6>',
                                'callbackFn' => 'noFieldOnlyText',
                                'subText' => '',
                            ),
                            'qu_forms_field_chk_'. esc_attr( $form['id'] ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Enable Send SMS to Field', 'experttexting-official' ),
                                'callbackFn' => 'checkboxField',
                                'optionName' => 'et_integration_qf',
                                'subText' => '<span class="fw-normal text-muted">Enable to Send an sms message to field when submit form. <br>(<b>Note:</b> Field of Mobile Number must be included in Quform.)</span>',
                            ),
                            'qu_forms_field_select_'. esc_attr( $form['id'] ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Select Quform Field', 'experttexting-official' ),
                                'callbackFn' => 'selectField',
                                'optionName' => 'et_integration_qf',
                                'additionalArgs' => 'class="chosen-select regular-text mb-2"',
                                'subText' => '<span class="fw-normal text-muted">Select Quform field of Mobile Number.</span>',
                                'selectOptions' => ( is_array( $qf_single_form_options ) && count( $qf_single_form_options ) ) ? $this->expt_escape_array_data( $qf_single_form_options ) : array()
                            ),
                            'qu_forms_field_body_'. esc_attr( $form['id'] ) => array(
                                'callback' => 'callbacks_mngr',
                                'title' => __( 'Message Body', 'experttexting-official' ),
                                'callbackFn' => 'textareaField',
                                'optionName' => 'et_integration_qf',
                                'additionalArgs' => 'maxlength="800" class="regular-text mb-1"',
                                'subText' => '<span class="fw-normal text-muted">Enter the contents of the SMS message. (Max 800 characters) <br><br> <ul style="font-style: normal !important; padding-left: 0 !important;"> <li><strong>Post Title:</strong> <mark>[qf_post_title]</mark></li> <li><strong>Form URL:</strong> <mark>[qf_form_url]</mark></li> <li><strong>Referring URL:</strong> <mark>[qf_referring_url]</mark></li> '. $field_tags .' </ul></span>',
                            )
                        );
                    }

                    $quform_options_arr = array_merge($quform_options_arr, $quform_options_field_arr);
                    $quform_options = array_merge($quform_options, $quform_options_arr);
                }

            } else {
                $quform_options = array(
                    'qu_forms_heading' => array(
                        'callback' => 'callbacks_mngr',
                        'title' => '<h6 class="text-uppercase">'. __( 'No Form Created', 'experttexting-official' ) .'</h6>',
                        'callbackFn' => 'noFieldOnlyText',
                        'subText' => 'No form has been created yet. Create form to use this integration.',
                    ),
                );
            }

        } else {
            $quform_options = array(
                'qu_forms_heading' => array(
                    'callback' => 'callbacks_mngr',
                    'title' => '',
                    'callbackFn' => 'noFieldOnlyText',
                    'subText' => '',
                ),
            );
        }

        $this->managers_Integration['Quform'] = $quform_options;

    }

    // ET only logo header
    public function plugin_et_header_logo( $class = 'w-25', $style = '' ) {
        _e( '
        <div class="d-flex align-items-end">
            <img style="' . $style . '" class="admin_logo me-2 ' . $class . '" src="' . $this->plugin_url . "assets/img/logo.png" . '" alt="logo.png">
            <p>v' . $this->plugin_version . '</p>
        </div>
        ' );
    }

    // ET header
    public function plugin_et_header() {
        ?>
        <header class="d-flex expt_plugin_header justify-content-between position-relative overflow-hidden">
            <div class="d-flex align-items-end">
                <img class="admin_logo img me-2" src="<?php _e( $this->plugin_url . "assets/img/logo.png" ); ?>" alt="logo.png">
                <p>v<?php _e( $this->plugin_version ); ?></p>
            </div>
            <img class="img exter d-md-block d-none" src="<?php _e( ($this->plugin_url . "assets/img/et-exter.png") ); ?>" alt="exter">
            <div class="align-items-end d-md-flex flex-column d-none"><span class="fw-bold h3 text-dark">The Official ExpertTexting Plugin for WordPress</span>
                <span class="fw-bold h5 text-black-50">Integrating SMS to amplify the world of WordPress</span>
            </div>
        </header>
        <?php
    }

    // ET Menu Activate/Deactivate
	public function activated( string $option, string $key ) {

        $option = $this->expt_escape_array_data( get_option( $option ) );

		return isset( $option[ $key ] ) ? $option[ $key ] : false;
	}

    public function defaultValues( $option, $key, $value ) {

        $option_val = $this->expt_escape_array_data( get_option( $option ) );
        empty( $option_val[$key] ) ? $option_val[$key] = $value : '';
        update_option( $option, $option_val, 'yes' );
    }

    public function ettimecheck( $option, $count ) {
        $balanceDateTime = strtotime( date( esc_attr( get_option( $option ) ) ) );
        $currentDateTime = strtotime( date( 'Y-m-d H:i:s' ) );
        $timeCheck = $currentDateTime - $balanceDateTime;
        
        if( $timeCheck < $count ) {
            return false;
        }

        return true;
    }

    /**
     * Save WPDB Query logs
     * 
     * @since 1.0.0
     */
    public function save_expt_wpdb_log() {

        global $wpdb;

        $bt = debug_backtrace();
        $caller = array_shift($bt);
        $file_Line = $caller['file'] .':'. $caller['line'];

        $this->expt_error_log( '[EXPT DB ERROR LOG]', $file_Line, 'Last Query => ' . $wpdb->last_query );
        $this->expt_error_log( '[EXPT DB ERROR LOG]', $file_Line, 'Last Error => ' . $wpdb->last_error );
    }

    /**
     * Save other Expt logs
     * 
     * @param array $data => ['error type', 'error message']
     * @param array $post_data
     * @since 1.0.0
     */
    public function save_expt_log( $data, $post_data = '' ) {

        $bt = debug_backtrace();
        $caller = array_shift($bt);
        $file_Line = $caller['file'] .':'. $caller['line'];

        if( !empty( $data ) ) {
            $data = $this->expt_sanitize_array_data( $data );
        } else {
            return;
        }

        $this->expt_error_log( '[EXPT ERROR LOG]', $file_Line, 'Error Type => ' . $data[0] );
        $this->expt_error_log( '[EXPT ERROR LOG]', $file_Line, 'Error Message => ' . $data[1] );

        if( !empty( $post_data ) ) {

            $post_data = $this->expt_sanitize_array_data( $post_data );
            $post_data_string = json_encode($post_data);
            $this->expt_error_log( '[EXPT ERROR LOG]', $file_Line, 'Post Data => ' . $post_data_string );
        }

    }


    /**
     * Sanitization for text or array
     * 
     * @param $array_or_string (array|string)
     * @since 1.0.0
     * 
     * @return mixed
     */
    public function expt_sanitize_array_data( $array_or_string ) {

        if( is_string( $array_or_string ) ) {
            $array_or_string = sanitize_textarea_field( $array_or_string );

        } elseif( is_array( $array_or_string ) ) {
            
            foreach ( $array_or_string as $key => &$value ) {
                if ( is_array( $value ) ) {
                    $value = $this->expt_sanitize_array_data( $value );
                }
                else {
                    $value = sanitize_textarea_field( $value );
                }
            }
        }

        return $array_or_string;
    }


    /**
     * Escaping for text or array
     * 
     * @param $array_or_string (array|string)
     * @since 1.0.0
     * 
     * @return mixed
     */
    public function expt_escape_array_data( $array_or_string ) {

        if( is_string( $array_or_string ) ) {
            $array_or_string = esc_attr( $array_or_string );

        } elseif( is_array( $array_or_string ) ) {

            foreach ( $array_or_string as $key => &$value ) {
                if ( is_array( $value ) ) {
                    $value = $this->expt_escape_array_data( $value );
                }
                else {
                    $value = esc_attr( $value );
                }
            }
        }

        return $array_or_string;
    }


    /**
     * Error Log file for ExpertTexting plugin
     * 
     * @param $type, $file_Line, $log (optional)
     * @since 1.0.0
     * 
     * Write on file
     * @return none
     */
    public function expt_error_log( $type = '', $file_Line = '', $log = '') {

        if( ! $this->activated( 'et_general_set', 'et_general_log_chk' ) ) return;

        if( !$log ) {

            // Turn on error reporting.
            error_reporting( E_ALL );
            // Sets to display errors on screen. Use 0 to turn off.
            ini_set( 'display_errors', 0 );
            // Sets to log errors. Use 0 (or omit) to not log errors.
            ini_set( 'log_errors', 1 );
            // Sets a log file path.
            ini_set( 'error_log', $this->expt_error_log_file );

        } else {

            $msg_err = $type . ' [' . date('d-M-Y H:i:s e', current_time('timestamp')) . '] ';
            $msg_err .= $log.' , '.$file_Line.PHP_EOL;
	        error_log( $msg_err, 3, $this->expt_error_log_file );
        }

        if( file_exists( $this->expt_error_log_file ) ) {
            update_option( 'expttxt_log_' . date('d-M-Y', current_time('timestamp')), date('d-M-Y', current_time('timestamp')) .'_expt_error_log_file.log', 'yes' );
        } else {
            delete_option( 'expttxt_log_' . date('d-M-Y', current_time('timestamp')) );
        }

    }

    function checkFiles( $log_file = '' ) {

        global $wpdb;

        if( $log_file ) {

            $file = $this->expt_error_log_file_PATHURL . $log_file;
            if( file_exists( $file ) ) {

                $size = filesize( $file );

                $units  = array( 'B', 'KB', 'MB', 'GB', 'TB' );
                $factor = floor( ( strlen( $size ) - 1 ) / 3 );
                $size   = sprintf( '%.2f', $size / pow( 1024, $factor ) ) . ' ' . $units[ $factor ];
                
                return $size;
            }
        }

        $prepare_err_file = $wpdb->prepare("SELECT * FROM `{$wpdb->prefix}options` WHERE option_name LIKE %s", '%expttxt_log_%');
        $res = $wpdb->get_results($prepare_err_file);

        if($res) {
            foreach($res as $filename) {
                if( !file_exists( $this->expt_error_log_file_PATHURL . $filename->{'option_value'} ) ) {
                    delete_option( $filename->{'option_name'} );
                }
            }
        }
    }

    /**
     * Generate random string
     * 
     * @param int $length | Default: 6
     * @since 1.0.0
     * 
     * @return string
     */
    function generateRandomString($length = 6) {
		$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}

    /**
     * Get countries list
     *
     * @since 1.2.0
     * 
     * @return array|mixed|object
     */
    public function expt_countries() {

        $countries = $this->plugin_url . 'assets/expt_countries.json';
        $countries = file_get_contents($countries);
        $result = json_decode($countries, true);
        return $result;
    }

}