<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Base;

use ExptTxt\Expt_Base\Expt_BaseController;

class Expt_Features extends Expt_BaseController {

    protected $table_name;
    protected $wp_db;
    protected $__Expt_NotificationsCallbacks;

    public function register() {

        global $wpdb, $__Expt_NotificationsCallbacks;
        
        $this->wp_db = $wpdb;
		$this->table_name = $this->wp_db->prefix . 'usermeta';
        
        $this->__Expt_NotificationsCallbacks = $__Expt_NotificationsCallbacks;

        if ( $this->activated( 'et_menu', 'add_mobile_field' ) ) {

            add_action('register_form', array($this, 'mobileField_RegisterForm'));
            add_action('user_new_form', array($this, 'mobileField_NewUserForm'));
            add_filter('user_contactmethods', array($this, 'mobileField_ProfileForm'));
            add_filter('registration_errors', array($this, 'mobileRegistration_Errors'), 10, 3);
            add_action('user_register', array($this, 'mobile_SaveRegister'));

            add_action('user_register', array($this, 'etUserRegisterSendCode'));
            add_action('user_register', array($this, 'checkAdminDuplicateNumber'));
            add_action('profile_update', array($this, 'checkAdminDuplicateNumber'));

            add_filter('manage_users_columns', array($this, 'etManageUsersColumnsMobile'));
            add_filter('manage_users_custom_column', array($this, 'etManageUsersColumnsMobileContent'), 10, 3);

            // Enable verify field and message to login forms
			add_filter('login_message', array($this, 'etLoginVerifyMessage'));
			add_action('login_form', array($this, 'etLoginVerifyField'));
            // Check login
			add_action('wp_login', array($this, 'etCheckLoginVerification'), 10, 2);
			add_action('wp_login', array($this, 'etCheckLoginVerificationOTP'), 10, 2);
        }

        // User login with mobile
        add_action('login_form', array($this, 'etLoginWithMobile'));
        add_action('login_header', array($this, 'login_head_script'));
        // Custom columns in user's table
        add_filter('manage_users_columns', array($this, 'etManageUsersColumns'));
        add_filter('manage_users_custom_column', array($this, 'etManageUsersColumnsContent'), 10, 3);
        // Edit mobile in profile
        add_action('show_user_profile', array($this, 'etManageUserProfile'), 10, 1);
        add_action('edit_user_profile', array($this, 'etManageUserProfile'), 10, 1);
        add_action('personal_options_update', array($this, 'etManageUserProfileUpdate'));
        add_action('edit_user_profile_update', array($this, 'etManageUserProfileUpdate'));

        if ( $this->activated( 'et_menu', 'login_with_mobile' ) || $this->activated( 'et_menu', 'author_post_chk' ) ) {

            if ( ! $this->activated( 'et_menu', 'add_mobile_field' ) ) {
                $option = $this->expt_escape_array_data( get_option( 'et_menu' ) );
                $option['add_mobile_field'] = 1;
                update_option( 'et_menu', $option, 'yes' );
            }

        }

    }

    /**
     * Check whether mobile number exists
     * 
     * @param $mobile_number
     * @param $user_id
     */
    private function checkMobileNumber( $mobile_number, $user_id = null ) {

        $mobile_number = (int)$mobile_number;

        if($user_id) {
			$result = $this->wp_db->get_results("SELECT * from $this->table_name WHERE meta_key = 'expt_mobile' AND meta_value LIKE '%$mobile_number%' AND user_id != '$user_id'");
		} else {
			$result = $this->wp_db->get_results("SELECT * from $this->table_name WHERE meta_key = 'expt_mobile' AND meta_value LIKE '%$mobile_number%'");
		}

		if($result) {
			return true;
		} else {

            if( $this->wp_db->last_error ) {
                $this->save_expt_wpdb_log();
            }

			return false;
		}

	}

    /**
     * Delete mobile number
     * 
     * @param $user_id
     */
    private function deleteUserMobileNumber( $user_id ) {
		$res = $this->wp_db->delete(
			$this->table_name,
			array(
				'user_id'	=>	$user_id,
				'meta_key'	=>	'expt_mobile',
			)
		);

        if(!$res) {

            if( $this->wp_db->last_error ) {
                $this->save_expt_wpdb_log();
            }
        }
	}

    /**
     * Add mobile field to register form
     * 
     */
    public function mobileField_RegisterForm() {
        
		$mobile = ( isset( $_POST['expt_mobile'] ) ) ? sanitize_text_field( $_POST['expt_mobile'] ): '';
		
        ?>
        <p>
            <label for="expt_mobile"><?php _e('Your Mobile Number', 'experttexting-official') ?><br />
            <input type="tel" name="expt_mobile" id="expt_mobile" class="input" value="<?php esc_attr_e(stripslashes($mobile)); ?>" size="25" maxlength="20" /></label>
        </p>
        <?php

        // If Intl Mobile Input active
        if ( $this->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
            _e('<script>
            var element = document.getElementById("expt_mobile");
            element.classList.add("expttxt-intl-input");
            </script>');
        }
        
	}

    /**
     * Add mobile field to new user form
     * 
     */
    public function mobileField_NewUserForm(){
		
        ?>
        <table class="form-table">
            <tr>
                <th><label for="expt_mobile"><?php _e('Mobile Number', 'experttexting-official'); ?></label></th>
                <td>
                    <input type="tel" class="regular-text" name="expt_mobile" value="" id="expt_mobile" maxlength="20" />
                </td>
            </tr>
        </table>
        <?php

        // If Intl Mobile Input active
        if ( $this->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
            _e('<script>
            var element = document.getElementById("expt_mobile");
            element.classList.add("expttxt-intl-input");
            </script>');
        }

	}

    /**
     * Add mobile field to profile form
     * 
     * @param $fields
     */
    public function mobileField_ProfileForm( $fields ) {
        
		$fields['expt_mobile'] = __('Mobile Number', 'experttexting-official');

        $current_user_id = get_current_user_id();
        $checkVerify = get_user_meta( $current_user_id, 'expttxt_verified', true );
        $codeVerify = get_user_meta( $current_user_id, 'expttxt_verify_code', true );
        $mobileNum = get_user_meta( $current_user_id, 'expt_mobile', true );
        
        if( !$checkVerify && $codeVerify && !empty( $mobileNum ) ) { 
            
            $fields['expt_mobile_code'] = __('Enter Mobile Verification Code', 'experttexting-official');
        }
        
		return $fields;
	}

    /**
     * Mobile registration errors
     * 
     * @param $errors
     * @param $sanitized_user_login
     * @param $user_email
     */
    public function mobileRegistration_Errors( $errors, $sanitized_user_login, $user_email ) {
        
        $check = isset( $_POST['expt_mobile'] ) ? $this->checkMobileNumber( (int)$_POST['expt_mobile'] ) : "";
        
		if ( empty( $_POST['expt_mobile'] ) ) {
			$errors->add( 'empty_mobile_error', __('<strong>Error</strong>: You must include a mobile number.', 'experttexting-official') );
		} elseif( $this->activated( 'et_menu', 'intl_tel_input_chk' ) && empty( $_POST['et_sms_mob_dial_code'] ) ) {
            // If Intl Mobile Input active
            $errors->add( 'invalid_mobile_number', __('<strong>Error</strong>: The entered mobile number is invalid, please enter the correct number.', 'experttexting-official') );
		} elseif( $check ) {
			$errors->add( 'duplicate_mobile_number', __('<strong>Error</strong>: The entered mobile number is already registered, please choose another one.', 'experttexting-official') );
		}

		return $errors;
	}

    /**
     * Save/Register mobile number
     * 
     * @param $user_id
     */
    public function mobile_SaveRegister( $user_id ) {
        
		if ( isset( $_POST['expt_mobile'] ) ) {

            $expt_mobile = sanitize_text_field( $_POST['expt_mobile'] );

            // If Intl Mobile Input active
            if ( $this->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
                $expt_mobile = sanitize_text_field( $_POST['et_sms_mob_dial_code'] );
            }
			update_user_meta( $user_id, 'expt_mobile', $expt_mobile );
			update_user_meta( $user_id, 'expttxt_user_register', 1 );
		}

	}

    /**
     * Check whether admin's number is duplicate
     * 
     * @param $user_id
     */
    public function checkAdminDuplicateNumber( $user_id ) {

        // Get user mobile
		$user_mobile = get_user_meta( $user_id, 'expt_mobile', true );
		$user_mobile_dialcode = get_user_meta( $user_id, 'et_sms_mob_dial_code', true );
        
		if( empty( $user_mobile ) ) {
			return;
		}

        // If Intl Mobile Input active
        if( $this->activated( 'et_menu', 'intl_tel_input_chk' ) && empty( $user_mobile_dialcode ) ) {
            update_user_meta( $user_id, 'expttxt_number_invalid', 1 );
        } elseif( $this->checkMobileNumber( $user_mobile, $user_id ) ) {
            // Delete user mobile
            $this->deleteUserMobileNumber( $user_id );
            delete_user_meta( $user_id, 'et_sms_mob_dial_code');
            update_user_meta( $user_id, 'expttxt_number_exist', 1 );
		} else {
            update_user_meta( $user_id, 'expt_mobile', $user_mobile_dialcode );
            $expttxt_user_register = get_user_meta( $user_id, 'expttxt_user_register', true );
            if ( $expttxt_user_register && !is_user_logged_in() ) {
                delete_user_meta( $user_id, 'expttxt_user_register');
                // Redirect to verification
                wp_redirect( wp_login_url() . '?reg=true&mobverification=yes' );
                exit();
            }
        }
	}

    /**
     * Adds mobile column to user's column
     * 
     * @param $columns
     */
    public function etManageUsersColumnsMobile( $columns ) {
        $new_columns = array(
            'expttxt_mobile' => __('Mobile Number', 'experttexting-official'),
        );
     
        return array_merge( $columns, $new_columns );
    }

    /**
     * Adds value to the custom added mobile column
     * 
     * @param $value
     * @param $column_name
     * @param $user_id
     */
    public function etManageUsersColumnsMobileContent( $value, $column_name, $user_id ) {

        if ( 'expttxt_mobile' == $column_name ) {
            $mobileNum = get_user_meta( $user_id, 'expt_mobile', true );
            $output = $mobileNum ? $mobileNum : '—';
            return $output;
        }
        return $value;
    }

    /**
     * User login with mobile
     * 
     * @param $subscriber
     */
    public function etLoginWithMobile( $subscriber ) {

        if ( ! $this->activated( 'et_menu', 'login_with_mobile' ) ) return;

        ?>

        <div class="bootstrap-et">
            <div class="text-center mb-3">
                <h6>-- OR --</h6>
            </div>
            <a href="#" class="button w-100 clearfix mb-3 py-1 text-center" onclick="exptPopup('<?php _e( $this->plugin_url ) ?>ExptTemplates/et_login_with_phone.php', 'Login', 600, 300); return false;">
                <span class="h6 fw-normal"><?php _e('Login with Mobile', 'experttexting-official'); ?> <span class="dashicons dashicons-phone h5"></span></span>
            </a>
        </div>

        
        <?php
        
    }

    /**
     * User login add CSS & JS to wp login head
     * 
     */
    public function login_head_script() {

        wp_enqueue_style( 'bootstrap', $this->plugin_url . 'assets/bootstrap/bootstrap.et.css' );
        ?>
        <script>
            function exptPopup(myURL, title, myWidth, myHeight) {
                var left = (screen.width - myWidth) / 2;
                var top = (screen.height - myHeight) / 4;
                window.open(myURL, title, 'toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=no, resizable=no, copyhistory=no, width=' + myWidth + ', height=' + myHeight + ', top=' + top + ', left=' + left);
            }
        </script>
        <?php
    }

    /**
     * Manage user's column
     * 
     * @param $columns
     * 
     * @return array
     */
    public function etManageUsersColumns( $columns ) {

        $new_columns = array(
            'expttxt_verified' => __('Mobile Verification Status', 'experttexting-official')
        );
     
        return array_merge( $columns, $new_columns );
    }

    /**
     * Adds content to the custom added columns above
     * 
     * @param $value, $column_name, $user_id
     * 
     * @return $value
     */
    public function etManageUsersColumnsContent( $value, $column_name, $user_id ) {

        if ( 'expttxt_verified' == $column_name ) {

            $checkVerify = get_user_meta( $user_id, 'expttxt_verified', true );
            $output = $checkVerify ? __( '<span style="color: green;">' . __('Verified', 'experttexting-official') . '</span>' ) : __( '<span style="color: red;">' . __('Not Verified', 'experttexting-official') . '</span>' );
            return $output;
        }
        return $value;
    }

    /**
     * Update fields and validations when profile loads
     * 
     * @param $user
     */
    public function etManageUserProfile( $user ) {
        
        global $__Expt_Notices;

        // If Intl Mobile Input active
        if ( $this->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
            _e('<script>
            var element = document.getElementById("expt_mobile");
            element.classList.add("expttxt-intl-input");
            </script>');
        }
        
        $checkVerify = get_user_meta( $user->ID, 'expttxt_verified', true );
        $user_mobile = get_user_meta( $user->ID, 'expt_mobile', true );
        
        if ( current_user_can( 'manage_options' ) ) {
            
            $checkOtherUserVerify = get_user_meta( $user->ID, 'expttxt_verified', true );
            $checkOtherUserVerify = $checkOtherUserVerify ? '"checked"' : 0;

            // Verify Checkbox for Admin
            _e('<script>
                function verifyCheckbox() {
                    x = document.getElementById("expt_mobile");
                    br = document.createElement("br");
                    br2 = document.createElement("br");
                    chkbox = document.createElement("input");
                    chkbox.type = "checkbox";
                    chkbox.name = "expttxt_verified";
                    chkbox.id = "expttxt_verified";
                    chkbox.value = "1";
                    '.( !$user_mobile ? 'chkbox.disabled = true;' : '' ).'
                    chkbox.checked = '. $checkOtherUserVerify .';
                    chkbox.style.marginBottom = "4px";
                    txt = document.createTextNode("Verify Number");
                    x.parentNode.insertBefore(txt, x.nextSibling);
                    x.parentNode.insertBefore(chkbox, x.nextSibling);
                    x.parentNode.insertBefore(br, x.nextSibling);
                    x.parentNode.insertBefore(br2, x.nextSibling);
                }
                verifyCheckbox();
            </script>');
        }

        // Check verification of mobile number
        if( $checkVerify ) {
            // Show "verified"
            _e('<script>
                function verifySpan() {
                    x = document.getElementById("expt_mobile");
                    sp = document.createElement("span");
                    sp.style.color = "green";
                    sp.style.marginLeft = "10px";
                    sp.innerHTML = "Verified";
                    x.parentNode.insertBefore(sp, x.nextSibling);
                }
                verifySpan();
            </script>');
        } elseif( $user_mobile && !$checkVerify ) {
            // Show "not verified"
            _e('<script>
                function verifySpan() {
                    x = document.getElementById("expt_mobile");
                    sp = document.createElement("span");
                    sp.style.color = "red";
                    sp.style.marginLeft = "10px";
                    sp.innerHTML = "Not Verified";
                    x.parentNode.insertBefore(sp, x.nextSibling);
                }
                verifySpan();
            </script>');

        }

        $codeVerify = get_user_meta( $user->ID, 'expttxt_verify_code', true );
        $codeVerifyEntered = get_user_meta( $user->ID, 'expt_mobile_code', true );
        $checkDupli = get_user_meta( $user->ID, 'expttxt_number_exist', true );
        $checkInvalid = get_user_meta( $user->ID, 'expttxt_number_invalid', true );

        // If Intl Mobile Input active
        if( $checkInvalid ) {
            $__Expt_Notices->notice( __( 'The entered mobile number is invalid.', 'experttexting-official' ), "error", true, true, true );
            delete_user_meta( $user->ID, 'expttxt_verify_code');
            delete_user_meta( $user->ID, 'expt_mobile_code');
            delete_user_meta( $user->ID, 'expttxt_number_invalid');
        } elseif( $checkDupli ) {
            $__Expt_Notices->notice( __( 'The entered mobile number is already exist.', 'experttexting-official' ), "error", true, true, true );
            delete_user_meta( $user->ID, 'expttxt_verify_code');
            delete_user_meta( $user->ID, 'expt_mobile_code');
            delete_user_meta( $user->ID, 'expttxt_number_exist');
        } elseif( !empty( $codeVerifyEntered ) && $codeVerifyEntered != $codeVerify ) {
            $__Expt_Notices->notice( __( 'The entered verification code is invalid.', 'experttexting-official' ), "error", true, true, true );
            delete_user_meta( $user->ID, 'expt_mobile_code');
        } elseif( !empty( $codeVerifyEntered ) && $codeVerifyEntered == $codeVerify ) {
            $__Expt_Notices->notice( __( 'Your mobile number is now verified.', 'experttexting-official' ), "success", true, true, true );
        } elseif( $codeVerify && $user_mobile ) {
            $__Expt_Notices->notice( __( 'Verification code has been sent to your mobile number. Enter the verification code.', 'experttexting-official' ), "info", true, true, true );
        }

        $IsCodeEntry = get_user_meta( $user->ID, 'expttxt_verify_code_entry', true );
        if( $IsCodeEntry && !isset( $_GET['resendCode'] ) ) {

            // Resend code button
            _e('<script>
                function resendAnchor() {
                    x = document.getElementById("expt_mobile_code");
                    btn = document.createElement("a");
                    btn.href = "'. esc_attr( admin_url("profile.php?resendCode=true") ) . '";
                    btn.className = "button";
                    btn.style.marginLeft = "10px";
                    btn.innerHTML = "Resend Code";
                    x.parentNode.insertBefore(btn, x.nextSibling);
                }
                resendAnchor();
            </script>');
        }

        if( $IsCodeEntry && isset( $_GET['resendCode'] ) && $_GET['resendCode'] == 'true' ) {

            delete_user_meta( $user->ID, 'expttxt_verify_code_entry');

            $this->etUserRegisterSendCode( $user->ID );
            wp_redirect( admin_url('profile.php') );
        }

        // If optional 2FA selected, show checkbox to users to enable/disable
        if( $this->activated( 'et_menu', 'login_2FA' ) && $this->activated( 'et_menu', 'login_2FA_method' )[0] == 'optional_enable_disable_2fa' ) {

            $enabled_otp = get_user_meta( $user->ID, 'expt_enable_otp', true );

            ?>

            <h3><?php _e( 'Two-Factor Authentication', 'experttexting-official' ); ?></h3>
            <table class="form-table">
                <tr>
                    <th>
                        <label for="expt_2fa_otp"><?php _e( 'Two-Factor Authentication', 'experttexting-official' ); ?></label>
                    </th>
                    <td>
                        <input name="expt_2fa_otp" type="checkbox" id="expt_2fa_otp"
                            value="1" <?php checked( $enabled_otp, '1' ); ?> style="margin-bottom: 4px;">
                        <?php _e( __('Enable', 'experttexting-official') .'<br> <p class="description">'. __('Add an extra layer of security to your account in case your password is stolen.', 'experttexting-official') .'</p>' ); ?>
                    </td>
                </tr>
            </table>

            <?php
        }
    }

    /**
     * Update fields and validations when profile loads
     * 
     * @param $user_id
     */
    public function etManageUserProfileUpdate( $user_id ) {

        $get_user_mobile = get_user_meta( $user_id, 'expt_mobile', true );
        $codeVerify = get_user_meta( $user_id, 'expttxt_verify_code', true );

        if( isset( $_POST['expt_mobile'] ) && (int)$_POST['expt_mobile'] != (int)$get_user_mobile ) {

            // If Intl Mobile Input active
            if( $this->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
                update_user_meta( $user_id, 'et_sms_mob_dial_code', sanitize_text_field( $_POST['et_sms_mob_dial_code'] ) );
            }

            update_user_meta( $user_id, 'expttxt_verified', 0 );

            $code = $this->generateRandomString();
            $user_mobile = sanitize_text_field( $_POST['expt_mobile'] );
            $message = "Your mobile number verification code is " . $code;

            update_user_meta( $user_id, 'expttxt_verify_code', $code );

            if ( $user_mobile ) {

                $this->__Expt_NotificationsCallbacks->sendMessageReq( $message, array($user_mobile) );
            }

        }

        if ( isset( $_POST['expt_mobile_code'] ) ) {

            $expt_mobile_code_input = sanitize_text_field( $_POST['expt_mobile_code'] );
            !empty( $expt_mobile_code_input ) ? update_user_meta( $user_id, 'expttxt_verify_code_entry', 1 ) : "";

            // Check if entered code is correct
            if( !empty( $expt_mobile_code_input ) && $expt_mobile_code_input == $codeVerify ) {

                update_user_meta( $user_id, 'expttxt_verified', 1 );
                delete_user_meta( $user_id, 'expttxt_verify_code');
                delete_user_meta( $user_id, 'expttxt_verify_code_entry');
                delete_user_meta( $user_id, 'expt_mobile_code');
                delete_user_meta( $user_id, 'et_sms_mob_dial_code');
                
            } else {
                
                update_user_meta( $user_id, 'expttxt_verified', 0 );
            }
		}

        // Check if Admin verify the user's number
        if ( current_user_can( 'manage_options' ) && empty( $_POST['expt_mobile_code'] ) ) {

            if ( isset( $_POST['expttxt_verified'] ) ) {

                update_user_meta( $user_id, 'expttxt_verified', 1 );
                delete_user_meta( $user_id, 'expttxt_verify_code');
                delete_user_meta( $user_id, 'expttxt_verify_code_entry');
                delete_user_meta( $user_id, 'expt_mobile_code');
                delete_user_meta( $user_id, 'et_sms_mob_dial_code');

            } else {
                
                update_user_meta( $user_id, 'expttxt_verified', 0 );
            }
        }
        
        // Check if user enable 2FA
        if( isset( $_POST['expt_2fa_otp'] ) ) {

            update_user_meta( $user_id, 'expt_enable_otp', 1 );
            
        } else {
            
            update_user_meta( $user_id, 'expt_enable_otp', 0 );
        }

    }

    /**
     * Send code when register user
     * 
     * @param $user_id
     */
    public function etUserRegisterSendCode( $user_id ) {
		// Get user mobile
		$user_mobile = get_user_meta( $user_id, 'expt_mobile', true );

		if( empty( $user_mobile ) ) {
			return;
		}

        $code = $this->generateRandomString();
        $message = "Your mobile number verification code is " . $code;

        update_user_meta( $user_id, 'expttxt_verify_code', $code );

        // Send code
		if( !$this->checkMobileNumber( $user_mobile, $user_id ) ) {
            $this->__Expt_NotificationsCallbacks->sendMessageReq( $message, array($user_mobile) );
        }
	}

    /**
     * Send code OTP for Login
     * 
     * @param $user_id
     */
    public function etUserSendCodeOTP( $user_id ) {

        global $__Expt_NotificationsCallbacks;
        
		// Get user mobile
		$user_mobile = get_user_meta( $user_id, 'expt_mobile', true );

		if( empty( $user_mobile ) ) {
			return;
		}

        $code = $this->generateRandomString();

        $userdata = get_userdata( $user_id );

        // Send code
		if( !$this->checkMobileNumber( $user_mobile, $user_id ) ) {
            $__Expt_NotificationsCallbacks->etLoginWithMobileCallback( $code, $user_mobile, $userdata, 'login_2FA_body' );
        }
	}

    /**
     * Login page messages
     * 
     * @param $user
     */
    public function etLoginVerifyMessage( $message ) {

		if ( empty( $_GET['mobverification'] ) ) {
			return $message;
		}

		if ( empty( $message ) ) {

            if( isset( $_GET['reg'] ) && $_GET['reg'] == 'true' ) {
                $message .= '<div class="message">' . __('Registration complete. Please check your email for Login details and also check your mobile for verification code.', 'experttexting-official') . '<br></div>';
            }
            
            if( isset( $_GET['mobverification'] ) && isset( $_GET['mobverifycode'] ) ) {

                if( $_GET['mobverifycode'] == 'false' ) {
                    $message .= '<div class="message" id="login_error">' . __('Verification code is invalid.', 'experttexting-official') . '<br></div>';
                }
                
            } elseif( isset( $_GET['mobverification'] ) && isset( $_GET['mobverificationcode'] ) ) {

                $message .= '<div class="message" id="login_error">' . __('Verification code sent limit reached. Try again later.', 'experttexting-official') . '<br></div>';

            } elseif( isset( $_GET['mobverification'] ) && isset( $_GET['expt_login'] ) ) {

                $message .= '<div class="message">' . __('Verification code has been sent to your mobile number. Please enter verification code below to login to WordPress.', 'experttexting-official') . '<br></div>';

            } else {

                $message .= '<div class="message">' . __('Verification code has been sent to your mobile number. Please enter verification code below to verify your mobile number.', 'experttexting-official') . '<br></div>';
            }

			return $message;

		} else {
			return $message;
		}
	}

    /**
     * Show mobile verification code field in Login form
     * 
     * @param $user
     */
    public function etLoginVerifyField() {

		if ( empty( $_GET['mobverification'] ) ) {
			return;
		}

        _e(
            '<label for="expttxt_verify_code">' . __( 'Mobile Verification Code', 'experttexting-official' ) . '</label>
            <input type="text" name="expttxt_verify_code" id="expttxt_verify_code" class="input">'
        );
	}

    /**
     * Check and verify the number when first time the user loggedin.
     * 
     * @param $user
     */
    public function etCheckLoginVerification( $user_login, $user ) {

		$checkVerify = get_user_meta( $user->ID, 'expttxt_verified', true );
		$codeVerify = get_user_meta( $user->ID, 'expttxt_verify_code', true );
		$mobileNum = get_user_meta( $user->ID, 'expt_mobile', true );
        
		if( $checkVerify == 1 || empty( $mobileNum ) ) return;

		if( isset( $_POST['expttxt_verify_code'] ) ) {

            if( $_POST['expttxt_verify_code'] == $codeVerify ) {

                update_user_meta( $user->ID, 'expttxt_verified', 1 );
                delete_user_meta( $user->ID, 'expttxt_verify_code');
                delete_user_meta( $user->ID, 'expt_mobile_code');
                delete_user_meta( $user->ID, 'et_sms_mob_dial_code');
				wp_redirect( get_edit_profile_url() );
				exit();
			} else {
                wp_redirect( wp_login_url() . '?mobverification=yes&mobverifycode=false' );
				exit();
            }
		}
        
        $expttxt_code_count = get_user_meta( $user->ID, 'expttxt_code_count', true );

        $et_menu = $this->expt_escape_array_data( get_option( 'et_menu' ) );
        $login_with_mobile_retries = !empty($et_menu) ? sanitize_textarea_field( $et_menu['login_with_mobile_retries'] ) : '';
        $login_with_mobile_retries_reset = !empty($et_menu) ? sanitize_textarea_field( $et_menu['login_with_mobile_retries_reset'] ) : '';

        $login_with_mobile_retries = !empty( $login_with_mobile_retries ) ? ( ( $login_with_mobile_retries > 0 ) ? $login_with_mobile_retries : 2 ) : 2;
        $login_with_mobile_retries_reset = !empty( $login_with_mobile_retries_reset ) ? ( ( $login_with_mobile_retries_reset > 0 ) ? $login_with_mobile_retries_reset : 2 ) : 2;

        $chk = $this->ettimecheck( 'et_send_code_limit_time' . $user->ID, ( $login_with_mobile_retries_reset * 60 ) );
        
        // Checking the code sent limit
        if( !$expttxt_code_count ) {
            update_user_meta( $user->ID, 'expttxt_code_count', 1 );
            update_option( "et_send_code_limit_time" . $user->ID, date('Y-m-d H:i:s'), "yes" );

        } elseif( $expttxt_code_count > $login_with_mobile_retries ) {
            
            if( !$chk ) {
                wp_redirect( wp_login_url() . '?mobverification=yes&mobverificationcode=false' );
                exit();
            } else {
                delete_user_meta( $user->ID, 'expttxt_code_count');
            }

        } else {
            update_user_meta( $user->ID, 'expttxt_code_count', ++$expttxt_code_count );
            update_option( "et_send_code_limit_time" . $user->ID, date('Y-m-d H:i:s'), "yes" );
        }

        $this->etUserRegisterSendCode( $user->ID );
		wp_redirect( wp_login_url() . '?mobverification=yes' );
		exit();
	}

    /**
     * Check and verify the authentication using OTP when the user loggedin and also if 2FA enabled.
     * 
     * @param $user
     */
    public function etCheckLoginVerificationOTP( $user_login, $user ) {

		$checkVerify = get_user_meta( $user->ID, 'expttxt_verified', true );
        $codeVerify = get_user_meta( $user->ID, 'expttxt_verify_code', true );
        $expt_enable_otp = get_user_meta( $user->ID, 'expt_enable_otp', true );
        $mobileNum = get_user_meta( $user->ID, 'expt_mobile', true );
        
		if( $checkVerify != 1 || empty( $mobileNum ) || ! $this->activated( 'et_menu', 'login_2FA' ) ) return;
        
        $expttxt_otp_count = get_user_meta( $user->ID, 'expttxt_otp_count', true );

		if( ( $this->activated( 'et_menu', 'login_2FA_method' )[0] == 'enable_for_all_2fa' ) || ( $this->activated( 'et_menu', 'login_2FA_method' )[0] == 'optional_enable_disable_2fa' && $expt_enable_otp == 1 ) ) {

            if( $this->activated( 'et_menu', 'login_2FA_runtime' )[0] == 'just_once_2fa' ) {

                if( $expttxt_otp_count >= 1 ) {
                    
                    return;
                }

            } else {

                delete_user_meta( $user->ID, 'expttxt_otp_count');
            }

            if( isset( $_POST['expttxt_verify_code'] ) ) {

                if( $_POST['expttxt_verify_code'] == $codeVerify ) {

                    update_user_meta( $user->ID, 'expttxt_otp_count', 1 );
                    delete_user_meta( $user->ID, 'expttxt_verify_code');
                    wp_redirect( get_edit_profile_url() );
                    exit();
    
                } else {
    
                    wp_redirect( wp_login_url() . '?mobverification=yes&mobverifycode=false' );
                    exit();
                }

            }
		} else {

            return;
        }
        
        $expttxt_code_count = get_user_meta( $user->ID, 'expttxt_code_count', true );

        $et_menu = $this->expt_escape_array_data( get_option( 'et_menu' ) );
        $login_2FA_retries = !empty($et_menu) ? sanitize_textarea_field( $et_menu['login_2FA_retries'] ) : '';
        $login_2FA_retries_reset = !empty($et_menu) ? sanitize_textarea_field( $et_menu['login_2FA_retries_reset'] ) : '';

        $login_2FA_retries = !empty( $login_2FA_retries ) ? ( ( $login_2FA_retries > 0 ) ? $login_2FA_retries : 2 ) : 2;
        $login_2FA_retries_reset = !empty( $login_2FA_retries_reset ) ? ( ( $login_2FA_retries_reset > 0 ) ? $login_2FA_retries_reset : 2 ) : 2;

        $chk = $this->ettimecheck( 'et_send_code_limit_time' . $user->ID, ( $login_2FA_retries_reset * 60 ) );
        
        // Checking the code sent limit
        if( !$expttxt_code_count ) {
            update_user_meta( $user->ID, 'expttxt_code_count', 1 );
            update_option( "et_send_code_limit_time" . $user->ID, date('Y-m-d H:i:s'), "yes" );

        } elseif( $expttxt_code_count > $login_2FA_retries ) {
            
            if( !$chk ) {
                wp_redirect( wp_login_url() . '?mobverification=yes&mobverificationcode=false' );
                exit();
            } else {
                delete_user_meta( $user->ID, 'expttxt_code_count');
            }

        } else {
            update_user_meta( $user->ID, 'expttxt_code_count', ++$expttxt_code_count );
            update_option( "et_send_code_limit_time" . $user->ID, date('Y-m-d H:i:s'), "yes" );
        }

        $this->etUserSendCodeOTP( $user->ID );
		wp_redirect( wp_login_url() . '?expt_login=true&mobverification=yes' );
		exit();
	}

}