<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Base;

use ExptTxt\Expt_Base\Expt_BaseController;

class Expt_Notices {

    private $text, $model, $close_button, $echo, $foruser, $style;
    protected $check_api_notice = '0';
    protected $check_api_notice_issue = '0';
    protected $check_admin_number_notice = '0';

    /**
     * Show Admin Wordpress Ui Notice
     *
     * @param string $text where Show Text Notification
     * @param string $model Type Of Model from list : error / warning / success / info
     * @param boolean $close_button Check Show close Button Or false otherwise
     * @param boolean $echo Check Echo or return in function
     * @param boolean $foruser Check notice for user or false otherwise
     * @param string $style add extra Css Style To Code
     *
     * @return string Wordpress html Notice code
     */
    
    function notice( $text, $model, $close_button = true, $echo = true, $foruser = false, $style = '' ) {

        global $__Expt_BaseController;

        $this->__Expt_BaseController = $__Expt_BaseController;

        $this->text = $text;
        $this->model = $model;
        $this->close_button = $close_button;
        $this->echo = $echo;
        $this->foruser = $foruser;
        $this->style = $style;

        add_action( 'init', array( $this->notice_display() ) );
    }

    public function notice_display() {

        if( !$this->foruser && ( !current_user_can( 'manage_options' ) || !is_admin() || ( isset( $_GET['page'] ) && $_GET['page'] === 'et-plugin-welcome' ) ) ) return;

        $notice_output = '<div class="expt_notice notice notice-' . $this->model . ' ' . ($this->close_button === true ? "is-dismissible" : "") . '" style="display: flex; align-items: center;">';
        $notice_output .= $this->model == 'success' ? '<span class="dashicons success dashicons-yes-alt"></span>' : '<span class="' . $this->model . ' dashicons dashicons-warning"></span>';
        $notice_output .= '<p style="' . $this->style . '"><b>[ExpertTexting]:</b> ' . $this->text . '</p></div>';
        
        if ($this->echo) {
            _e( wp_kses( $notice_output, $this->__Expt_BaseController->allowed_html ) );
        } else {
            printf( wp_kses( $notice_output, $this->__Expt_BaseController->allowed_html ) );
        }

    }

    public function api_notice() {
        
        if( $this->check_api_notice != '1' ) {
            $this->notice( '<b>' . esc_attr( get_option( 'etApiAuth' ) ) . '</b> ' . __( 'Please configure the API settings by', 'experttexting-official' ) . ' <a href="' . esc_attr( admin_url("admin.php?page=et_settings&sub=api") ) . '">' . __( 'clicking here', 'experttexting-official' ) . '</a>', 'error' );
            
            $this->check_api_notice = '1';
        }

    }

    public function api_notice_issue() {

        if( $this->check_api_notice_issue != '1' ) {
            $this->notice( __( 'An error occurred while configuring your API, please try again later.', 'experttexting-official' ), 'error' );

            $this->check_api_notice_issue = '1';
        }

    }

    public function admin_number_notice() {

        if( $this->check_admin_number_notice != '1' ) {
            $this->notice( __( 'It seems like you have opted to receive SMS notifications from WP notifications and integrations, but you haven\'t added admin number yet. Add admin number to get SMS notifications from this plugin by', 'experttexting-official' ) . ' <a href="' . esc_attr( admin_url("admin.php?page=et_settings&sub=general") ) . '">' . __( 'clicking here', 'experttexting-official' ) . '</a>', 'error' );

            $this->check_admin_number_notice = '1';
        }

    }

}