<?php 
/**
 * @package  ExpertTextingOfficial
 */
namespace ExptTxt\Expt_Base;

use ExptTxt\Expt_Base\Expt_BaseController;
use ExptTxt\Expt_Api\Expt_Widgets\Expt_Widget;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_ApiCallbacks;
use ExptTxt\Expt_Api\Expt_Callbacks\Expt_NotificationsCallbacks;

/**
* 
*/
class Expt_WidgetController extends Expt_BaseController {

	public function register() {

		if ( ! $this->activated( 'et_menu', 'check_widget' ) ) return;
		add_shortcode('expt-sub-form', array($this, 'register_shortcode'));

		// Here we register our "handle_subscribe_form" function to handle our AJAX request.
		add_action('wp_ajax_handle_subscribe_form', array( $this, 'handle_subscribe_form' ) ); // This is for authenticated users
		add_action('wp_ajax_nopriv_handle_subscribe_form', array( $this, 'handle_subscribe_form' ) ); // This is for unauthenticated users.

		add_action('wp_ajax_handle_subscribe_form_activation', array( $this, 'handle_subscribe_form_activation' ) ); // This is for authenticated users
		add_action('wp_ajax_nopriv_handle_subscribe_form_activation', array( $this, 'handle_subscribe_form_activation' ) ); // This is for unauthenticated users.
        
        $et_widget = new Expt_Widget();
		$et_widget->register();
	}
	

	/**
     * Plugin Shortcode
     *
     * @param $atts
     *
     * @return false|string
     * @internal param param $Not
     */
    public function register_shortcode( $atts ) {
		
		static $i = 9999;

		$v['widget_id'] = 'et_widget-' . $i;

		ob_start();

        $et_widget = new Expt_Widget();
		$et_widget->subscriber_form( $atts, $v );

		$i++;

		return ob_get_clean();
    }


	/**
	 * Activation Check Function
	 * In this function we will handle the form inputs and send to DB.
	 *
	 * @return void
	 */
	
	function handle_subscribe_form_activation() {
	
		// This is a secure process to validate if this request comes from a valid source.
		if (! DOING_AJAX || ! check_ajax_referer('et-subscribe-active-nonce', 'nonce') ) {
			return $this->return_json('error');
		}
		
		global $wpdb;

		$activation_code = sanitize_text_field($_POST["et_sms_activation_code"]);
		
		$data = array(
			'activate_key' => $activation_code
		);

		$formSubmit = $this->form_submit_activation($data);

		if ( $formSubmit ) {

			return $this->return_json($formSubmit);
		}

		return $this->return_json('error');
	}


	public function form_submit_activation( $data ) {

		global $wpdb, $__Expt_NotificationsCallbacks;

		$table_name = $wpdb->prefix . 'et_subscribes';

		$rowcount = $wpdb->get_var("SELECT COUNT(*) FROM $table_name WHERE activate_key = '{$data["activate_key"]}' AND status = 0");
		$res = $wpdb->get_row("SELECT * FROM $table_name WHERE activate_key = '{$data["activate_key"]}' AND status = 0");

		if ( $rowcount > 0 ) {

			$wpdb->update( $table_name, array('status' => 1), array('activate_key' => $data["activate_key"]) );
			
			$__Expt_NotificationsCallbacks->onUserSubscribesNewsletter( $res );
			
			ob_clean();
			
			return 'success';
			
		} else {

			if( $wpdb->last_error ) {
				$this->save_expt_wpdb_log();
			}

			ob_clean();

			return 'invalid-active-code';
		}

		return false;

	}

	/**
	 * In this function we will handle the form inputs and send to DB.
	 *
	 * @return void
	 */
	
	function handle_subscribe_form() {
		
		// This is a secure process to validate if this request comes from a valid source.
		if (! DOING_AJAX || ! check_ajax_referer('et-subscribe-nonce', 'nonce') ) {
			return $this->return_json('error');
		}
		
		/**
		 * First we make some validations, 
		 */
		global $wpdb;
		
		$name = sanitize_text_field( $_POST["et_sms_name"] );
		$mob = sanitize_text_field( $_POST["et_sms_mob"] );
		$dialcode = isset($_POST["et_sms_mob_dial_code"]) && !empty($_POST["et_sms_mob_dial_code"]) ? sanitize_text_field( $_POST["et_sms_mob_dial_code"] ) : '';
		$group = isset($_POST["et_sms_group"]) ? sanitize_text_field( $_POST["et_sms_group"] ) : '';
		$subscriber = sanitize_title( $_POST["et_sms_subscribers"] );
		
		if ( $this->activated( 'et_menu', 'verify_subscriber' ) ) {
			
			$activation_code = $this->generateRandomString();
			$status = 0;
		} else {
			
			$activation_code = NULL;
			$status = 1;
		}
		
		$data = array(
			'ID' => NULL,
			'date' => date('Y-m-d H:i:s', current_time('timestamp')),
			'name' => $name,
			'mobile' => $mob,
			'dialcode' => $dialcode,
			'status' => $status,
			'activate_key' => $activation_code,
			'group_ID' => $group ? $group : 0,
		);
		
		$formSubmit = $this->form_submit($data, $subscriber);
		
		if ( $formSubmit ) {
			
			return $this->return_json($formSubmit);
		}
		
		return $this->return_json('error');
	}

	
	public function return_json($status) {
		$return = array(
			'status' => $status
		);
		wp_send_json($return);

		wp_die();
	}

	
	public function form_submit( $data, $sub ) {
		
		global $wpdb, $__Expt_NotificationsCallbacks, $__Expt_ApiCallbacks, $__Expt_BaseController;

		$table_name = $wpdb->prefix . 'et_subscribes';

		$mobLen = strlen($data["mobile"]);
		$dialcode = $data["dialcode"];
		$etNewsletter = $this->expt_escape_array_data( get_option( 'et_newsletter' ) );
		$mobNoMin = isset($etNewsletter['newsletter_mobile_min']) ? esc_attr( $etNewsletter['newsletter_mobile_min'] ) : '';
		$mobNoMax = isset($etNewsletter['newsletter_mobile_max']) ? esc_attr( $etNewsletter['newsletter_mobile_max'] ) : '';

		if ( $mobNoMin ) {
			if ( !($mobLen >= $mobNoMin) ) return 'min-error';
		}
		if ( $mobNoMax ) {
			if ( !($mobLen <= $mobNoMax) ) return 'max-error';
		}
		if ( $__Expt_BaseController->activated( 'et_menu', 'intl_tel_input_chk' ) ) {
			if ( !($dialcode) ) return 'invalid-mobile-number';
			$data["mobile"] = $data['dialcode'];
		}
		
		unset($data['dialcode']);

		if ( $sub == "subscribe" ) {

			$rowcount = $wpdb->get_var("SELECT COUNT(*) FROM  $table_name WHERE mobile = '{$data["mobile"]}'");

			if ( $rowcount == 0 ) {

				$result = $wpdb->insert( $table_name, $data );

				if(!$result) {

					if( $wpdb->last_error ) {
						$this->save_expt_wpdb_log();
					}
				}

				if ( $this->activated( 'et_menu', 'verify_subscriber' ) ) {

					update_option( "et_subscriber_data", $data, "yes" );

					$txt = "Your activation code is " . $data['activate_key'];

					$et_send_code = array(
						'et_send_from' => $this->apiSender,
						'et_send_to' => array( $data['mobile'] ),
						'et_send_msg' => $txt,
						'et_send_check_unicode' => 'false'
					);

					update_option( "et_send_sms", $et_send_code, "yes" );
					$res = $__Expt_ApiCallbacks->sendMessage();

					if( $res ) {

						$this->save_expt_log( $res, $et_send_code );
					}

					ob_clean();

					return 'activation-show';
				}

				$__Expt_NotificationsCallbacks->onUserSubscribesNewsletter( $data );

				ob_clean();

				return 'success';

			} else {

				if( $wpdb->last_error ) {
					$this->save_expt_wpdb_log();
				}

				ob_clean();

				return 'exist';
			}

		} else if ( $sub == "unsubscribe" ) {

			$rowcount = $wpdb->get_var("SELECT COUNT(*) FROM  $table_name WHERE mobile = '{$data["mobile"]}'");

			if ( $rowcount > 0 ) {

				$wpdb->show_errors();
				$result = $wpdb->delete( $table_name, array( 'mobile' => $data["mobile"] ) );

				if(!$result) {

					if( $wpdb->last_error ) {
						$this->save_expt_wpdb_log();
					}
				}

				ob_clean();

				return 'success-unsub';

			} else {

				if( $wpdb->last_error ) {
					$this->save_expt_wpdb_log();
				}

				ob_clean();

				return 'notfound';
			}

		}

		return false;

	}
}