<?php

/**
 * @package ExpertTextingOfficial
 */

namespace ExptTxt\Expt_Pages;

use \ExptTxt\Expt_Base\Expt_BaseController;
use \ExptTxt\Expt_Api\Expt_SettingsApi;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_AdminCallbacks;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_ManagerCallbacks;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_ApiCallbacks;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_NotificationsCallbacks;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_NewsletterCallbacks;
use \ExptTxt\Expt_Api\Expt_Callbacks\Expt_IntegrationCallbacks;

class Expt_Dashboard extends Expt_BaseController {

    public $settings;

    public $callbacks;
    public $callbacks_mngr;
    public $callbacks_api;
    public $pages = array();
    public $subpages = array();
    
    public function register() {

        $this->settings = new Expt_SettingsApi();
        
        $this->callbacks = new Expt_AdminCallbacks();
        $this->callbacks_mngr = new Expt_ManagerCallbacks();
        $this->callbacks_api = new Expt_ApiCallbacks();
        
        $this->setPages();
        $this->setSubPages();
        
        $this->setSettings();
        $this->setSections();
        $this->setFields();


        $this->settings->addPages( $this->pages )->withSubPage( 'Dashboard' )->addSubPages( $this->subpages )->register();

        add_action('admin_bar_menu', array($this, 'admin_bar_item_credit'), 500);
        add_action('admin_bar_menu', array($this, 'admin_bar_item'), 1);
        add_action('wp_dashboard_setup', array($this, 'et_add_dashboard_widgets'));
        
    }


    /**
     * Add a new dashboard widget.
     */
    public function et_add_dashboard_widgets() {
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }
        
        wp_add_dashboard_widget( 'dashboard_widget', ' ExpertTexting (v' . $this->plugin_version . ') at a Glance', array($this, 'et_dashboard_glance'), null, null, 'side', 'high' );
    }

    
    /**
     * Dashboard ET widget
     */
    public function et_dashboard_glance() {

        global $wpdb;
        $subscribe = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->prefix}et_subscribes");
        $group = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->prefix}et_subscribes_group");
        $credit = esc_attr( get_option('et_balance') );

        if(!$subscribe || !$group) {

            if( $wpdb->last_error ) {
                $this->save_expt_wpdb_log();
            }
        }

        _e( '<div class="bootstrap-et">
                <ul class="list-unstyled d-flex justify-content-between">
                    <li class="et-icon"><a href="' . admin_url('admin.php?page=et_subscribers') . '">' . sprintf(__('%s Subscriber(s)', 'experttexting-official'), $subscribe) . '</a></li>
                    <li class="et-icon"><a href="' . admin_url('admin.php?page=et_subscribers_group') . '">' . sprintf(__('%s Group(s)', 'experttexting-official'), $group) . '</a></li>
                    <li class="et-icon"><a href="' . admin_url('admin.php?page=et_settings&sub=api') . '">' . sprintf(__('$%s Credits', 'experttexting-official'), ( !empty($credit) ? $credit : 0 )) . '</a></li>
                </ul>
                <p class="m-0">' . __('Go to ExpertTexting Plugin', 'experttexting-official') . ' => <a class="button" href="' . admin_url('admin.php?page=et_menu') . '">' . __('Dashboard', 'experttexting-official') . '</a></p>
            </div>' );
    }


    /**
     * Credit in Admin bar
     */
    function admin_bar_item_credit ( $admin_bar ) {
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        if ( $this->activated( 'et_menu', 'show_balance_admin_menu' ) ) {

            $credit = esc_attr( get_option('et_balance') );
            $admin_bar->add_menu( array(
                'id' => 'et-credit',
                'parent' => null,
                'group' => null,
                'title' => '<span class="et-icon">' . ( !empty($credit) ? '$' . $credit : '$0' ) . '</span>',
                'href' => admin_url('admin.php?page=et_settings&sub=api'),
                'meta' => [
                    'title' => __( 'ExpertTexting Credits', 'experttexting-official' ),
                    ]
            ) );
        }
        
    }


    /**
     * Send SMS menu in Admin bar
     */
    function admin_bar_item ( $admin_bar ) {
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        $admin_bar->add_menu(array(
            'id'     => 'et-send-sms',
            'parent' => 'new-content',
            'title'  => __('ExpertTexting SMS', 'experttexting-official'),
            'href'   => admin_url('admin.php?page=et_send_sms')
        ));
    }


    public function setPages() {

        $this->pages = array(
            array(
                'page_title' => 'ExpertTexting Official',
                'menu_title' => 'ExpertTexting',
                'capability' => 'manage_options',
                'menu_slug' => 'et_menu',
                'callback' => array( $this->callbacks, 'adminDashboard' ),
                'icon_url' => $this->plugin_url . "assets/img/et_icon.png",
                'position' => 60
            )
        );
    }

    public function setSubPages() {

        $this->subpages = array( 
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Send SMS | ExpertTexting',
                'menu_title' => 'Send SMS',
                'capability' => 'manage_options',
                'menu_slug' => 'et_send_sms',
                'callback' => array( $this->callbacks, 'adminSendSms' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Outbox | ExpertTexting',
                'menu_title' => 'Outbox',
                'capability' => 'manage_options',
                'menu_slug' => 'et_subscribers_outbox',
                'callback' => array( $this->callbacks, 'adminOutbox' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Subscribers | ExpertTexting',
                'menu_title' => 'Subscribers',
                'capability' => 'manage_options',
                'menu_slug' => 'et_subscribers',
                'callback' => array( $this->callbacks, 'adminSubscribers' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Groups | ExpertTexting',
                'menu_title' => 'Groups',
                'capability' => 'manage_options',
                'menu_slug' => 'et_subscribers_group',
                'callback' => array( $this->callbacks, 'adminGroups' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Integrations | ExpertTexting',
                'menu_title' => 'Integrations',
                'capability' => 'manage_options',
                'menu_slug' => 'et_integration_settings',
                'callback' => array( $this->callbacks, 'adminIntegrations' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Settings | ExpertTexting',
                'menu_title' => 'Settings',
                'capability' => 'manage_options',
                'menu_slug' => 'et_settings',
                'callback' => array( $this->callbacks, 'adminSettings' ),
            ),
            array(
                'parent_slug' => 'et_menu',
                'page_title' => 'Hooks & Functions | ExpertTexting',
                'menu_title' => 'Hooks & Functions <br> (For Developers)',
                'capability' => 'manage_options',
                'menu_slug' => 'et_hooks_n_functions',
                'callback' => array( $this->callbacks, 'adminHooksNFunctions' ),
            ),
        );

        /**
         * Add new sub menu
         *
         * @param array empty
         * @since 1.2.0
         */
        $add_filter_subpages = apply_filters( 'expttxt_main_menu', array() );
        if( is_array($add_filter_subpages) ) {
            foreach( $add_filter_subpages as $filtered_subpages ) {
                if( is_array($filtered_subpages) ) {
                    $this->subpages[] = array_merge( $this->subpages, $filtered_subpages );
                } else {
                    $this->save_expt_log( ['Filter (expttxt_main_menu) Error', 'Array not given in filter => expttxt_main_menu'], $add_filter_subpages );
                }
            }
        } else {
            $this->save_expt_log( ['Filter (expttxt_main_menu) Error', 'Array not given in filter => expttxt_main_menu'], [$add_filter_subpages] );
        }
    }

    public function setSettings() {

        $args = array(
            array(
                'option_group' => 'et_plugin_settings',
                'option_name' => 'et_menu',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_api_settings',
                'option_name' => 'et_api_set',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_api_settings',
                'option_name' => 'etApiAuthHidden',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_sendSms_settings',
                'option_name' => 'et_send_sms',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_notifications_settings',
                'option_name' => 'et_notifications',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_general_settings',
                'option_name' => 'et_general_set',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
            array(
                'option_group' => 'et_plugin_newsletter_settings',
                'option_name' => 'et_newsletter',
                'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
            ),
        );

        $args_integration = array();
        
        foreach( $this->integrate_arr as $key => $value ) {

            $arr_int = array(
                array(
                    'option_group' => 'et_plugin_'.$key.'_settings',
                    'option_name' => "et_$key",
                    'callback' => array( $this->callbacks_mngr, 'etSanitizeSetGroup' )
                )
            );

            $args_integration = array_merge($args_integration, $arr_int);
        }

        $args = array_merge($args, $args_integration);

        $this->settings->setSettings( $args );
    }

    public function setSections() {

        $args = array(
            array(
                'id' => 'et_admin_index',
                'title' => 'Manage Features',
                'callback' => array( $this->callbacks_mngr, 'etAdminSectionManager' ),
                'page' => 'et_menu'
            ),
            array(
                'id' => 'et_admin_api_index',
                'title' => 'API Configuration',
                'callback' => array( $this->callbacks_mngr, 'etApiSetManager' ),
                'page' => 'et_api_settings'
            ),
            array(
                'id' => 'et_admin_sendSms_index',
                'title' => 'Send SMS',
                'callback' => array( $this->callbacks_mngr, 'etSetSMSManager' ),
                'page' => 'et_sendSms_form'
            ),
            array(
                'id' => 'et_admin_notifications_index',
                'title' => 'Manage Notifications',
                'callback' => array( $this->callbacks_mngr, 'etNotificationsSetManager' ),
                'page' => 'et_notifications_settings'
            ),
            array(
                'id' => 'et_admin_general_index',
                'title' => 'Manage General Settings',
                'callback' => array( $this->callbacks_mngr, 'etGeneralSetManager' ),
                'page' => 'et_general_settings'
            ),
            array(
                'id' => 'et_admin_newsletter_index',
                'title' => 'Manage Newsletter Settings',
                'callback' => array( $this->callbacks_mngr, 'etNewsletterSetManager' ),
                'page' => 'et_newsletter_settings'
            )
        );

        $args_integration = array();
        
        foreach( $this->integrate_arr as $key => $value ) {

            $arr_int = array(
                array(
                    'id' => 'et_admin_'.$key.'_index',
                    'title' => $value,
                    'callback' => array( $this->callbacks_mngr, 'etIntegrationSetManager' ),
                    'page' => 'et_'.$key.'_settings'
                )
            );

            $args_integration = array_merge($args_integration, $arr_int);
            
        }

        $args = array_merge($args, $args_integration);

        $this->settings->setSections( $args );
    }
    
    public function setFields() {

        $args = array();

        // Settings
        foreach ($this->managers as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $selectOptions = !empty($value['selectOptions']) ? $value['selectOptions'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback , $callbackFn ),
                'page' => 'et_menu',
                'section' => 'et_admin_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'select_option' => $selectOptions,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_Settings'
                    )
                );
        }

        // API Settings Fields
        foreach ($this->managers_API as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback , $callbackFn ),
                'page' => 'et_api_settings',
                'section' => 'et_admin_api_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_API'
                    )
                );
        }

        // Send SMS Fields
        foreach ($this->managers_SendSMS as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $selectOptions = !empty($value['selectOptions']) ? $value['selectOptions'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback , $callbackFn ),
                'page' => 'et_sendSms_form',
                'section' => 'et_admin_sendSms_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'select_option' => $selectOptions,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_SendSMS'
                    )
                );
        }

        // Notification Settings
        foreach ($this->managers_Notifications as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $selectOptions = !empty($value['selectOptions']) ? $value['selectOptions'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback, $callbackFn ),
                'page' => 'et_notifications_settings',
                'section' => 'et_admin_notifications_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'select_option' => $selectOptions,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_Notifications'
                    )
                );
        }

        // General Settings Fields
        foreach ($this->managers_General as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback , $callbackFn ),
                'page' => 'et_general_settings',
                'section' => 'et_admin_general_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_General'
                    )
                );
        }

        // Newsletter Settings
        foreach ($this->managers_Newsletter as $key => $value) {

            $callback = !empty($value['callback']) ? $value['callback'] : '';
            $title = !empty($value['title']) ? $value['title'] : '';
            $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
            $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
            $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
            $subText = !empty($value['subText']) ? $value['subText'] : '';
            $selectOptions = !empty($value['selectOptions']) ? $value['selectOptions'] : '';
            $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';

            $args[] = array(
                'id' => $key, // Settings Group's Option Name
                'title' => $title,
                'callback' => array( $this->$callback, $callbackFn ),
                'page' => 'et_newsletter_settings',
                'section' => 'et_admin_newsletter_index',
                'args' => array(
                    'option_name' => $optionName,
                    'label_for' => $key,
                    'additionalArgs' => $additionalArgs,
                    'subText' => $subText,
                    'select_option' => $selectOptions,
                    'isHtmlSupported' => $isHtmlSupported,
                    'sectionIs' => 'managers_Newsletter'
                    )
                );
        }

        // Integration Settings
        foreach ($this->managers_Integration as $plugin_name => $value_arr) {

            $page = isset( $_GET['page'] ) ? sanitize_text_field( $_GET['page'] ) : '';
            $sub = ( $page === 'et_integration_settings' && isset( $_GET['sub'] ) ) ? sanitize_text_field( $_GET['sub'] ) : 'integration_cf7';
            $sub_page = !empty( $sub ) ? ( !empty( $this->integrate_arr[$sub] ) ? $this->integrate_arr[$sub] : false ) : false;

            if( $page === 'et_integration_settings' && !$sub_page ) {
                header('location: ' . admin_url("admin.php?page=et_integration_settings"));
            }

            $callback = $title = $callbackFn = $optionName = $additionalArgs = $subText = $selectOptions = '';
            $check_plugin_installation = \ExptTxt\Expt_Api\Expt_SettingsApi::is_plugin_installed( $plugin_name );

            if( $sub_page && $sub_page === $plugin_name ) {
                foreach ($value_arr as $key => $value) {

                    if( $check_plugin_installation === 'installed' ) {
                    
                        $callback = !empty($value['callback']) ? $value['callback'] : '';
                        $title = !empty($value['title']) ? $value['title'] : '';
                        $callbackFn = !empty($value['callbackFn']) ? $value['callbackFn'] : '';
                        $optionName = !empty($value['optionName']) ? $value['optionName'] : '';
                        $additionalArgs = !empty($value['additionalArgs']) ? $value['additionalArgs'] : '';
                        $subText = !empty($value['subText']) ? $value['subText'] : '';
                        $selectOptions = !empty($value['selectOptions']) ? $value['selectOptions'] : '';
                        $isHtmlSupported = !empty($value['isHtmlSupported']) ? $value['isHtmlSupported'] : '';
                        
                    } else {

                        $key = $plugin_name . '_msg';
                        $callback = 'callbacks_mngr';
                        $title = $check_plugin_installation === 'inactive-plugin' ? '<h6 class="text-uppercase">'. __( 'Plugin not activated', 'experttexting-official' ) .'</h6>' : '<h6 class="text-uppercase">'. __( 'Plugin not installed', 'experttexting-official' ) .'</h6>';
                        $callbackFn = 'noFieldOnlyText';
                        $subText = $check_plugin_installation === 'inactive-plugin' ? "<span class='fw-normal text-muted'>". __( "The \"$plugin_name\" plugin is not activated. Activate the plugin to use this integration.", 'experttexting-official' ) ."</span>" : "<span class='fw-normal text-muted'>". __( "The \"$plugin_name\" plugin is not installed. Install the plugin to use this integration.", 'experttexting-official' ) ."</span>";
                    }

                    $args[] = array(
                        'id' => $key, // Settings Group's Option Name
                        'title' => $title,
                        'callback' => array( $this->$callback, $callbackFn ),
                        'page' => 'et_'.$sub.'_settings',
                        'section' => 'et_admin_'.$sub.'_index',
                        'args' => array(
                            'option_name' => $optionName,
                            'label_for' => $key,
                            'additionalArgs' => $additionalArgs,
                            'subText' => $subText,
                            'select_option' => $selectOptions,
                            'isHtmlSupported' => $isHtmlSupported,
                            'sectionIs' => 'managers_integration'
                            )
                        );
                    
                }
            }

        }

        $this->settings->setFields( $args );
    }

}